% median deviation over sigma accumulator (MDA)
function [mask, accmat] = get_roa_mask(stack, cycle, channel, baseline_frames, blank_frames, treshold, accum_frames)
    
    if nargin < 7
        accum_frames = 5;
    end
    if nargin < 6
        treshold = 10;
    end
    if nargin < 5
        blank_frames = [];
    end
    if nargin < 4
        baseline_frames = 1:stack.frames_in_cycle;
    end

    %stack = begonia.scantype.find_scans(PATH);
    mat = stack.get_mat(cycle, channel);
    mat = single(mat(:,:,:));
    
    % remove blanking regions:
    if ~isempty(blank_frames)
        remreg = mat(:,:,blank_frames);
        mat(:,:,blank_frames) = zeros(size(remreg));
    end
    
    % smooth in 3D
    mat = smooth3(mat);

    % correct drift:
%     if ~stack.has_var('channel_trace')
%         begonia.stack_modules.channel_traces.write_vars(stack);
%     end
%     drift_corr = begonia.stack_modules.drift_correction.read(stack);
%     %mat_dc = mat .* drift_corr;
% 

    % get median image:
    mat_median = median(mat(:,:,baseline_frames), 3);
    mat_std = std(mat(:,:,baseline_frames), 0, 3);

    % get median deviation stack:
    mat_mdos = (mat - mat_median) ./ mat_std;
    
    % accumulate median values:
    accmat = running_accumulator(mat_mdos, accum_frames);
    mask = accmat > treshold;
end

function acc = running_accumulator(mat, start)
    [w, h, l] = size(mat);
    acc = zeros(w,h,l - start);
    for f = start:l
        acc(:,:,f) = sum(mat(:,:,(f-(start-1)):f), 3);
    end
end