% median deviation over sigma accumulator (MDA)
function [mask, mat_mdev] = get_roa_mask(stack, cycle, channel, baseline_frames, blank_frames)
    
    if nargin < 5
        blank_frames = [];
    end
    
    if nargin < 4
        baseline_frames = 1:stack.frames_in_cycle;
    end
    
    % get matrix
    mat = stack.get_mat(cycle,channel);
    mat = double(mat(:,:,:));
    
    % remove blank frames:
    if ~isempty(blank_frames)
        rem = mat(:,:,blank_frames);
        mat(:,:,blank_frames) = zeros(size(rem));
    end

    % gaussean filtering in time and space 1px:
    sigma_dx = 1;
    sigma_dt = 0.1/stack.dt;

    win_dx = begonia.util.gausswin(sigma_dx);
    win_dt = begonia.util.gausswin(sigma_dt);
    win_dt = reshape(win_dt,1,1,[]);

    filter = win_dx*win_dx';
    filter = repmat(filter,1,1,length(win_dt));
    filter = filter.*win_dt;
    filter = filter/sum(filter(:));
    filter = single(filter);

    mat = convn(mat,filter,'same');

    % get measures
    mat_median = median(mat(:,:,baseline_frames), 3);
    mat_sd = std(mat(:,:,baseline_frames), 0, 3);
    
    % pixels deviating > 2 sd from median:
    mat_mdev = zeros(size(mat));
    mat_2sd = zeros(size(mat));
    for t = 1:size(mat,3)
        mat_mdev(:,:,t) = mat(:,:,t) - mat_median;
        mat_2sd(:,:,t) = abs(mat_mdev(:,:,t)) > 2 * mat_sd;
    end

    % pixels in which at least two most recent have been > 2 std:
    mat_2sdsum = zeros(size(mat));
    for t = 2:size(mat,3) - 1
        mat_2sdsum(:,:,t) = sum(mat_2sd(:,:,t-1:t+1), 3);
    end

    mask = mat_2sdsum >= 3;
end
