classdef RoAMarkingReader < handle
    %ROAMARKER Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        inner_reader
        roa_array
    end
    
    methods
        function obj = RoAMarkingReader(reader, roa_array)
            obj.inner_reader = reader;
            obj.roa_array = roa_array;
        end
        
         % override for builtin size function:
        function n = size(obj, d)
            size_vec = size(obj.inner_reader);
            
            if nargin < 2
               n = size_vec;
            else
                n = size_vec(d);
            end
        end
        
        % override for allow using the reader with indexes:
        function result = subsref(obj, indicies)
            if(~strcmp(indicies(1).type,'()'))
                % we only want to deal with indexing, and pass other stuff
                % onto the builtin subsref function. This prevents this
                % modification from interfering with e.g. dot-notation.
                result = builtin('subsref', obj, indicies);
            else
                result = uint16([]);
                if length(indicies.subs) < 3
                    error('Reader requires three indexes')
                end
                
                % grab frames and extract subarea:
                xr = indicies.subs{1};
                yr = indicies.subs{2};
                zr = indicies.subs{3};
                
                % extract full matrix:
                mat = obj.inner_reader(xr, yr, zr);
                mat_max = max(mat(:));
                
                % find roas for this region:
                starts = arrayfun(@(r) r.t_idx(1) < zr, obj.roa_array);
                ends = arrayfun(@(r) r.t_idx(end) > zr, obj.roa_array);
                roas = obj.roa_array(starts & ends);
                for roa = roas
                    
                    x_min = min(roa.x_idx);
                    x_max = max(roa.x_idx);
                    y_min = min(roa.y_idx);
                    y_max = max(roa.y_idx);
                    
                    mat(y_min, x_min:x_max) = mat_max;
                    mat(y_max, x_min:x_max) = mat_max;
                    mat(y_min:y_max, x_min) = mat_max;
                    mat(y_min:y_max, x_max) = mat_max;
                end
                
                
                result = mat;
                
            end
        end
        
    end
end


function img = drawrectangle(img, xbounds, ybounds)
    img(ybounds, xbounds(1):xbounds(2)) = 1;
    img(ybounds(1):ybounds(2), xbounds) = 1;
end