function results = extract_roas(tss, cy, ch, crits, method, baseline_frames, blank_region, divs, use_gpu)
    import begonia.analysis.roa.*;
    
    % default: no gpu, 2 divs, first channel and cycle:

    if nargin < 9
        use_gpu = false;
    end
    if nargin < 8
        divs = 2;
    end
    if nargin < 7
        blank_region = [];
    end
    if nargin < 5
        method = 'f0';
    end
    if nargin < 4
        crits = RoACriteriaDefaults.ROAS_3um2;
    end
    if nargin < 3
        ch = 1;
    end
    if nargin < 2
        cy = 1;
    end
    
    % perform extraction on all TSeries given:
    results = RoACollection.empty;
    for ts = tss
        
        % if not otherise specified, use all frames for f0 image:
        if nargin < 6
            baseline_frames = 1:ts.frames_in_cycle;
        end

        % matrix, and perform calculations:
        ref_img = ts.get_avg_img(cy, ch);
        traces = begonia.stack_modules.channel_traces.read(ts);
        dcorr = begonia.stack_modules.drift_correction.read(ts, false, traces);
        
        % switch method for RoA calculation:
        if method == "f0"
            roa_mask = begonia.stack_modules.roa_mask.read(ts, cy, ch, 2, false, dcorr, blank_region);
        elseif method == "mdosa"
            roa_mask = begonia.analysis.roa.mdosa.get_roa_mask(ts, cy, ch, baseline_frames, blank_region);
        elseif method == "pof"
            roa_mask = begonia.analysis.roa.pof.get_roa_mask(ts, cy, ch, baseline_frames, blank_region);
        elseif method == "improved"
            conf = begonia.stack_modules.roa_mask.original.roa_config;
            conf.channel = ch;
            conf.cycle = cy;
            roa_mask = begonia.stack_modules.roa_mask.original.read(ts, conf);
        else
            error('Unknown mask generation method. Valid options: "f0", "mdosa"');
        end


        % extract RoAs and ignore mask:
        %img_f0 = begonia.stack_modules.f0_smooth.read_lowmem(ts, cy, ch, divs, use_gpu, baseline_frames);
        img_f0 = begonia.stack_modules.f0_smooth.read(ts, cy, ch);
        [roa_array, ignore_mask] = begonia.stack_modules.roa.read(... 
            ts, ...
            img_f0, ...
            roa_mask, ...
            100, ...
            crits.min_size_um2, ...
            crits.min_duration_s);

        % extract active pixel count:
        active_area = get_active_area(roa_mask, ignore_mask, ts.dx, ts.dt);
        
        % keep result in a collection:
        result = RoACollection(ts, cy, ch);

        result.img_ref = ref_img;
        result.channel_trace = traces;
        result.drift_correction = dcorr;
        result.roa_mask = roa_mask;
        result.active_area = active_area;
        result.img_f0 = img_f0;
        result.roas = roa_array;
        result.roa_ignore_mask = ignore_mask;
        result.baseline_frames = baseline_frames;
        
        
        % set roas to empty is nothing there:
        if ~isfield(result.roas, 'area')
            result.roas = [];
        end
    
        results(end + 1) = result;  %#ok<AGROW>
    end
end

