%#ok<*AGROW>
function data = make_roa_table(roa_collections)
    import begonia.util.logging.*;
    
    % set up vars for table:
    row = 1;
    ts_uuid = categorical();
    ts_name = categorical();
    ts_cy = [];
    ts_ch = [];
    roa_size = zeros(0,1);
    roa_center = zeros(0,2);
    roa_start = [];
    roa_end = [];
    roa_peak = [];
    roa_duration = zeros(0,1);
    roa_fov_size = zeros(0,1);
    roa_group = categorical();
    roa_panac_assoc = categorical();
    roa_activity_group = categorical();
    
    for col = roa_collections
        vlog(1,['[make_roa_table] Processing ' char(col.ts_name)]);

        % calculate FOV size:
        fov_size = sum(col.roa_ignore_mask(:)) * col.ts_dx * col.ts_dx;

        % add roas to table:
        for idx_roa = 1:length(col.roas)
            roa = col.roas(idx_roa);
            
            if ~isfield(roa, 'area')
                warning('No RaA area - skipping')
                continue;
            end

            ts_uuid(row,1) = col.ts_uuid;
            ts_name(row,1) = col.ts_name;
            ts_cy(row,1) = col.ts_cy;
            ts_ch(row,1) = col.ts_ch;
            
            roa_size(row,1) = roa.area;
            roa_center(row,1:2) = roa.center;
            roa_duration(row,1) = roa.t_end - roa.t_start;
            roa_start(row,1) = roa.t_start; 
            roa_end(row,1) = roa.t_end;
            roa_peak(row,1) = roa.t_peak;
            roa_fov_size(row,1) = fov_size;

            row = row + 1;
        end
    end
    
    % create table:
    data = table(ts_uuid, ts_name, ts_cy, ts_ch, roa_size, roa_duration, roa_center,roa_start,roa_end,roa_peak,roa_fov_size);

end

