function plot_roa_overview(colls)
    for coll = colls
        plot_inner(coll);
    end
end

function plot_inner(roas)
    fig = figure;
    fig.Position(4) = fig.Position(4)*2;
    fig.Position(2) = 0;
    fig.Color = 'w';
    
    c = 2;
    r = 4;
    
    % reference image:
    subplot(r,c,1);
    imagesc(roas.img_ref)
    cystr = num2str(roas.ts_cy);
    chstr = num2str(roas.ts_ch);
    titstr = [roas.ts_name ' : Cy ' cystr ' / Ch ' chstr];
    title(titstr);
    fig.Name = titstr;
    fig.ToolBar = 'none';
   
    
    % mask sum:
    subplot(r,c,2);
    mask_sum = sum(roas.roa_mask, 3);
    imagesc(mask_sum);
    title('Pixel activity');
    colorbar;
    
    % f0 image
    subplot(r,c,3);
    imagesc(roas.img_f0);
    fblf = num2str(roas.baseline_frames(1));
    lblf = num2str(roas.baseline_frames(end));
    tf = num2str(roas.ts_frames);
    title(['f_0 image (base: ' fblf ':' lblf ' of ' tf ')']);
    
    % ignore mask
    subplot(r,c,4);
    imagesc(roas.roa_ignore_mask);
    title('Ignore mask');
    

    % counts over time:
    subplot(r,c,5:6);
    if ~isempty(roas.roas)
        counter = zeros(1, roas.ts_frames);
        total_area = zeros(1, roas.ts_frames);
        
        t = (1:roas.ts_frames) * roas.ts_dt;
        for roa = roas.roas
            s = floor(roa.t_start / roas.ts_dt);
            d = floor((roa.t_end - roa.t_start) / roas.ts_dt);
            counter(s:s+d) = counter(s:s+d) + 1;
            total_area(s:s+d) = total_area(s:s+d) + roa.area;
        end
        plot(t, smooth(counter,15), 'Color',[0,0,0]);
        title(['RoA count over time (n=' num2str(roas.count) ')'])
        ylabel('Active RoAs (#)');
        xlabel('Time (S)')
        xlim([0 roas.ts_frames * roas.ts_dt]);

        yyaxis right
        plot(t, smooth(total_area,15));
        ylabel('RoA area (um^2)');
    else
        title('(No RoAs to plot)')
    end
    
    % mask active pixels over time:
    subplot(r,c,7:8);

    plot(roas.active_area);  % um^2
    xlim([0 roas.ts_frames * roas.ts_dt]);

end

