function coll = extract_roi_signals(ts, extract_doughnuts)
    import begonia.analysis.roi.*;

    % create a collection to keep the RoI data:
    coll = RoiSignalCollection(ts);
    
    if ~ts.has_var('roi_array')
        error('Provided TSeries does not have a roi_array variable - not marked in RoiManager?');
    end
    
    coll.roi_array = ts.load_var('roi_array');
    
    coll.channel_trace = begonia.stack_modules.channel_traces.read(ts);
    ts.save_var('channel_traces', coll.channel_trace);
    
    coll.drift_correction = begonia.stack_modules.drift_correction.read(ts);
    ts.save_var('drift_correction_trace', coll.drift_correction);
    
    coll.raw_signals = calendula.stack_modules.ca_signal.read(ts);
    ts.save_var('ca_signal', coll.raw_signals)
    
    coll.dff_signals = calendula.stack_modules.ca_signal_df_f0.read(ts);
    ts.save_var('ca_signal_df_f0', coll.dff_signals);
    
    % read neuron doughnut RoI signals:
    if extract_doughnuts
        disp("Doughnut extraction");
        [drois, dsigs, douter] = calendula.stack_modules.roi_doughnuts.read(ts);

        coll.neu_doughnut_rois = drois;
        coll.neu_doughnut_raw_signals = dsigs;
        coll.neu_doughnut_outer = douter;

        ts.save_var('roi_doughnuts', drois);
        ts.save_var('ca_signal_doughnuts', dsigs);
        
        % process the raw dounought signal:
        dsigs.Data = calendula.stack_modules.ca_signal_df_f0.filter_signal_neurons(dsigs.Data,ts.dt);
        coll.neu_doughnut_dff_signals = dsigs;
        
        % do the doughnut subtraction:
        disp('Subtracting doughnut signal');
        dsig_sub = calendula.stack_modules.ca_signal_neurons_subtracted.read(ts);
        coll.dff_signals_doughnut_subtracted = dsig_sub;
        ts.save_var('ca_signal_neurons_subtracted', dsig_sub);
    end
end

