function fig = plot_3dspin(mat, title_str, spintime_s, fps, spin_deg, dt, vid_path)
    if nargin < 7
        vid_path = [];
    end
    if nargin < 6
        dt = [];
    end

    fig = figure();
    fig.Color = 'white';
    s = surf(mat(:,:,10));
    s.EdgeColor = 'none';

    stack_frames = size(mat, 3); 

    % time calculatioins
%     spintime_s = 5; % 
%     fps = 30;
%     spin_deg = 360; % degrees
    deg_pr_incr = spin_deg / (fps * spintime_s); % deg/f
    frames_tot = fps * spintime_s;  % # of frames total
    incr_delay = fps/1000;  % delay seconds between each frame

    % calculate zscale max/min:
    ma = max(mat(:));
    mi = min(mat(:));
    zlim([mi ma]);

    % resize window for better resolution:
    fig.Position(3) = fig.Position(3) * 2;
    fig.Position(4) = fig.Position(4) * 2;

    fig.Position(1) = 0;
    fig.Position(2) = 0;

    fnr = 1;

    an = annotation('textbox',[0 0 1 1],'String','','FitBoxToText','on');
    an.FontSize = 20;
    an.LineStyle = 'none';
    an.Color = [0.4 0.4 0.4];

    % record?
    if ~isempty(vid_path)
        video = VideoWriter(vid_path, 'MPEG-4');
        video.FrameRate = fps;
        open(video);
        record = true;
    else
        record = false;
    end
    
    pause on;
    for i = 1:frames_tot
        if ~isvalid(fig)
            pause off;
            if record
                close(video);
            end
            break;
        end

        s.ZData = mat(:,:,round(fnr));
    %    view(i, 45 + sin(deg2rad(i)) * 90);
        view(i * deg_pr_incr, 60);

        % make annotation string:

        fnr_str = num2str(ceil(fnr));
        ftot_str = num2str(stack_frames);
        sec_str = num2str(fnr * dt, '%4.1f');

        an_str = [title_str newline fnr_str ', ' sec_str ' sec'];

        an.String = an_str;


        % increment and delay:
        fnr = fnr + stack_frames/frames_tot;
        
        % write video:
        if record
            writeVideo(video, getframe(fig));
        end
        
        pause(incr_delay);
    end
    
    pause off;
    if record
        close(video);
    end
end

