classdef Experiment < begonia.data_management.DataLocation
    
    properties
        links
    end
    
    methods
        function self = Experiment(path)
            self@begonia.data_management.DataLocation(path);
            
            links_file = fullfile(path,'links.mat');
            assert(isfile(links_file), ...
                'begonia:load:missing_file', ...
                'Input path does not contain the ''links.mat'' file.');
            var = load(links_file);
            self.links = var.links;
        end
    end
    
    methods (Static)
        function exps = find_experiments(path,constructor)
            % Scan path and all subfolders for valid Experiments. 
            %
            %   path        -   (char) Path to a directory where
            %                   Experiments are located. 
            %   constructor -   (function_handle,optional) Default returns
            %                   an object of this Experiment class. 
            if nargin < 2
                constructor = @begonia.data_management.Experiment;
            end
            if isa(constructor,'function_handle')
                constructor = {constructor};
            end
            
            begonia.util.logging.vlog(1,'Discovering directories');
            all_dirs = begonia.path.list_dirs(path);
            
            % Ignore dirs where the name of the last folder is 'metadata'.
            last_dir = [filesep,'metadata'];
            I = cellfun(@(x)strcmp(x(end-length(last_dir)+1:end),last_dir),all_dirs);
            all_dirs(I) = [];
            
            exps = begonia.path.init_classes_from_paths(all_dirs,constructor);
        end
        
        function exp = new(save_dir,uuid,constructor)
            % Create a new Experiment data location. If uuid is not 
            % supplied a generic one is created. 
            %
            %   save_dir    -   (char) Directory which the new Experiment 
            %                   will be created inside. 
            %   uuid        -   (char, optional) Unique ID of the
            %                   Experiment which is also used for the
            %                   folder name. 
            %   constructor -   (function_handle,optional) Default returns
            %                   an object of this Experiment class. 
            if nargin < 2
                % Engage. 
                uuid = begonia.util.make_snowflake_id('USS');
            end
            if nargin < 3
                constructor = @begonia.data_management.Experiment;
            end
            path = fullfile(save_dir,uuid);
            
            if exist(path,'dir')
                exp = constructor(path);
                return;
            end
            
            begonia.path.make_dirs(path);
            links = containers.Map;
            links_filename = fullfile(path,'links.mat');
            save(links_filename,'links');
            
            exp = constructor(path);
            exp.dl_unique_id = uuid;
        end
    end
end

