classdef TSeriesExperiment < begonia.data_management.Experiment
    
    properties
        trial
        tseries
    end
    
    methods (Static)
        
        function exps = find_experiments(path,constructor)
            % Scan path and all subfolders for valid TSeriesExperiments. 
            %
            %   path        -   (char) Path to a directory where
            %                   Experiments are located. 
            %   constructor -   (function_handle,optional) Default returns
            %                   an object of this TSeriesExperiment class. 
            if nargin < 2
                constructor = @begonia.experiment_templates.TSeriesExperiment;
            end
            exps = begonia.data_management.Experiment.find_experiments(path,constructor);
        end
        
        function exp = new_from_trials(save_dir,trials,tseries)
            % Creates new experiments with a link to each 'trial' and
            % 'tseries'. The length of 'trials' and 'tseries' must be the
            % same. Will not overwrite if the experiment already exists. 
            %
            %   save_dir    -   (char) Directory which the new Experiments 
            %                   will be created inside. 
            %   trials      -   (child of begonia.data_management.DataInfo)
            %                   List of DataInfo objects. 
            %   tseries     -   (child of begonia.scantype.TSeries). 
            %                   List of TSeries objects. 
            
            % Make sure both inputs have time info. 
            assert(isa(trials,'begonia.data_management.DataInfo'));
            assert(isa(tseries,'begonia.scantype.TSeries'));
            assert(length(trials) == length(tseries));
            
            begonia.util.logging.backwrite();
            for i = 1:length(trials)
                begonia.util.logging.backwrite(1,'Constructing TSeriesExperiment (%d/%d)',i,length(trials));
                tr = trials(i);
                ts = tseries(i);
                
                % Assume the start time in seconds of the trial can be used
                % as a uniq ID. This is only to have human readable unique
                % ID. 
                tr.start_time_abs.Format = 'uuuu MM dd HH mm ss';
                uuid = char(tr.start_time_abs); 
                
                % Return a TSeriesExperiment instead of Experiment.
                constructor = @begonia.experiment_templates.TSeriesExperiment;
                
                exp(i) = begonia.data_management.Experiment.new(save_dir,uuid,constructor);
                exp(i).links('tseries') = ts.dl_unique_id;
                exp(i).links('trial') = tr.dl_unique_id;
                exp(i).tseries = ts;
                exp(i).trial = tr;
            end
            
        end
    end
end

