function [ca_events, ca_events_extended_data] = roi_event_roa_components(stack, PLOT_DIR)

    %% load data needed:

    
    disp('Loading base data')
    %rois = stack.load_var('rois');s
    roi_array =  stack.load_var('roi_array');
    %roa_array = stack.load_var('roa_array');
    roa_mask = stack.load_var('roa_mask');
    ca_signal_df_f0 = stack.load_var('ca_signal_df_f0');
    mat_ch2 = stack.get_mat(1,1);

    %% 1) extract calsium events if needed:
    if ~stack.has_var('ca_events')
        roi_cnt = length(roi_array);
        for i = 1:roi_cnt
            disp("RoI event extraction " + i + "/" + roi_cnt)
            trace = ca_signal_df_f0.Data(:,i);
            params = alyssum.peaks.highpass.default_parameters();
            peaks = alyssum.peaks.highpass.find_peaks_ast(trace, 30, params);

            for j = 1:length(peaks)
                peaks(j).roi_num = i;
            end

            if i == 1
                ca_events = peaks;
            else
                ca_events = [ca_events peaks];
            end
        end

        stack.save_var(ca_events);
    else
        disp('loading existing ca_events');
        ca_events = stack.load_var('ca_events');
    end

    %% 2) look for connected components in the submasks:
    ev_cnt = length(ca_events);
    if ~stack.has_var('ca_events_extended_data')
        for i = 1:ev_cnt
        %for i = ev_cnt-40:ev_cnt  
            disp("Event submask components " + i + "/" + ev_cnt)
            ev = ca_events(i);
            roi = roi_array(ev.roi_num);
            bounds = roi.boundary{:};
            x = min(bounds(:,2));
            x_max = max(bounds(:,2)); 
            y = min(bounds(:,1));
            y_max = max(bounds(:,1));

            % get local mask:
            mask = roa_mask(:,:,ev.x_start_idx:ev.x_end_idx) .* roi.mask;
            submask = mask(y:y_max, x:x_max,:);

            % get roi pixels:
            submat = mat_ch2(:,:,ev.x_start_idx:ev.x_end_idx);
            submat = submat .* uint16(roi.mask);
            submat = submat(y:y_max, x:x_max,:);

            % grab connected components from submask + calc heatmap:
            % keep only comps with a volume of 10 pixels:
            cc = bwconncomp(submask,6);
            comps = [cc.PixelIdxList];
            comps_lengths = arrayfun(@(c) length(c{:}),comps);
            comps = comps(comps_lengths > 10);
            ca_events_extended_data(i).n_peaks = ev.n_peaks;
            ca_events_extended_data(i).comps_bt10 = comps;
            ca_events_extended_data(i).comps_bt10_count = length(comps);
            ca_events_extended_data(i).mask_heatmap = sum(submask,3);
            ca_events_extended_data(i).mask = submask;
            ca_events_extended_data(i).mat = submat;

            exp_tab(i).ts_name = string(stack.name);
            exp_tab(i).roi_nr = ev.roi_num;
            exp_tab(i).event_nr = i;
            exp_tab(i).n_peaks = ev.n_peaks;
            exp_tab(i).comps_bt10_count = length(comps);
        end
        stack.save_var('ca_events_extended_data')
        tab = struct2table(exp_tab);
        writetable(tab, fullfile(PLOT_DIR, stack.name + "_ca_events_extended_data.xlsx"));
    else
        disp('loading existing ca_events_extended_data');
        ca_events_extended_data = stack.load_var('ca_events_extended_data');
    end

    % plot that:
    fig = figure(); fig.Color = 'white';
    scatter([ca_events_extended_data.n_peaks], [ca_events_extended_data.comps_bt10_count], 'filled');
    xlabel('RoI event peaks (#)');
    ylabel('RoA components > 10 pix volume (#)');

    xticks(0:max([ca_events_extended_data.n_peaks]) + 1)
    xlim([.5 max([ca_events_extended_data.n_peaks]) + 1.5])
    title('Peak vs component count');
    savefig(fig, fullfile(PLOT_DIR, stack.name + "_peaks_vs_comps.fig"));

    %% 3) extract roi traces from the events:
    offset_multiples = 1.5;
    y_offset = 0;
    fig = figure('Position', [100,100, 1000,1000]);
    fig.Color = 'white';
    dffs = ca_signal_df_f0.Data;
    t = ca_signal_df_f0.Time;
    for i = 1:ev_cnt
        ev = ca_events(i);
        ev_exd = ca_events_extended_data(i);

        % only work with multipeak events:
        if ev.n_peaks < 2; continue; end
        y_offset = y_offset + 1;

        % get the corresponding calsium signal segment:
        sig_full = dffs(:,ev.roi_num);
        sig_full_sm = smooth(sig_full);
        sig_seg = sig_full_sm(ev.x_start_idx:ev.x_end_idx);
        sig_seg_t = t(ev.x_start_idx:ev.x_end_idx);

        hmap(y_offset, 1:length(sig_seg)) = sig_seg;
        hold on;

        % plot each component:
        max_mask = size(ev_exd.mask, 1) *  size(ev_exd.mask, 2);
        for comp = ev_exd.comps_bt10
            comp_mask = false(size(ev_exd.mat));
            comp_mask(comp{:}) = 1;
            %comp_mat = double(ev_exd.mat) .* comp_mask;
            trace = knutil.mat_to_sum_trace(comp_mask);
            trace_norm = knutil.normalize_mat(trace, max_mask) * 0.5;
            %area(sig_seg_t, trace_norm + (y_offset * 1.5), 'BaseValue', y_offset * 1.5 , 'FaceColor', [rand rand rand]);
            trace_norm(1) = 0;
            trace_norm(end) = 0;
            fill(sig_seg_t, trace_norm + (y_offset * 1.5), [rand rand 1]);

            hold on;
        end

        % plot RoI trace:
        plot(t, sig_full_sm + (y_offset * 1.5), 'Color', [.7 .7 .7])
        hold on;
        plot(sig_seg_t, sig_seg + (y_offset * 1.5), 'Color', [.1 .1 .1])
    end

    title("Multipeaks w/components in RoIs (" + stack.name + ")");
    xlabel("Time (s)");
    ylabel("df/f_0 (multiples of" + offset_multiples);
    savefig(fig, fullfile(PLOT_DIR, stack.name + "_multipeak_traces.fig"));


end

