function [ax, fig, bh] = plot_box(value_sets, group_names, sigstars, style, ax)
    if nargin < 5
        fig = figure();
        ax = gca;
    else
        fig = [];
    end
    
    nbars = length(value_sets(1).values);
    
    x = 1:nbars;
    data = [];
    err_high = [];
    err_low = [];
    for vset = value_sets
        data = [data ; vset.values]; %#ok<*AGROW>
        err_high = [err_high ; vset.error_high];
        err_low = [err_low ; vset.error_low];
    end 

    %bh = bar([1,2 ; 3,4 ; 1 4], 'EdgeColor', 'k','LineWidth',3)
    bh = bar(ax, x, data, 'EdgeColor', 'k','LineWidth',3);
    hold on;
    
    xticklabels(group_names)
    
    
    % face colors:
    for i = 1:length(value_sets)
        bh(i).FaceColor = value_sets(i).color; 
    end

    % error bars:
    for i = 1:length(bh)
        bgrp = bh(i);
        x = bgrp.XEndPoints;
        ys = data(i,:);
        highs = err_high(i,:);
        lows = err_low(i,:);
        errorbar(x, ys, highs, lows, '.', 'color', 'k', 'LineWidth', 3);

    end

    % sigstars:
    if ~isempty(sigstars)
        ylims = ylim();
        sigline_y = ylims(2) * 1.20;
        mark_y = ylims(2) * 1.15;
        ylims(2) = ylims(2) * 1.30;
        ylim(ylims);
        for i = 1:length(sigstars)
            star = sigstars(i);
            xs = [bh(1).XEndPoints(i) bh(2).XEndPoints(i)];
            ys = [bh(1).YEndPoints(i) bh(2).YEndPoints(i)];
            sig_ys = [sigline_y sigline_y];
            lh = line(xs, sig_ys, 'LineWidth', style.line_width, 'color', 'k');

            sig_offset = (lh.XData(2) - lh.XData(1))/2;

            text(lh.XData(1) + sig_offset, mark_y, star, ...
                'FontSize', style.font_size, ...
                'HorizontalAlignment','center');
        end
    end
    
    ax.XAxis.TickLength = [0 0];
    
    % thick lines:
    if ~isempty(fig)
        axs = findall(fig,'type','axes');

        for a = axs'
            a.LineWidth = style.line_width;
            a.FontSize = style.font_size;
            a.FontName = style.font_name;
            %ax.FontWeight = "bold";
            a.TitleFontSizeMultiplier = style.title_size_multiplier;
            set(findobj(a,'type','line'),'linew', style.line_width)
            %ax.Color = 'none';
        end
    end

end

