function data = trace_with_sem(varargin)
p = inputParser;
p.addRequired('t');
p.addRequired('traces');
p.addRequired('group');
p.addParameter('output_folder','');
p.addParameter('y_label','');
p.addParameter('x_label','');
p.addParameter('plot_callback',[]);
% Hack to allow 'p_values' be an optional positional input and still
% have parameter pairs work.
if nargin == 2
    varargin = [varargin(1:2),{[]}];
end
if nargin > 2 && ischar(varargin{3})
    varargin = [varargin(1:2),{[]},varargin(3:end)];
end
p.parse(varargin{:});
begonia.util.dump_inputParser_vars_to_caller_workspace(p);
%%

if ~isempty(output_folder)
    begonia.path.make_dirs(output_folder);
end

t = reshape(t,[],1);

set(0, 'DefaultTextInterpreter', 'none')
set(0, 'DefaultLegendInterpreter', 'none')

assert(length(t) == size(traces,2));

if isempty(group)
    mu = nanmean(traces,1);
    [sigma,N] = begonia.util.nansem(traces,1);
    
    mu = mu';
    sigma = sigma';
    N = N';

    f = figure;
    f.Position(3:4) = [1000,500];
    
    p = plot(t,mu);
    begonia.plot.plot_continuous_sem(p,sigma);
    
    ylabel(y_label);
    xlabel(x_label);

    ax = gca;
    ax.FontSize = 20;
    
    if ~isempty(plot_callback)
        plot_callback(f,ax);
    end

    if ~isempty(output_folder)
        export_fig(f,fullfile(output_folder,'trace.png'));
        export_fig(f,fullfile(output_folder,'trace.fig'));
    end
    %% N
    f_N = figure;
    f_N.Position(3:4) = [1000,500];

    plot(t,N);

    ax_N = gca;

    ylabel('Number of traces');
    xlabel(x_label);
    ax_N.FontSize = 20;

    if ~isempty(output_folder)
        export_fig(f_N,fullfile(output_folder,'trace_N.fig'));
        export_fig(f_N,fullfile(output_folder,'trace_N.fig'));
    end
    %% Data
    data = table(t,mu,sigma,N);
else
    [G,cats] = findgroups(group);
    cats = cellstr(cats);
    cats = strrep(cats,' ','_');
    
    mu = splitapply(@(x)nanmean(x,1), traces, G);
    sigma = splitapply(@(x)begonia.util.nansem(x,1), traces, G);
    N = splitapply(@(x)sum(~isnan(x),1),traces,G);
    
    data = cell(1,length(cats));
    for i = 1:length(cats)
        %%
        f(i) = figure;
        f(i).Position(3:4) = [1000,500];

        p = plot(t,mu(i,:));
        begonia.plot.plot_continuous_sem(p,sigma(i,:));
        title(cats{i});
        
        ylabel(y_label);
        xlabel(x_label);
        
        ax(i) = gca;
        ax(i).FontSize = 20;
    
        if ~isempty(plot_callback)
            plot_callback(f(i),ax(i),cats{i});
        end
        
        if ~isempty(output_folder)
            file_name = sprintf('trace_%s.png',cats{i});
            export_fig(f(i),fullfile(output_folder,file_name));
            
            file_name = sprintf('trace_%s.fig',cats{i});
            export_fig(f(i),fullfile(output_folder,file_name));
        end
        
        %%
        f_N(i) = figure;
        f_N(i).Position(3:4) = [1000,500];

        plot(t,N(i,:));
        title(cats{i});
        
        ax_N(i) = gca;
        
        ylabel('Number of traces');
        xlabel(x_label);
        ax_N(i).FontSize = 20;
        
        if ~isempty(output_folder)
            file_name = sprintf('trace_%s_N.png',cats{i});
            export_fig(f_N(i),fullfile(output_folder,file_name));
            
            file_name = sprintf('trace_%s_N.fig',cats{i});
            export_fig(f_N(i),fullfile(output_folder,file_name));
        end
        %% Save data
        cat_var = strrep(cats{i},':','_');
        var_names = {[cat_var,'_mu'],[cat_var,'_sigma'],[cat_var,'_N']};
        data{i} = table(mu(i,:)',sigma(i,:)',N(i,:)', ...
            'VariableNames',var_names);
        %%
        if length(cats) > 6
            close(f(i));
            close(f_N(i));
        end
    end
    data = cat(2,data{:});
    data = [table(t),data];
end

if ~isempty(output_folder)
    file_name = fullfile(output_folder,'data.csv');
    if exist(file_name, 'file')==2
      delete(file_name);
    end
    writetable(data,file_name)
end

end

