function transition_traces = create_transition_traces(t,traces,transition_point_trace_index,transition_point)
% The output is a matrix with size "length(t) x length(transition_point)"
% and is sorted in the same order as the transitions points. This means
% that transition_traces(:,23) is the trace around transition_point(23).

dt = t(2) - t(1);
fs = 1/dt;

if isrow(traces)
    traces = reshape(traces,[],1);
end

if iscell(traces)
    N_traces = length(traces);
else
    N_traces = size(traces,2);
end

transition_traces = nan(length(t),length(transition_point));

begonia.util.logging.backwrite();
for i = 1:N_traces
    str = sprintf('create_transition_traces (%d/%d)',i,N_traces);
    begonia.util.logging.backwrite(1,str);
    
    % Find each time point with this trace id.
    I = find(transition_point_trace_index == i);
    
    if isempty(I)
        continue
    end
    
    % Get trace
    if iscell(traces)
        trace = traces{i};
        trace = reshape(trace,[],1);
    else
        trace = traces(:,i);
    end
    
    if isempty(trace)
        continue;
    end
    
    % Find the start point of the variable 't' in the traces.
    t_start = transition_point(I) + t(1);
    t_start_idx = floor(t_start*fs) + 1;
    
    % Define a (Nx2) array of indices that contains the start and stop 
    % index of the transition trace.  
    I2 = zeros(length(t_start),2);
    I2(:,1) = t_start_idx;
    I2(:,2) = I2(:,1) + length(t) - 1;
    
    % Limit the indices to the length of the trace. 
    I2(:,1) = max(I2(:,1),1);
    I2(:,2) = min(I2(:,2),length(trace));
    
    % Convert to the indicies of t.
    I1 = I2 - t_start_idx + 1;
    
    % For each timepoint, extract the trace around the transition. 
    for j = 1:length(I)
        transition_traces(I1(j,1):I1(j,2),I(j)) = trace(I2(j,1):I2(j,2));
    end
end

end

