function vec = extract_roi_trace(mask,mat)
frames = size(mat,3);

[y,x] = find(mask);

if isempty(y)
    vec = [];
    return;
end

% Find the bounding box of the ROI. 
min_x = min(x);
min_y = min(y);
max_x = max(x);
max_y = max(y);

% Safety to keep all indices within the image 
min_x = max(min_x,1);
min_y = max(min_y,1);
max_x = min(max_x,size(mat,2));
max_y = min(max_y,size(mat,1));

% Extract the mask of the ROI with the bounding box.  
roi_mask = mask(min_y:max_y,min_x:max_x);
% Create a 3d mask with size equal to the input matrix. 
roi_mask = repmat(roi_mask,1,1,frames);

% Use the bounding box on the input matrix. 
chunk = mat(min_y:max_y,min_x:max_x,:);
% Areas not in the mask are zero. 
chunk(~roi_mask) = 0;

% Calculate the signal vector. 
vec = sum(sum(chunk, 1), 2) / length(x);
% Make it a row vector. 
vec = reshape(vec,1,[]);
end

