function transition_traces = extract_transitions(t,traces,transition_points,transition_points_trace_index)
% t - Specifies which time interval will be extracted. 
% traces - The trace(s) of which subsets will be extraced. Accepts cell
%       arrays or a matrix with trances along the columns. 
% transition_points - Timepoints of the transitions, which will be aligned
%       to 0 in the time vector, t.
% transition_points_trace_index (optional) - Specifies which trace the transitions
%       transition points should be extraced from. If not specified all the
%       transitions will be extraced from the first trace. 
%
% This is an updated version of create_transitions_traces. The older version
% is kept because of backward compatability.

dt = t(2) - t(1);
fs = 1/dt;

if nargin < 4
    transition_points_trace_index = ones(length(transition_points),1);
end

if isrow(traces)
    traces = reshape(traces,[],1);
end

if iscell(traces)
    N_traces = length(traces);
else
    N_traces = size(traces,2);
end

transition_traces = nan(length(t),length(transition_points));

begonia.util.logging.backwrite();
for i = 1:N_traces
    if N_traces > 1
        begonia.util.logging.backwrite(2,'extracting transitions (%d/%d)',i,N_traces);
    end
    
    % Find each time point with this trace id.
    I = find(transition_points_trace_index == i);
    
    if isempty(I)
        continue
    end
    
    % Get trace
    if iscell(traces)
        trace = traces{i};
        trace = reshape(trace,[],1);
    else
        trace = traces(:,i);
    end
    
    if isempty(trace)
        continue;
    end
    
    % Find the start point of the variable 't' in the traces.
    t_start = transition_points(I) + t(1);
    t_start_idx = floor(t_start*fs) + 1;
    
    % Define a (Nx2) array of indices that contains the start and stop 
    % index of the transition trace.  
    I2 = zeros(length(t_start),2);
    I2(:,1) = t_start_idx;
    I2(:,2) = I2(:,1) + length(t) - 1;
    
    % Limit the indices to the length of the trace. 
    I2(:,1) = max(I2(:,1),1);
    I2(:,2) = min(I2(:,2),length(trace));
    
    % Convert to the indicies of t.
    I1 = I2 - t_start_idx + 1;
    
    % For each timepoint, extract the trace around the transition. 
    for j = 1:length(I)
        transition_traces(I1(j,1):I1(j,2),I(j)) = trace(I2(j,1):I2(j,2));
    end
end

end

