function metadata = create_metadata_from_prarie_xml(stack)
% Create a default metadata_file from a prarie xml file.

xml_files = begonia.path.find_files(stack.path,'.xml',false);

assert(~isempty(xml_files),'No xml file found.')

xml_file = xml_files{1};

str = sprintf('Parsing : %s',begonia.util.logging.shorten_path(xml_file));
begonia.util.logging.vlog(1,str);

m = begonia.scantype.prairie.PrairieOutput.parse_xml_file_fast(xml_file);


name = strsplit(xml_file,filesep);
name = name{end-1};


files = cell(m.cycles,m.channels);
for cycle = 1:m.cycles
    for channel = 1:m.channels
        files{cycle,channel} = sprintf('%s_cycle_%.3d_ch%.2d.h5', name,cycle,channel);
    end
end

% Properties of : beognia.data_management.DataInfo
metadata.name = name;
metadata.source = 'Prairie';
metadata.type = m.type;
          
metadata.start_time_abs = m.date;
metadata.duration = seconds(m.frames_in_cycle * m.dt(1));
metadata.duration.Format = 'hh:mm:ss.SSS';   
metadata.time_correction = duration(0,0,0);

% Properties of : beognia.scantypes.TSeries

metadata.cycles = m.cycles;
metadata.channels = m.channels;
metadata.channel_names = m.channel_names;
metadata.frames_in_cycle = m.frames_in_cycle;
metadata.original_img_dim = m.original_img_dim;

metadata.dx = m.dx;
metadata.dy = m.dy;
metadata.dt = m.dt(1);

metadata.optical_zoom = m.optical_zoom;
metadata.processing_state = 'raw';
metadata.format = 'StackH5';

% Properties of : begonia.scantypes.procstack.TSeriesProcessed
metadata.files = files;
end

