classdef TSeriesPrairie < begonia.scantype.TSeries ...
        & begonia.scantype.prairie.PrairieOutput

    properties
        files
        time_relative
        time_absolute
        frame_period
        frame_position_um
    end
    
    methods
        function obj = TSeriesPrairie(path)
            import begonia.util.logging.*;

            obj@begonia.scantype.prairie.PrairieOutput(path);
            
            obj.frame_period = obj.dt;
            obj.dt = obj.frame_period(1);
            obj.time_relative = obj.metadata.relative_times;
            obj.time_absolute = obj.metadata.absolute_times;

            % check this path is a tseries:
            assert(exist(fullfile(path,'References'),'dir') == 7, ...
                'begonia:load:missing_Ref_folder', ...
                'References folder not found, used as an indicator for prarie tseries.');
            
            assert(contains(obj.metadata.type,'TSeries'), ...
                'begonia:load:unsupported_type', ...
                'Only supports TSeries as of now.');

                        
            obj.files = cellfun(@(filename) [obj.path,filesep,filename], ...
                obj.metadata.files, 'UniformOutput',false);
            
            files = obj.files(:);
            assert(~isempty(files),'begonia:load:empty_xml','The xml file has no tif files.');
            assert(exist(files{end},'file') == 2,'begonia:load:missing_tiffs','Missing tiff files!');
            
            obj.start_time_abs = obj.metadata.date;
            obj.duration = seconds(obj.frames_in_cycle * obj.dt');
            obj.duration.Format = 'hh:mm:ss.SSS';
            
            frame_position_um = [0 0 0];
        end
        
        function mat = get_mat(self,cycle,channel)
            files = self.files(cycle,channel,:);  %#ok<*PROPLC>
            mat = begonia.frame_providers.PrarieFrameProvider(files);
        end
        
        function mat = get_stack(obj, cy, ch)
            warning('(TSeriesPrairie) using depricated get_stack() call instead of get_mat()')
            mat = obj.get_mat(cy, ch);
        end
        
        function pos = get.frame_position_um(obj)
            import begonia.scantype.prairie.*;
            pos = get_coordinate_from_tsxml(obj.xml_file);
        end
        
    end
end

