classdef TSeriesProcessed < begonia.scantype.TSeries ...
        & begonia.data_management.DataLocation ...
        & begonia.data_management.DataInfo ...
        & dynamicprops
    
    properties 
        uuid
        name
        type
        source = 'TSeriesProcessed'
        cycles
        channels
        channel_names
        frames_in_cycle
        original_img_dim
        dx
        dy
        dt
        optical_zoom
        record_date
        files
        processing_state
        format
        xml_file
        metadata
        
        time_relative
        time_absolute
        frame_period
        
        % DataInfo props:
        start_time_abs
        duration
        time_correction
        
        frame_position_um
    end
    
    methods
        %% Constructor
        function obj = TSeriesProcessed(path)
            import begonia.util.logging.*;
            import begonia.scantype.prairie.*;
            
            obj@begonia.data_management.DataLocation(path);

            obj.metadata = obj.load_metadata();

            obj.dl_ensure_has_uuid();
            obj.uuid = obj.dl_unique_id;
            
            frame_position_um = [0 0 0];
        end
        
        %% Unimplemented methods
        function mat = get_mat(~,~,~)
            error('TSeriesProcessed.get_mat needs to be overridden!');
        end
        
        function mat = get_stack(obj, cy, ch)
            warning('(TSeriesH5) using depricated get_stack() call instead of get_mat()')
            mat = obj.get_mat(cy, ch);
        end
        
            
        function pos = get.frame_position_um(obj)
            import begonia.scantype.prairie.*;
            pos = get_coordinate_from_tsxml(obj.xml_file);
        end
        
        function path = get.xml_file(obj)
            path = char(fullfile(obj.path, obj.name + ".xml"));
        end
        
        %% Methods in folder
        metadata = load_metadata(self);
        
        img = get_avg_img(self,cycle,channel);
        img = get_std_img(self,cycle,channel);
        img = get_max_img(self,cycle,channel);
        img = get_f0_img(self,cycle,channel);
    end

    
end

