classdef GPNGWriter < handle
    %MPNGWRITER Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        path
        frame = 1;
        title = 'Untitled'
        dim
        ds_ratio
        file_prefix
        original_frames
        frames
        
        file_number
        row
        column
        frame_in_file
    end
    
    properties(Access = private)
        writing_file_nr = 1
        writing_mat = [];
        width
        height
        file_list;
    end
    
    methods
        function obj = GPNGWriter(path, dim, ds_ratio, file_prefix, title)
            obj.path = path;
            obj.title = title;
            obj.dim = dim;
            obj.ds_ratio = ds_ratio;
            obj.file_prefix = file_prefix;
        end
        
        % numer of the file given current frame
        function fn = get.file_number(obj)
            fn = ceil(obj.frame/obj.dim^2);
        end
        
        % fram number in current file, given current frame
        function ff = get.frame_in_file(obj)
            ff = obj.frame - (obj.file_number - 1) * obj.dim^2;
        end
        
        % row of current frame in current file
        function row = get.row(obj)
            row = ceil((obj.frame_in_file/(obj.dim^2))*obj.dim);
        end
        
        % column of curent fram in current file
        function col = get.column(obj)
            col = obj.frame_in_file - ((obj.row - 1)*obj.dim);
        end

        function open(obj, mat)
            obj.frame = 1;
            obj.writing_file_nr = 1;
            obj.width = size(mat, 1);
            obj.height = size(mat, 2);
            obj.writing_mat = uint16([]);
            obj.original_frames = size(mat, 3);
            obj.file_list = [];
        end
        
        % writes the matrix to current file, and 
        function write(obj, mat)
            
            % create matrix to write if it does not exist:
            if isempty(obj.writing_mat)
                obj.writing_mat = repmat(mat(1,1,1) ...
                    , obj.width * obj.dim ...
                    , obj.height * obj.dim);
            end
            
            % conversely, do we need to save writing buffer?
            if obj.writing_file_nr ~= obj.file_number
                obj.save_buffer();
                obj.writing_mat =  uint16([]);
            end

            x = 1 + ((obj.column - 1) * obj.width);
            y = 1 + ((obj.row - 1) * obj.height);
            obj.writing_mat(x:(x+obj.width-1),y:(y+obj.height-1)) = mat;
            obj.frame = obj.frame + 1;
        end
        
        % ensures last writing buffer is saved to file:
        function close(obj)
            obj.save_buffer();
            
            % write basic metadata for reader:
            config = struct();
            config.api = '1.0';
            config.title = obj.title;
            config.dim = obj.dim;
            config.ds_ratio = obj.ds_ratio;
            config.original_frames = obj.original_frames;
            config.width = obj.width;
            config.height = obj.height;
            config.frames = floor(obj.original_frames / obj.dim^2);
            config.file_list = obj.file_list;
            config.written = datetime();
            config.matlab = version;
            
            confpath = fullfile(obj.path, 'mpng_config.json');
            json = jsonencode(config);
            fid = fopen(confpath,'wt');
            fprintf(fid, json);
            fclose(fid);
        end
        
        % takes current writing buffer, and saves that to writing file
        % number:
        function save_buffer(obj)
            import begonia.util.logging.*;
            
            fname = [obj.file_prefix '_file_' ...
                num2str(obj.writing_file_nr) '.png'];
            
            vlog(2, ['Wrote: ' fname]);
            obj.file_list = [obj.file_list ; string(fname)];
            
            fpath = fullfile(obj.path, fname);
            imwrite(obj.writing_mat, fpath);
            obj.writing_file_nr = obj.writing_file_nr + 1;
        end

    end
end

























