classdef SITSseries < begonia.scantype.TSeries & ...
        begonia.data_management.DataLocation
    
    properties 
        uuid
        name
        type
        source
        
        start_time_abs
        duration
        time_correction
        
        cycles
        channels
        channel_names
        frames_in_cycle
        original_img_dim
        
        dx
        dy
        dt
        time_relative
        time_absolute
        frame_period
        
        optical_zoom
        record_date
        processing_state
        format
        
        frame_position_um
        
        metadata_string
    end
    
    methods
        function self = SITSseries(path)
            self@begonia.data_management.DataLocation(path);
            
            assert(~isfolder(path), ...
                'begonia:load:is_directory', ...
                'Input path is a directory. SITSseries only accepts .tif files.');
            
            self.load_metadata();
        end
        
        function load_metadata(self,force)
            if nargin < 2
                force = false;
            end
            
            if force || ~self.has_var('scanimage_metadata')
                scanimage_metadata = ...
                    begonia.scantype.scanimage.extract_metadata(self.path);
                self.save_var(scanimage_metadata);
            else
                scanimage_metadata = self.load_var('scanimage_metadata');
            end
            
            self.uuid = self.dl_unique_id;
            
            self.channel_names      = scanimage_metadata.channel_names;
            self.channels           = scanimage_metadata.channels;
            self.original_img_dim   = scanimage_metadata.original_img_dim;
            self.dt                 = scanimage_metadata.dt;
            self.dx                 = scanimage_metadata.dx;
            self.dy                 = scanimage_metadata.dx;
            self.cycles             = scanimage_metadata.cycles;
            self.optical_zoom       = scanimage_metadata.optical_zoom;
            self.start_time_abs     = scanimage_metadata.start_time_abs;
            self.duration           = scanimage_metadata.duration;
            self.metadata_string    = scanimage_metadata.metadata_string;
            self.frames_in_cycle    = scanimage_metadata.frames_in_cycle;
            
            self.start_time_abs.Format = 'uuuu/MM/dd HH:mm:ss';
            
            self.name = ['ScanImageTiff ',char(self.start_time_abs)];
            self.format = 'ScanImageTiff';
            self.processing_state = 'raw';
        end
        
        function metadata = extract_metadata(self)
            metadata = begonia.scantype.scanimage.extract_metadata(self.path);
        end
        
        function mat = get_mat(self,cycle,channel)
            reader = ScanImageTiffReader.ScanImageTiffReader(self.path);
            mat = reader.data();
            mat = mat(:,:,channel:self.channels:end);
        end
    end
end

