function metadata = extract_metadata(path)
% Load a certain data from the metadata from the tif file and
% returns a struct of that data. 

metadata = struct;

reader = ScanImageTiffReader.ScanImageTiffReader(path);
txt = reader.metadata();

assert(~isempty(txt), ...
    'begonia:load:metadata_read_error', ...
    'Could not read the ScanImage metadata from .tif');

temp = regexp(txt,'(?<=SI.hChannels.channelName = ).*?(?=\n)','match');
metadata.channel_names = eval(temp{1});
metadata.channels = length(metadata.channel_names);

y = regexp(txt,'(?<=SI.hRoiManager.linesPerFrame = ).*?(?=\n)','match');
y = str2double(y{1});
x = regexp(txt,'(?<=SI.hRoiManager.pixelsPerLine = ).*?(?=\n)','match');
x = str2double(x{1});
metadata.original_img_dim = [x,y];

temp = regexp(txt,'(?<=SI.hRoiManager.scanFramePeriod = ).*?(?=\n)','match');
metadata.dt = str2double(temp{1});

temp = regexp(txt,'(?<=SI.hCycleManager.totalCycles = ).*?(?=\n)','match');
metadata.cycles = str2double(temp{1});

temp = regexp(txt,'(?<=SI.hRoiManager.scanZoomFactor = ).*?(?=\n)','match');
metadata.optical_zoom = str2double(temp{1});

temp = regexp(txt,'(?<=SI.hScan2D.fillFractionSpatial = ).*?(?=\n)','match');
metadata.dx = str2double(temp{1});
metadata.dy = metadata.dx;

metadata.metadata_string = txt;

frames = reader.descriptions;
temp = regexp(frames{1},'(?<=epoch = ).*?(?=\n)','match');
first = eval(temp{1});
first = datetime(first);

metadata.start_time_abs = first;

metadata.frames_in_cycle = length(frames) / metadata.channels;

dur = seconds(metadata.frames_in_cycle*metadata.dt);
dur.Format= 'hh:mm:ss.SSS';
metadata.duration = dur;

delete(reader);
end

