classdef TSeries < begonia.data_management.DataInfo
    
    properties(Abstract)
        cycles
        channels
        channel_names
        frames_in_cycle
        original_img_dim
        
        dx
        dy
        dt
        time_relative
        time_absolute
        frame_period
        
        optical_zoom
        record_date
        processing_state
        format
        
        frame_position_um
    end
    
    methods (Abstract)
        mat = get_mat(self,cycle,channel);
    end
    
    
    methods
        function img = get_f0_img(self,cycle,channel)
            mat = self.get_mat(cycle,channel);
            
            c = begonia.util.Chunker(mat,'chunk_axis',1,'data_type','double');
            
            imgs = cell(1,c.chunks);
            begonia.util.logging.backwrite();
            for i = 1:c.chunks
                str = sprintf('Calculating f0 image chunk %d/%d',i,c.chunks);
                begonia.util.logging.backwrite(1,str);
                
                I = c.matrix_cell_index(i);
                
                imgs{i} = mode(mat(I{:}),3);
            end
            img = cat(1,imgs{:});
            begonia.util.logging.vlog(1,'Complete');
        end
        
        
        function img = get_std_img(self,cycle,channel)
            mat = self.get_mat(cycle,channel);
            
            c = begonia.util.Chunker(mat,'chunk_axis',1,'data_type','single');
            
            imgs = cell(1,c.chunks);
            begonia.util.logging.backwrite();
            for i = 1:c.chunks
                str = sprintf('Calculating std image chunk %d/%d',i,c.chunks);
                begonia.util.logging.backwrite(1,str);
                
                I = c.matrix_cell_index(i);
                
                imgs{i} = std(single(mat(I{:})),[],3);
            end
            img = cat(1,imgs{:});
            begonia.util.logging.vlog(1,'Complete');
        end
        
        
        function img = get_max_img(self,cycle,channel)
            mat = self.get_mat(cycle,channel);
            
            c = begonia.util.Chunker(mat,'chunk_axis',1);
            
            imgs = cell(1,c.chunks);
            begonia.util.logging.backwrite();
            for i = 1:c.chunks
                str = sprintf('Calculating max image chunk %d/%d',i,c.chunks);
                begonia.util.logging.backwrite(1,str);
                
                I = c.matrix_cell_index(i);
                
                imgs{i} = max(mat(I{:}),[],3);
            end
            img = cat(1,imgs{:});
            begonia.util.logging.vlog(1,'Complete');
        end

        function img = get_avg_img(self,cycle,channel)
            mat = self.get_mat(cycle,channel);
            
            c = begonia.util.Chunker(mat,'chunk_axis',1);
            
            imgs = cell(1,c.chunks);
            begonia.util.logging.backwrite();
            for i = 1:c.chunks
                str = sprintf('Calculating average image chunk %d/%d',i,c.chunks);
                begonia.util.logging.backwrite(1,str);
                
                I = c.matrix_cell_index(i);
                
                imgs{i} = mean(mat(I{:}),3);
            end
            img = cat(1,imgs{:});
            begonia.util.logging.vlog(1,'Complete');
        end
        
        function img = get_dff_std_image(self, cycle, channel)
            import begonia.util.logging.*;
            mat = self.get_mat(cycle,channel);
            mat_f0 = self.get_f0_img(cycle, channel);
            
            img = std(mat_dff,[],3);
        end
    end
    
    
end

