function stacks = find_scans(varargin)
    import begonia.scantype.*;
    import begonia.scantype.prairie.*;
    import begonia.scantype.h5.*;

    % find_stacks recursively searchs for stacks at the specified directory.
    %   Looks for StackPrarie,StackH5 or StackOldV1.
    %
    %   stacks = find_stacks(dir_in)
    %   stacks = find_stacks(...,NAME,VALUE)
    %
    %   REQUIRED
    %   dir_in              - (char) directory.
    %
    %   PARAMETERS
    %   show_waitbar        - (logical) 
    %                           Show a gui waiting window while searching for
    %                           stacks (default false).
    %   ignore_str          - (char,default:'') 
    %                           Directores containing ignore_str will not be 
    %                       entered when searching for stacks. This is useful 
    %                       in the case where stacks might contain many
    %                       files that slows down the search. E.g. if a stack
    %                       folder is named 'TSeries001', if input_str =
    %                       'TSeries' the stack will still be found be the
    %                       algorithm will not look inside 'TSeries001' for more
    %                       stacks.
    %
    %   RETURNED
    %   stacks              - (begonia.stack.Stack) empty if not found. 
    p = inputParser;
    p.addRequired('dir_in',...
        @(x) validateattributes(x,{'char'},{'nonempty'}));
    p.addParameter('show_waitbar',false, ...
        @(x) validateattributes(x,{'logical'},{}));
    p.addParameter('ignore_str','', ...
        @(x) validateattributes(x,{'char'},{}));
    p.parse(varargin{:});
    begonia.util.dump_inputParser_vars_to_caller_workspace(p);

    stack_classes = {...
        @TSeriesH5, ...
        @TSeriesPrairie, ...
        @LinescanPrairie, ...
        @ZStackPrairie, ...
        @SingleImagePrairie, ...
        @begonia.scantype.proxy.TSeriesProxy};
    
    begonia.util.logging.vlog(1,'Looking for scans');
    stacks = find_by_class(dir_in,stack_classes,...
        'show_waitbar',show_waitbar,...
        'ignore_str',ignore_str);

end

