function drift_correction_trace = read(stack, plot_traces, drift_trace)
if nargin < 2
    plot_traces = false;
end
str = sprintf('Drift correcting : %s',stack.path);
begonia.util.logging.vlog(1,str);
%% Get a trace to calculate drifting on.
if nargin < 3
    drift_trace = stack.load_var('channel_traces');
end

drift_trace_t = drift_trace.Time;
drift_trace = drift_trace.Data(:,1);
drift_trace_complete = drift_trace;
drift_trace_complete_t = drift_trace_t;
drift_correction_trace_t = drift_trace_t;
%% Remove outliers (usually pan activations)
sigma = std(drift_trace);
mu = mean(drift_trace);
I = drift_trace > mu + 2*sigma;
I = begonia.util.broaden_positives(I,stack.dt,5);
drift_trace_t(I) = [];
drift_trace(I) = [];
%% Drift corretion / fitting
coeff = polyfit(drift_trace_t,drift_trace,1);

drift_correction_trace = coeff(end)./polyval(coeff,drift_correction_trace_t);

str = sprintf('df/f0 drift at end of signal = %.3f',drift_correction_trace(end)-1);
begonia.util.logging.vlog(1,str);

if plot_traces
    figure
    hold on
    plot(drift_trace_t,drift_trace)
    plot(drift_trace_t,polyval(coeff,drift_trace_t))
    hold off


    figure
    hold on
    p(1) = plot(drift_trace_complete_t,drift_trace_complete,'DisplayName','raw');
    p(2) = plot(drift_trace_complete_t,drift_trace_complete.*drift_correction_trace,'DisplayName','corrected');
    hold off
    legend(p);
end
end

