function [roa_mask,img_f0,img_sigma] = read(stack, roa_config)
if nargin < 2
    roa_config = begonia.stack_modules.roa_mask.original.roa_config;
end

% to enable testing on "low-memory" (16 bloody gigs..), limit frames are
% possible:
if isempty(roa_config.frames)
    frames = 1:stack.frames_in_cycle;
else
    frames = roa_config.frames;
end

str = sprintf('Calculating ROA mask : %s',stack.path);
begonia.util.logging.vlog(2,str);

%% Load drift correction.
begonia.util.logging.vlog(2,'Loading matrix');
mat = stack.get_mat(roa_config.cycle,roa_config.channel);
mat = mat(:,:,frames);
mat = single(mat);

if roa_config.drift_correct
    drift_correction_trace = stack.load_var('drift_correction_trace');
    drift_correction_trace = reshape(drift_correction_trace,1,1,[]);
    drift_correction_trace = single(drift_correction_trace);
    mat = mat .* drift_correction_trace(frames);
end
%% Smoothing xy

if ~isnan(roa_config.filter_sigma_dx)
    sigma_dx = ceil(roa_config.filter_sigma_dx/stack.dx);
elseif ~isnan(roa_config.filter_sigma_abs_dx)
    sigma_dx = roa_config.filter_sigma_abs_dx;
else
    sigma_dx = [];
end

if ~isempty(sigma_dx)
    filter_spatial = ones(1+2*sigma_dx,1+2*sigma_dx);
    filter_spatial = filter_spatial./sum(filter_spatial(:));
    filter_spatial = single(filter_spatial);
    
    begonia.util.logging.vlog(2,'Spatial smoothing');
    mat = convn(mat,filter_spatial,'same');
end
%% Temporal filter
if ~isnan(roa_config.filter_sigma_dt)
    sigma_dt = ceil(roa_config.filter_sigma_dt/stack.dt);
elseif ~isnan(roa_config.filter_sigma_abs_dt)
    sigma_dt = roa_config.filter_sigma_abs_dt;
else
    error('Temporal filter must be defined.');
end

filter_temporal = ones(1,1,1+2*sigma_dt);
filter_temporal = filter_temporal./sum(filter_temporal(:));
filter_temporal = single(filter_temporal);

begonia.util.logging.vlog(2,'Temporal smoothing');
mat_smooth = convn(mat,filter_temporal,'same');

%% Calculate f0 image
begonia.util.logging.vlog(2,'Calculating f0 img');
mat_smooth = uint16(mat_smooth);
img_f0 = mode(mat_smooth,3);
img_f0 = single(img_f0);
clear mat_smooth;

%% smooth df/f0 video
begonia.util.logging.vlog(2,'Temporal smoothing');
mat = mat ./ img_f0 - 1;
mat_smooth = convn(mat,filter_temporal,'same');
mat = mat - mat_smooth;
%%
begonia.util.logging.vlog(2,'img_sigma');
img_sigma = std(mat,[],3);
img_sigma = img_sigma/sqrt(numel(filter_temporal));

clear mat;

roa_mask = mat_smooth > img_sigma * roa_config.threshold;

begonia.util.logging.vlog(2,'Complete');
end

