%{
Low memory implementation of the read_fast veroisn of the roa mask
extraction. Uses chunkxyfun on the 3D array.
%}

function automatic_roi_mask = read(stack, cycle, channel, divisions, use_gpu, drift_correction_trace, blank_frames)
    import begonia.util.logging.*;
    import begonia.util.*;
    import begonia.stack_modules.roa_mask.*;
    tic; t = tic;
    
    % default arguments. MatLab: death by 1000 little cuts.
    if nargin < 7
        blank_frames = [];
    end
    if nargin < 6 || isempty(drift_correction_trace)
        drift_correction_trace = stack.load_var('drift_correction_trace');
    end
    if nargin < 5
        use_gpu = false;
    end
    if nargin < 4
        vlog(1, '(RoA mask) Defaulting to 2^2 divisions');
        divisions = 2;
    end
    if nargin < 3
        channel = 1;
    end
    if nargin < 2
       cycle = 1; 
    end
    
    % load matrix:
    mat = stack.get_mat(cycle, channel);
    
    % make filter:
    vlog(1, '(RoA mask) Creating filter');
    sigma_dx = 1;
    sigma_dt = 0.1/stack.dt;

    win_dx = gausswin(sigma_dx);
    win_dt = gausswin(sigma_dt);
    win_dt = reshape(win_dt,1,1,[]);
    chunk_overlap = (length(win_dt) - 1) / 2;

    filter = win_dx*win_dx';
    filter = repmat(filter,1,1,length(win_dt));
    filter = filter.*win_dt;
    filter = filter/sum(filter(:));
    filter = retype(filter);


    % Load drift correction.
    drift_correction_trace = reshape(drift_correction_trace,1,1,[]);
    drift_correction_trace = retype(drift_correction_trace);
    
    if use_gpu
        filter = gpuArray(filter);
        drift_correction_trace = gpuArray(drift_correction_trace);
    end
    
    % mask retrival function:
    vlog(1, '(RoA mask) Starting processing');
    maskfunc = @(m,xc,yx,ct) get_mask(m, drift_correction_trace, filter, sigma_dt, use_gpu, blank_frames);
    automatic_roi_mask = chunkxyfun(maskfunc, mat, divisions, chunk_overlap);
    automatic_roi_mask = automatic_roi_mask  > 2.0;
    
    vlog(1, ['(RoA mask) done - time taken: ' num2str(toc(t)) 's']);
end

function automatic_roi_mask = get_mask(mat, drift_correction_trace, filter, sigma_dt, use_gpu, blank_frames)
    import begonia.util.logging.*;
    import begonia.util.*;
    
    tic;
    
    if use_gpu
        vlog(1, '(RoA mask:inner) Transfering to gpuArray');
        mat = gpuArray(mat);
    end
    
    if ~isempty(blank_frames)
        vlog(1, '(RoA mask:inner) Blanking frame range' );
        remreg = mat(:,:,blank_frames);
        mat(:,:,blank_frames) = zeros(size(remreg));
    end
    
    vlog(1, '(RoA mask:inner) Correcting drift');
    mat = retype(mat);
    mat = mat .* drift_correction_trace;
    
    vlog(1, '(RoA mask:inner) Applying filter');
    mat = convn(mat, filter, 'same');

    vlog(1, '(RoA mask:inner) Downsampling and convert to 16 bit');
    mat_down = mat(:,:,1:ceil(sigma_dt):end);
    mat_down = round(mat_down); % less data, and rounding is req. for mode
    
    vlog(1, '(RoA mask:inner) Calculating mode');
    img_f0 = mode(mat_down,3);
    img_f0 = retype(img_f0);
    
    vlog(1, '(RoA mask:inner) Calculating mask');
    mat = retype(mat) ./ retype(img_f0);
    automatic_roi_mask = mat; %>= 2.0;
    
    if use_gpu
        vlog(1, '(RoA mask:inner) Transfering from gpuArray');
        automatic_roi_mask = gather(automatic_roi_mask);
    end
    
    vlog(1, '(RoA mask:inner) Done');
end


function to = retype(ti)
    to = single(ti);
end


