function roa_table = read(stack)
str = sprintf('Loading roa (experimental) data (duration : %ds)',round(seconds(stack.duration)));
begonia.util.logging.vlog(2,str);
mat = stack.load_var('roa_mask');

dt = stack.dt;
dx = stack.dx;
%%
roa_ignore_mask = false(size(mat,1),size(mat,2));
roa_ignore_mask = stack.load_var('roa_ignore_mask',roa_ignore_mask);

edge_ignore_width = 15;
roa_ignore_mask(1:edge_ignore_width,:) = true;
roa_ignore_mask(end-edge_ignore_width:end,:) = true;
roa_ignore_mask(:,1:edge_ignore_width) = true;
roa_ignore_mask(:,end-edge_ignore_width:end) = true;

begonia.util.logging.vlog(2,'Applying ignore mask')
mat = mat & ~roa_ignore_mask;
%%
begonia.util.logging.vlog(2,'Finding connected components')
CC = bwconncomp(mat,6);
%%
begonia.util.logging.vlog(2,'Gathering data of each connected component')
roa_t_start = zeros(CC.NumObjects,1);
roa_t_end = zeros(CC.NumObjects,1);
roa_vol_size = zeros(CC.NumObjects,1);
roa_xy_size = zeros(CC.NumObjects,1);

for i = 1:CC.NumObjects
    [x,y,t] = ind2sub(CC.ImageSize,CC.PixelIdxList{i});
    
    t = unique(t);
    
    % Calculate the max x,y roa size. 
%     xy_size = unique([x,y],'rows');
%     xy_size = size(xy_size,1);
    % This method is the same as the commented lines above, but twice as
    % fast.
    xy_size = unique(x*(CC.ImageSize(2)+1)+y);
    xy_size = length(xy_size);
    
    roa_t_start(i) = t(1) * dt;
    roa_t_end(i) = t(end) * dt + dt;
    roa_xy_size(i) = xy_size * dx * dx;
    roa_vol_size(i) = length(CC.PixelIdxList{i}) * dt * dx * dx;
end
roa_table = table(roa_t_start,roa_t_end,roa_xy_size,roa_vol_size);
%%
begonia.util.logging.vlog(2,'Finished');
end

