function roa_table_ext = read(stack)
str = sprintf('Loading roa data (duration : %ds)',round(seconds(stack.duration)));
begonia.util.logging.vlog(2,str);

roa_mask        = stack.load_var('roa_mask');
img_f0          = stack.load_var('img_f0');
img_f0          = single(img_f0);

%%
begonia.util.logging.vlog(2,'Loading recording');
mat = stack.get_mat(1,1);
mat = mat(:,:,:);
mat = single(mat);

% filter = ones(3,3,3,'single');
% filter = filter./sum(filter(:));

% filter = begonia.util.gausswin(3);
% filter = filter.*filter'.*reshape(filter,1,1,[]);
% mat = convn(mat,filter,'same');

%%
roa_ignore_mask = false(size(mat,1),size(mat,2));
roa_ignore_mask = stack.load_var('roa_ignore_mask',roa_ignore_mask);

edge_ignore_width = 15;
roa_ignore_mask(1:edge_ignore_width,:) = true;
roa_ignore_mask(end-edge_ignore_width:end,:) = true;
roa_ignore_mask(:,1:edge_ignore_width) = true;
roa_ignore_mask(:,end-edge_ignore_width:end) = true;

begonia.util.logging.vlog(2,'Applying ignore mask')
roa_mask = roa_mask & ~roa_ignore_mask;
%%
begonia.util.logging.vlog(2,'Finding connected components')
CC = bwconncomp(roa_mask,6);
%% Parameters
img_size = CC.ImageSize(1:2);

dt = stack.dt;
dx = stack.dx;

roa_trace_fs        = 30; % Hz
roa_trace_duration  = 30; % s
roa_trace_start     = 5; % s
roa_trace_N = roa_trace_duration * roa_trace_fs;
%%
begonia.util.logging.vlog(2,'Gathering data of each connected component')
roa_t_start     = zeros(CC.NumObjects,1);
roa_t_end       = zeros(CC.NumObjects,1);
roa_frame_start = zeros(CC.NumObjects,1);
roa_vol_size    = zeros(CC.NumObjects,1);
roa_xy_size     = zeros(CC.NumObjects,1);
roa_trace       = nan(CC.NumObjects,roa_trace_N,'single');

begonia.util.logging.backwrite();
for i = 1:CC.NumObjects
    if mod(i-1,100) == 0
        str = sprintf('ROA (%d/%d)',i,CC.NumObjects);
        begonia.util.logging.backwrite(2,str);
    end
    
    roa_vol_size(i) = length(CC.PixelIdxList{i}) * dt * dx * dx;
    
    [x,y,t] = ind2sub(CC.ImageSize,CC.PixelIdxList{i});
    
    % Find all x and y indices which has been active in the ROA. 
    xy = unique([x,y],'rows');
    
    % Project the 3d active voxels to a 2d "ROI" / black and white image.
    I = sub2ind(img_size,xy(:,1),xy(:,2));
    roa_img_tmp = false(img_size);
    roa_img_tmp(I) = true;
    
    % Calculate the size of the 2d ROA.
    roa_xy_size(i) = sum(roa_img_tmp(:)) * dx * dx;
    
    % Calculate start and stop indices of the roa trace. The first index
    % is defined as the start of the roa minus a number of seconds defined
    % in 'roa_trace_start'. 
    t1 = min(t) - roa_trace_start * roa_trace_fs;
    t2 = t1 + roa_trace_N - 1;
    
    % If the start of the trace is before the start of the recording then
    % the assignment is skipped and the whole trace will be NaN. 
    if t1 > 0 && t2 < size(mat,3)
        % Slice a square around the 2d ROA and extract the recording. 
        x1 = min(x);
        y1 = min(y);
        x2 = max(x);
        y2 = max(y);
        mat_slice = mat(x1:x2,y1:y2,t1:t2);
        
        % Calculate df/f0.
        f0_slice = img_f0(x1:x2,y1:y2);
        mat_slice = mat_slice ./ f0_slice - 1;
        
        % Multiply areas outside the 2d ROA with 0. 
        mat_slice = mat_slice .* roa_img_tmp(x1:x2,y1:y2);
        % Create a mean trace from the 3d volume. 
        mat_slice = sum(mat_slice,1);
        mat_slice = sum(mat_slice,2);
        roa_trace_tmp = reshape(mat_slice,1,[]) / sum(roa_img_tmp(:));
        
        % Assign.
        roa_trace(i,:) = roa_trace_tmp;
    end
    
    roa_t_start(i) = min(t-1) * dt;
    roa_t_end(i) = max(t-1) * dt + dt;
    roa_frame_start(i) = min(t);
end

roa_duration = roa_t_end - roa_t_start;

roa_table_ext = table(roa_duration,roa_t_start,roa_t_end,roa_xy_size,roa_vol_size,roa_trace,roa_frame_start);
%%
begonia.util.logging.vlog(2,'Finished');
end

