function roi_fluorescence(tseries,preload_imaging_data)
if nargin < 2
    preload_imaging_data = true;
end

% Delete the previous data. 
tseries.clear_var('roi_fluorescence');

% Load roi_array, if it is not stored return an empty value.  
roi_array = tseries.load_var('roi_array',[]);

% If it does not exist or is empty return.
if isempty(roi_array)
    return;
end

% roi_array has previously been stored as structs, but tables are easier to
% work with.
if isstruct(roi_array)
    roi_array = struct2table(roi_array,'AsArray',true);
end

% Make a copy of the ROI table where the traces will be appended to.
roi_fluorescence = roi_array;

% Add the frame rate.
roi_fluorescence.fs = repmat(1/tseries.dt,height(roi_fluorescence),1);
% Init empty values for the traces. 
roi_fluorescence.fluo_trace = cell(height(roi_fluorescence),1);

% Extract traces from each cycle and channel. 
for cy = 1:tseries.cycles
    for ch = 1:tseries.channels
        % If there are no ROIs in this combination of channel and cycle,
        % skip. 
        I = find(roi_fluorescence.channel == ch & roi_fluorescence.cycle == cy);
        if isempty(I)
            continue;
        end
        
        % Get the imaging data for the current cycle and channel. 
        begonia.util.logging.vlog(2,'Loading imaging from channel %d cycle %d.',ch,cy);
        mat = tseries.get_mat(cy,ch);
        
        % The data from tseries.get_mat is usually lazy. Preloading the
        % data makes the program run faster, but machines with little RAM
        % can run out of memory. 
        if preload_imaging_data
            mat = mat(:,:,:);
        end
        
        % Make I a row vector so we can use it in a for loop. 
        I = reshape(I,1,[]);
        
        % Extract the fluo. signal from each ROI. 
        begonia.util.logging.backwrite();
        for i = I
            begonia.util.logging.backwrite(2,'Extract trace from ROI #%d.',i);
            % Get the ROI mask. 
            mask = roi_fluorescence.mask{i};
            % Extract the trace. 
            vec = begonia.processing.extract_roi_trace(mask,mat);
            % Save it to the table. 
            roi_fluorescence.fluo_trace{i} = vec;
        end
    end
end

tseries.save_var(roi_fluorescence);

end

