function ax = plot_traces(stack, mark_treshold, shade_non_zero_mode)
    rpa = stack.load_var('rpa');
    traces = cell2mat(rpa.pixel_activity);

    fig = figure();
    fig.Color = 'white';    
    
    cols = struct();
    cols.main = [.3 .3 .3];
    cols.mark = 'red';
    cols.shade = [.9294 .6902 .1294];
    
    % adding three nan-points so we can have legend by color:
    plot(nan, 'Color', cols.main); hold on;
    plot(nan, 'Color', cols.mark);
    plot(nan, 'Color', cols.shade);
    
    % offset the graph by 2 std for every roi,:
    offset = 0;
    offset_step = 2*std(traces(:));
    offset_step = ceil(offset_step/10) * 10; % round to nearest 10 because humans
    
    for i = 1:height(rpa)
        
        data = rpa(i,:).pixel_activity{:};
        data_t = (1:size(traces, 2)) * stack.dt;
        
        color = cols.main;
        mark_color = cols.mark;
        
        % draw in different color if marking non-zero mode traces:
        if shade_non_zero_mode && mode(data(:) ~= 0)
            color = cols.shade;
            mark_color = color;
        end
        
        plot(data_t, data + offset, 'Color', color)
        hold on;
        
        % mark areas of trace > 0
        data(find(data <= mark_treshold)) = nan;
        plot(data_t, data + offset, 'Color', mark_color);
        
        
        offset = offset + offset_step;
    end
    
    title(stack.name + " marked > " + mark_treshold);
    xlabel("Time (s)");
    ylabel("Active pizels (#) (in multiples of " + offset_step + ")")
    xlim([0 data_t(end)])
    ax = fig.CurrentAxes;
    legend({'# active pixels', ['> ' num2str(mark_treshold)], '0 not dominant value'});
end

