%{
ROI PIXEL ACTIVITY (RPA)
This module loads each roi of a stack, then multiplies each frame in roa
mask with the roi, and counts remaining active pixels.
%}
function table_rpa = read(stack)

    import begonia.util.logging.*;
    
    % check stuff is ok to go:
    if ~stack.has_var('roi_array') || ~stack.has_var('roa_mask') 
        vlog(2, stack.name + " Stack missing roi array or roa mask - skipping")
        error("Missing data - check log");
    end
    
    if ~stack.has_var('roa_config')
        warning("No roa_config - are you using an old mask/stack?")
    end
    
    % sum active pixels pr. roi pr. frame:
    ts_name_ = categorical({stack.name});
    roi_array = stack.load_var('roi_array');
    roa_mask = stack.load_var('roa_mask');
    
    roi_cnt = length(roi_array);
    frame_cnt = size(roa_mask, 3);
    
    for i = 1:roi_cnt
        vlog(2,"RoI " + i + "/" + roi_cnt);
        roi = roi_array(i);
        sub_mask = roa_mask & roi.mask;
        
        % note: movsum crashes here for some memory reason, cellfun doesnt
        % support dimentions?
        pix_sum = zeros(1,frame_cnt);
        for j = 1:frame_cnt
            frame = sub_mask(:,:,j);
            pix_sum(j) = sum(frame(:));
        end
        
        area = sum(roi.mask(:));
        
        ts_name(i,:) = ts_name_;
        ts_uuid(i,:) = categorical({stack.uuid});
        ts_dt(i,:) = stack.dt;
        roi_id(i,:) = {roi.id};
        roi_group(i,:) = string(roi.group);
        roi_channel(i,:) = roi.channel;
        roi_mask(i,:) = {roi.mask};
        roi_area_px(i,:) = area;
        pixel_activity(i,:) = {pix_sum}; %#ok<AGROW>
        pixel_activity_pct(i,:) = {pix_sum/area}; %#ok<AGROW>
    end
    
    table_rpa = table(ts_name, ts_uuid, ts_dt, roi_id, roi_group, roi_channel, roi_mask, roi_area_px, pixel_activity, pixel_activity_pct);
end



