function dir_out = align(dir_in, dir_out, conf, new_api)
    % align alignes all StackPrarie tseries found in the specified folder. 
    %   The aligned stacks will be saved in the hdf5 format in a new directory
    % called the same as dir_in, but with '_aligned' added to the directory
    % name. 
    %   The aligned stacks can be read with begonia.stack.StackH5 .
    %
    %   dir_out = align(dir_in,dir_out,rigid)
    %
    %   dir_in      - (char) directory to align. 
    %   dir_out     - (char, default:dir_in+'_aligned') Output directory.
    %   rigid       - (logical, default:false) Use normcorre rigid alignment.
    %   dir_out     - (char) output directory, same as dir_in but with
    %                 '_aligned' added to the end of the directory name. 
    import begonia.util.logging.*;

    if nargin < 2
        dir_out = [dir_in,'_aligned'];
    end
    if nargin < 3
        conf = begonia.stack_operations.AlignmentSettings();
    end
    if nargin < 4
        new_api = false;
    end
    %ALIGN Summary of this function goes here
    %   Detailed explanation goes here
    if dir_in(end) == filesep
        dir_in(end) = [];
    end

    % assert(exist(dir_out,'dir') == 0, 'begonia:existing_directory', ...
    %     'Output folder must be empty for alignment to proceed.')
    vlog(1, 'Searching for stacks ...');
    
    if new_api
        stacks = begonia.scantype.find_by_class(dir_in,{@begonia.scantype.prairie.TSeriesPrairie});
    else  
        stacks = begonia.stack_search.find_stacks_by_class(dir_in,{@begonia.stack.StackPrarie});
    end
    
    % Create the seperate output dirs for each stack. 
    for i = 1:length(stacks)
        dirs_out{i} = fullfile(dir_out,strrep(stacks(i).path,dir_in,''));
    end

    vlog(1, fprintf('Saving output to : %s\n', dir_out));
    begonia.tseries_operations.align_stacks(stacks,dirs_out,conf)
    %begonia.stack_operations.align_stacks(stacks,dirs_out, conf);

end

