function stacks_out = denoise_stacks(varargin)
p = inputParser;
p.addRequired('stacks',...
    @(x) validateattributes(x,{'begonia.stack.Stack'},{'nonempty'}));
p.addRequired('dirs_out', ...
    @(x) validateattributes(x,{'cell','char'},{'nonempty'}));
p.addOptional('overwrite', false, ...
    @(x) validateattributes(x,{'logical'},{'nonempty'}));
p.parse(varargin{:});
begonia.util.dump_inputParser_vars_to_caller_workspace(p);

if ischar(dirs_out) %#ok<*NODEF>
    dirs_out = {dirs_out};
end

global BEGONIA_VERBOSE;

assert(length(stacks)==length(dirs_out),'Number of stacks and output paths must be equal');

stacks_out = begonia.stack.Stack.empty;

for i = 1:length(stacks)
    stack = stacks(i);
    
    begonia.path.make_dirs(dirs_out{i});
    
    for cycle = 1:stack.cycles
        for channel = 1:stack.channels
            mat = stack.get_stack(cycle,channel);
        end
    end
    
    %% Save metadata
    metadata = begonia.stack.load_default_metadata_struct(stack);
    
%     metadata.frames_in_cycle = stack.frames_in_cycle;
%     metadata.dt = stack.dt;
%     metadata.files = output_files;
    metadata.processing_state = 'denoised';
    metadata.format = 'StackH5';
    
    save(fullfile(dirs_out{i},'metadata.mat'), 'metadata');
    
    %% Copy over the xml file.
    xml_files = begonia.path.find_files(stack.path,'.xml');
    if length(xml_files) == 1
        [~,xml_filename,~] = fileparts(xml_files{1});
        xml_new = fullfile(dirs_out{i},[xml_filename,'.xml']);
        copyfile(xml_files{1},xml_new);
    end
    
    %% Get the new stack
    verb = BEGONIA_VERBOSE;
    BEGONIA_VERBOSE = 0;
    stacks_out(i) = begonia.stack.StackH5(dirs_out{i});
    BEGONIA_VERBOSE = verb;
    %% create other images.
    stacks_out(i).create_additional_images();
    %% print
    if BEGONIA_VERBOSE >= 1
        fprintf('\n');
    end
    
end


end

