function dir_out = tar(dir_in,varargin) %#ok<STOUT>
% tar finds all stacks recursivly and tar's the directories separately.
%       The output is saved in a new directory at the same location as dir_in
%   but with the name dir_in+'_tar' unless otherwise specified. 
%
%   dir_out = align(dir_in)
%   dir_out = align(...,NAME,VALUE)
%
%   REQUIRED
%   dir_in          - (char) directory to align. 
%   
%   PARAMETERS
%   dir_out         - (char,default:dir_in+'_tar') 
%                       Output directory, absolute or relative path.
%   rigid           - (logical,default:false) 
%                       false   : Normcorre non-rigid alignment.
%                       true    : Normcorre rigid alignment.
%   ignore_str      - (char,default:'') Directores containing ignore_str
%                     will not be entered when searching for stacks. This
%                     is useful in the case where stacks might contain many
%                     files that slows down the search. E.g. if a stack
%                     folder is named 'TSeries001', if input_str =
%                     'TSeries' the stack will still be found be the
%                     algorithm will not look inside 'TSeries001' for more
%                     stacks.
%
%   RETURNED
%   dir_out         - (char) output directory, same as dir_in but with
%                     '_aligned' added to the end of the directory name. 
p = inputParser;
p.addRequired('dir_in',...
    @(x) validateattributes(x,{'char'},{'nonempty'}));
p.addParameter('dir_out',[dir_in,'_tar'],...
    @(x) validateattributes(x,{'char'},{'nonempty'}));
p.addParameter('overwrite', false, ...
    @(x) validateattributes(x,{'logical'},{'nonempty'}));
p.addParameter('ignore_str','', ...
    @(x) validateattributes(x,{'char'},{}));
p.parse(dir_in,varargin{:});
begonia.util.dump_inputParser_vars_to_caller_workspace(p);

if dir_in(end) == filesep
    dir_in(end) = [];
end

disp('Searching for stacks ...')
stacks = begonia.stack_search.find_stacks(dir_in,'ignore_str',ignore_str);

% Create the seperate output paths for each stack. 
for i = 1:length(stacks)
    paths_out{i} = fullfile(dir_out,[strrep(stacks(i).path,dir_in,''),'.tar']);
end

fprintf('Saving output to : %s\n', dir_out); 
begonia.stack_operations.tar_stacks(stacks,paths_out,overwrite);

end

