function tar_stacks(varargin)
% tar_stacks packs all the stacks into .tar at the specified paths.
%
%   tar_stacks(stacks,tar_paths,overwrite)
%
%   stacks          - (1-D begonia.stack.Stack) stack objects to tar.
%   tar_paths       - (1-D cell of char or char) output path with '.tar'
%                     extension in the path.
%   overwrite       - (logical,default:false) Overwrites existing files (not
%                     directories).
p = inputParser;
p.addRequired('stacks',...
    @(x) validateattributes(x,{'begonia.stack.Stack'},{'nonempty'}));
p.addRequired('tar_paths', ...
    @(x) validateattributes(x,{'cell','char'},{'nonempty'}));
p.addOptional('overwrite', false, ...
    @(x) validateattributes(x,{'logical'},{'nonempty'}));
p.parse(varargin{:});
begonia.util.dump_inputParser_vars_to_caller_workspace(p);

if ischar(tar_paths) %#ok<*NODEF>
    tar_paths = {tar_paths};
end

assert(length(stacks)==length(tar_paths),'Number of stacks and output paths must be equal');

for i = 1:length(stacks)
    assert(strcmp(tar_paths{i}(end-3:end),'.tar'),'File must end in .tar')
end

for i = 1:length(stacks)
    stack = stacks(i);
    if exist(tar_paths{i}) == 7
        fprintf('path: %s is a folder. Skipping.\n',tar_paths{i});
        continue
    elseif exist(tar_paths{i}) ~= 0
        if overwrite
            delete(tar_paths{i});
        else
            fprintf('path: %s already exists. Skipping.\n',tar_paths{i})
            continue
        end
    end
    
    fprintf('Saving tar to: %s\n',tar_paths{i});
    tic
    begonia.path.make_dirs(tar_paths{i});
    switch computer
        case isunix
            system(sprintf('tar -P -cf %s %s', ...
                begonia.path.escape_special_chars(tar_paths{i}), ...
                begonia.path.escape_special_chars(stack.path)));
        otherwise
            tar(tar_paths{i},stack.path);
    end
    toc
    
end

end

