function dir_out = align_memmat(dir_in, dir_out, conf)
% align alignes all TSeriesPrarie found in the specified folder. 
%   The aligned stacks will be saved in the hdf5 format in a new directory
% called the same as dir_in, but with '_aligned' added to the directory
% name. 
%
%   dir_out = align(dir_in)
%   dir_out = align(...,NAME,VALUE)
%
%   REQUIRED
%   dir_in              - (char) Input directory.
%
%   PARAMETERS
%   dir_out             - (char) Default :  dir_in + '_aligned'
%                       Output directory.
%
%   conf                - (struct) Default: begonia.tseries_operations.AlignmentSettings()
%                       Struct with configuration properties for the
%                       alignment.
%
%   RETURNED
%   dir_out             - (char) same as dir_out. 
% p = inputParser;
% p.addRequired('dir_in',...
%     @(x) validateattributes(x,{'char'},{'nonempty'}));
% p.addParameter('dir_out',[dir_in,'_aligned'], ...
%     @(x) validateattributes(x,{'char'},{'nonempty'}));
% p.addParameter('conf',begonia.tseries_operations.AlignmentSettings(), ...
%     @(x) validateattributes(x,{'struct','begonia.tseries_operations.AlignmentSettings'},{}));
% p.parse(dir_in,varargin{:});
% begonia.util.dump_inputParser_vars_to_caller_workspace(p);

stacks = begonia.scantype.find_by_class(dir_in,{@begonia.scantype.prairie.TSeriesPrairie});

% Create the seperate output dirs for each stack. 
% dirs_out = {};
% for i = 1:length(stacks)
%     dirs_out{i} = fullfile(dir_out,strrep(stacks(i).path,dir_in,''));
% end

begonia.util.logging.vlog(1, sprintf('Saving output to : %s', dir_out));

begonia.tseries_operations.align_stacks_memmat(stacks,dir_out, conf);

end

