function create_movie(stack,channel,cycle,playback_speed,window,st,en)
assert(length(stack) == 1);
if nargin < 2
    channel = 1;
end
if nargin < 3
    cycle = 1;
end
if nargin < 4
    playback_speed = 10;
end
if nargin < 5
    window = ceil(playback_speed);
end
if nargin < 6
    st = [];
    en = [];
end
assert(window >= 1)

if channel > stack.channels
    return;
end
if cycle > stack.cycles
    return;
end

% Movie parameters
map = begonia.colormaps.viridis;
fps = 30;

filename = sprintf('%s_cycle_%.3d_ch%.2d_short_video.mp4',stack.name,cycle,channel);
path = fullfile(stack.path,filename);

begonia.util.logging.vlog(1,sprintf('Starting creation of %s',filename));

%% Load
begonia.util.logging.vlog(1,'Load mat');
mat = stack.get_mat(cycle,channel);

% Select time
if ~isempty(st)
    st_idx = round(st / stack.dt) + 1;
    en_idx = round(en / stack.dt) + 1;
    I = st_idx:en_idx;
    
    mat = mat(:,:,I);
    roa_mask = roa_mask(:,:,I);
else
    % The mat is usually lazy, index it to get a real matlab matrix. 
    mat = mat(:,:,:);
end

% Average window frames together
begonia.util.logging.vlog(1,'Averaging frames');
dim = size(mat);
I = 1:window:dim(3);
I(end) = [];
if isempty(I); error('Not enough frames?'); end
mat_new = zeros(dim(1),dim(2),length(I),class(mat));
for i = 1:length(I)
    mat_new(:,:,i) = mean(mat(:,:,I(i):I(i)+window-1),3);
end
mat = mat_new;
clear mat_new;

%% Movie

% Resample
frames = size(mat,3);
fs = 1 / stack.dt / window;
I = begonia.util.calculate_movie_indices(frames,fs,fps,playback_speed);
mat = mat(:,:,I);

% The number of frames changed.
frames = size(mat,3);

% Convert to rgb. 
begonia.util.logging.vlog(1,'Converting to rgb');
clim = prctile(mat(:),[2,98]);
mat_rgb = begonia.mat_functions.mat2rgb(mat,map,clim);

begonia.path.make_dirs(path);

warning off
begonia.util.logging.vlog(1,'Creating video');
v = VideoWriter(path,'MPEG-4');
v.FrameRate = fps;
v.Quality = 100;
v.open();
v.writeVideo(mat_rgb);
v.close();
warning on

end

