function backup(root,paths,path_out)
% paths are full path of the directories or files to be tarred and compressed. 
% root is the path that will serve as the root directory of the compressed
%       files.
% path_out is the output directory of the files. The tree structure in the 
%       'paths' will be duplicated under path_out relative to root.
%
% Example: 
%     path_out = '/Volumes/NIRD_NS9021K/gustavo_backup';
% 
%     root = '/Volumes/gustavo';
% 
%     paths = {};
%     paths{end+1} = '/Volumes/gustavo/FinalRed';
%     paths{end+1} = '/Volumes/gustavo/FinalDatasetProcessed';
%     paths{end+1} = '/Volumes/gustavo/AK';
%     
%     begonia.util.backup(root,paths,path_out);
%
% Output: 
%       '/Volumes/NIRD_NS9021K/gustavo_backup/FinalRed.tar.gz'
%       '/Volumes/NIRD_NS9021K/gustavo_backup/FinalDatasetProcessed.tar.gz'
%       '/Volumes/NIRD_NS9021K/gustavo_backup/AK.tar.gz'

assert(isunix,'Why are you using Windows? (This function does not work on Windows).');

setenv('PATH', '/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin');

if root(end) == filesep
    root(end) = [];
end

relative_paths = strrep(paths,[root,filesep],'');
output_paths = strrep(paths,root,path_out);
output_parent_dirs = cellfun(@fileparts,output_paths, ...
    'UniformOutput',false);
output_files = cellfun(@(x) strcat(x,'.tar.gz'),output_paths, ...
    'UniformOutput',false);

for i = 1:length(output_files)
    if exist(output_files{i},'file')
        begonia.util.logging.vlog(1,sprintf('Following path already exists %s',output_files{i}));
        continue;
    end
    if ~exist(paths{i},'file')
        begonia.util.logging.vlog(1,sprintf('Following path does not exist %s',paths{i}));
        continue;
    end
    
    begonia.path.make_dirs(output_parent_dirs{i});
    
    cmd = sprintf('tar c - -C %s %s | pigz > %s',root,relative_paths{i},output_files{i});
    begonia.util.logging.vlog(1,cmd)
    system(cmd);
end

end

