function freq_table = categorical_frequency_table(arr_1,arr_2)
% categorical_frequency_table returns the number of rows of 2 categorical arrays.
%
%   freq_table = categorical_frequency_table(cat_1,cat_2)
%
%   REQUIRED
%   cat_1               - (categorical)
%   cat_2               - (categorical)
%
%   RETURNED
%   freq_table          - (table) 
%                           Frequencies of each group in the 2 input
%                           categories.
validateattributes(arr_1,{'categorical'},{})
validateattributes(arr_2,{'categorical'},{})
cat_1 = categories(arr_1);
cat_2 = categories(arr_2);

mat = zeros(length(cat_1),length(cat_2));

for i = 1:length(cat_1)
    for j = 1:length(cat_2)
        tmp_1 = arr_1 == cat_1(i);
        tmp_2 = arr_2 == cat_2(j);
        mat(i,j) = sum(tmp_1 & tmp_2);
    end
end

freq_table = array2table(mat);
freq_table.Properties.VariableNames = cat_2;
freq_table.Properties.RowNames = cat_1;

end

