%{
Function that applies a function to a matrix in multiple columns down the
third axis. "chunks-xy-func".
%}
function result = chunkxyfun(func, mat, divisions, overlap)
    import begonia.util.logging.*;
    
    result = [];

    
    if divisions < 2
        vlog(2, '(chunkxyfun) No divisions - result = func(matrix(:,:,:))');
        result = func(mat(:,:,:));
        return;
    end
    
    % how large are the chunks?
    width = size(mat,1);
    height = size(mat,2);
    xdiv_len = ceil(width/divisions);
    ydiv_len = ceil(height/divisions);
    
    % loop over divisions to collect result matrix:
    c = 1;
    for xdiv = 1:divisions
        x_start = 1 + (xdiv - 1) * xdiv_len;
        x_range = x_start:x_start + (xdiv_len - 1);
        for ydiv = 1:divisions
            tic;
            vlog(1, ['(chunkxyfun) Chunk ' num2str(c) '/' num2str(divisions^2)]); 
            
            y_start = 1 + (ydiv - 1) * ydiv_len;
            y_range = y_start:y_start + (ydiv_len - 1);
            xr_padded = padd(x_range, overlap, xdiv, divisions);
            yr_padded = padd(y_range, overlap, ydiv, divisions);
            
            vlog(2, '(chunkxyfun) Extracting chunk matrix'); 
            submat = double(mat(xr_padded, yr_padded, :));
            
            vlog(2, '(chunkxyfun) Running output function');
            func_out = func(submat);
            
            % establish output matrix on first result:
            if isempty(result)
                if size(func_out,3) == 1
                    result = repmat(func_out(1,1), width, height);
                else
                    result = repmat(func_out(1,1,1), size(mat));
                end
            end
            
            % extract result:
            [xr_unpad, yr_unpad] = unpadd(size(submat) ...
                , overlap ...
                , xdiv ...
                , ydiv ...
                , divisions);
            
            result(x_range,y_range,:) = func_out(xr_unpad, yr_unpad,:);
            c = c + 1;
        end
    end
end


function range = padd(range, overlap, n, d)
    
    if n == 1
        % first only has paddign on end:
        range = [range ...
            range(end)+1:range(end)+1+(overlap-1) ...
            ];
    elseif n == d
        % last only has padding on beginning:
        range = [range(1)-overlap:1:range(1)-1 ...
            range ...
            ];
    else
        range = [range(1)-overlap:1:range(1)-1 ...
            range ...
            range(end)+1:range(end)+1+(overlap-1) ...
            ];
    end
end


function [xr, yr] = unpadd(matsize, overlap, xdiv, ydiv, dim)
    % x range:
    if xdiv == 1
        xr = 1:matsize(1) - overlap;
    elseif xdiv == dim
        xr = overlap+1:matsize(1);
    else
        xr = overlap+1:matsize(1) - overlap;
    end
    
    % y range:
    if ydiv == 1
        yr = 1:matsize(2) - overlap;
    elseif ydiv == dim
        yr = overlap+1:matsize(2);
    else
        yr =  overlap+1:(matsize(2) - overlap);
    end
    
end




