function compress_dirs(output_path,root,dirs)
% output_path - Destination path. 
% dirs - Cell array of the directories that should be compressed. 
% root - The data will be uncompressed to paths relative to this path. It
%       is important that root is a part of all the paths in 'dirs'.
% 
% Skipps compression if the output (.tar.gz) file already exists. 

% Set some environment variables, some fix to have access to tar ang pigz
% in the terminal. 
setenv('PATH', '/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin');

if ischar(dirs)
    dirs = {dirs};
end
assert(iscellstr(dirs));

if root(end) == filesep
    root(end) = [];
end

if output_path(end) == filesep
    output_path(end) = [];
end

relative_dirs = strrep(dirs,[root,'/'],'');
output_dirs = strrep(dirs,root,output_path);
output_parent_dirs = cellfun(@fileparts,output_dirs, ...
    'UniformOutput',false);
output_files = cellfun(@(x) strcat(x,'.tar.gz'),output_dirs, ...
    'UniformOutput',false);

%%
for i = 1:length(output_files)
    if exist(output_files{i},'file')
        continue;
    end
    
    begonia.path.make_dirs(output_parent_dirs{i});
    
    cmd = sprintf('tar c - -C %s %s | pigz > %s',root,relative_dirs{i},output_files{i});
    begonia.util.logging.vlog(1,cmd)
    system(cmd);
end

end

