function create_additional_images(stacks)
% create_additional_images saves some summary images of the stack.
%   Creates and saves summary images such as average and std. Used mainly
%   by the roimanager. 
%
%   create_additional_images(stacks)
%
%   stacks      - (array of begonia.stack.Stack) Stacks to create images
%                 of. 
for stack = stacks
    fprintf('Calculating images for %s\n',stack.name);
    pixel_min_max = {};
    for cycle = 1:stack.cycles
        for channel = 1:stack.channels
            fprintf('Calculating cycle %d channel %d.\n',cycle,channel);
            mat = stack.get_stack(cycle,channel);
            % Make that mat into a matlab matrix if it is lazy.
            mat = single(mat(:,:,:));
            
            img_avg = mean(mat,3);
            img_std = std(mat,0,3);
            img_max = max(mat,[],3);
            
            pixel_min_max{cycle,channel} = [min(mat(:)),max(mat(:))];
            
            file = sprintf('avg_cycle_%.3d_channel_%.2d.tif',cycle,channel);
            file = fullfile(stack.path,file);
            begonia.util.mat2tiff(uint16(img_avg),file); %#ok<*USENS>
            
            file = sprintf('std_cycle_%.3d_channel_%.2d.tif',cycle,channel);
            file = fullfile(stack.path,file);
            begonia.util.mat2tiff(uint16(img_std),file);
            
            file = sprintf('max_cycle_%.3d_channel_%.2d.tif',cycle,channel);
            file = fullfile(stack.path,file);
            begonia.util.mat2tiff(uint16(img_max),file);
        end
    end
      
    % Append  to metadata file if it exists.
    file = 'metadata.mat';
    file = fullfile(stack.path,file);
    if exist(file,'file')
        tmp = load(file);
        metadata = tmp.metadata;
    else
        metadata = struct;
    end
    metadata.pixel_min_max = pixel_min_max;
    save(file, 'metadata');
    
    fprintf('Finished \n');
end
end

