classdef StackH5 < begonia.stack.Stack
    %StackH5 handles tseries saved as hdf5. 
    
    properties (Access = private)
        files
    end
    
    methods
        function obj = StackH5(path)
            obj@begonia.stack.Stack(path);
            
            metadata = begonia.stack.load_default_metadata_struct(obj);

            % Get the files of each cycle and channel.
            assert(isfield(metadata,'files'), ...
                'begonia:invalid_stack:missing_struct_var', ...
                'metadata struct is missing ''files'' variable.');
            obj.files = cellfun(@(filename) fullfile(obj.path,filename), ...
                metadata.files, 'UniformOutput',false);
            
            assert(strcmp(metadata.format,'StackH5'), ...
                'begonia:invalid_stack:wrong_format', ...
                'Expected a spesific hdf5 format.');
            
            global BEGONIA_VERBOSE
            if BEGONIA_VERBOSE >= 1
                fprintf('Found StackH5: %s\n',obj.path);
            end
            
        end
        
        function mat = get_stack(self,cycle,channel)
            file = self.files{cycle,channel};
            mat = begonia.util.h5arr(file);
        end
        
    end
    
end

