classdef StackOldV1 < begonia.stack.Stack
    % Loads stacks of the first version of our alignment program. 
    
    properties (Access = private)
        files
    end
    
    methods
        function obj = StackOldV1(path)
            obj@begonia.stack.Stack(path);

            assert(exist(fullfile(obj.path,'sima-0.3.2.sh'),'file') == 2, ...
                'begonia:invalid_stack:missing_sima_file', ...
                'Sima file not found, used as an indicator for StackV1.');
            
            xml_files = begonia.path.find_files(obj.path,'.xml',false);
            assert(~isempty(xml_files),'begonia:invalid_stack:missing_xml','Missing xml file.');
            
            % Parse the xml file.
            xml_file = xml_files{1};
            meta = begonia.stack.StackPrarie.parse_xml_file_fast(xml_file);
            
            assert(contains(meta.type,'TSeries'), ...
                'begonia:invalid_stack:unsupported_type', ...
                'Only supports TSeries as of now.');
            
            obj.type = meta.type;
            obj.dx = meta.dx;
            obj.dy = meta.dx;
            obj.dt = meta.dt;
            obj.cycles = meta.cycles;
            % This format is missing the last frame.
            obj.frames_in_cycle = meta.frames_in_cycle - 1;
            
            obj.channels = meta.channels;
            obj.record_date = meta.date;
            obj.channel_names = meta.channel_names;
            
            name = strsplit(path,filesep);
            name = name{end};
            % This is a hotfix, not needed in most cases, but probably wont
            % hurt. 
            name = regexprep(name,'-ali.*','');
            obj.name = name;
            
            % All stacks of this kind were created using sima.
            obj.processing_state = 'aligned_sima_v1';
            obj.format = 'stack_v1';
            
            % Find the correct multipage tiff files based on cycle and
            % channel.
            tif_files = begonia.path.find_files(obj.path,'.tif',false);
            for cycle = 1:obj.cycles
                for channel = 1:obj.channels
                    % Get the channel number from the channel name.
                    channel_num = regexp(obj.channel_names{channel},'\d*','Match');
                    channel_num = str2num(channel_num{1});
                    % Find files ending in 'cycleXXX_chXX.tif'
                    match = contains(tif_files,sprintf('cycle%03d_ch%02d.tif',cycle,channel_num));
                    assert(sum(match) == 1, ...
                        'begonia:invalid_stack:tif_mismatch', ...
                        'Missing or wrong tif files.');
                    files{cycle,channel} = tif_files{match};
                end
            end
            obj.files = files;

            % The original image dimentions are not stored in this format.
            obj.original_img_dim = [];
            
            global BEGONIA_VERBOSE
            if BEGONIA_VERBOSE >= 1
                fprintf('Found stack v.1 (multipage tiff): %s\n',obj.path);
            end
            
        end
        
        function mat = get_stack(self,cycle,channel)
            file = self.files{cycle,channel};
            mat = TIFFStack(file);
        end
        
    end
    
end

