function P1 = fft_one_side(mat, padding)
% fft_one_side gives the one sided fft along the columns of a mat. 
%   If the mat is a 1xN vector the mat will be transposed before the
% operation.
%   The frequencies beloning to the fft coefficients (P1) are: 
%       f = (0:floor(N/2))*sampling_frequency/N/padding;
%
%   P1 = fft_one_side(mat,padding)
% 
%   mat             - (NxM numerical) input array.
%   padding         - (integer, default:1) The factor of zero padding. For
%                     example if padding=2 the fft will run on the input
%                     mat with a shape of 2*N x M matrix, where the end of
%                     columns are padded with zero. The output 'resolution'
%                     of P1 would then be doubled. 
%   P1              - (floor(N/2)xM double) fft output.
if size(mat,1) == 1
    mat = mat';
end
if nargin < 2
    padding = 1;
end
padding = size(mat,1)*padding;
% Do the fft along each column. 
Y = fft(mat,padding);
N = size(Y,1);
P2 = abs(Y/N);
P1 = P2(1:floor(N/2)+1,:);
P1(2:end-1,:) = 2*P1(2:end-1,:);

%f = sampling_freq*(0:(N/2))/N/padding;

end

