
%% Save and load
% save and store variables on a data location (any directory path)
path = tempname();
mkdir(path);
dloc = begonia.data_management.DataLocation(path);

var1 = "!UNITTEST";
var2 = 43545;
var3 = struct();
var3.data = "demo";

dloc.save_var('first', var1);
dloc.save_var('second', var2);
dloc.save_var('third', var3);

restored_1 = dloc.load_var('first');
restored_2 = dloc.load_var('second');
restored_3 = dloc.load_var('third');

% values put in shold be restored:
assert(restored_1 == var1);
assert(restored_2 == var2);
assert(restored_3.data == var3.data);


%% List vars
%  dloc.saved_vars contains all save variables as a list
path = tempname();
mkdir(path);
dloc = begonia.data_management.DataLocation(path);

var1 = "!UNITTEST";
var2 = 43545;
var3 = struct();
var3.data = "demo";

dloc.save_var('first', var1);
dloc.save_var('second', var2);
dloc.save_var('third', var3);


expected_vars = {'first', 'second', 'third'};
vars = dloc.saved_vars;
assert(isempty(setdiff(dloc.saved_vars, expected_vars)));


%% Clear variable
% clearing a variable removes the data from disk:
path = tempname();
mkdir(path);
dloc = begonia.data_management.DataLocation(path);

% clear a single varilabe:
assert(isempty(dloc.saved_vars));

var1 = "!UNITTEST";
dloc.save_var('first', var1);

assert(length(dloc.saved_vars) == 1);

dloc.clear_var('first');
assert(isempty(dloc.saved_vars));

%% Copy from 
import begonia.data_management.*
path_1 = tempname(); mkdir(path_1);
path_2 = tempname(); mkdir(path_2);

dloc_1 = DataLocation(path_1);  
dloc_2 = DataLocation(path_2);

% save some data:
dloc_1.save_var('alpha', pi);   % not in 2
dloc_1.save_var('beta', "dloc1");

dloc_2.save_var('gamma', "second"); % not in 1
dloc_2.save_var('beta', "dloc2");

% assert that is correct:
assert(contains("alpha", dloc_1.saved_vars));
assert(contains("beta", dloc_1.saved_vars));
assert(~contains("gamma", dloc_1.saved_vars));

assert(contains("gamma", dloc_2.saved_vars));
assert(contains("beta", dloc_2.saved_vars));

% do the copy:
dloc_1.copy_from(dloc_2, false);

% assert all vars have arrived, and that alpha and beta did not get
% overwritten:
assert(contains("alpha", dloc_1.saved_vars));
assert(contains("beta", dloc_1.saved_vars));
assert(contains("gamma", dloc_1.saved_vars));

assert(dloc_1.load_var('beta') == "dloc1"); 
assert(dloc_1.load_var('alpha') == pi); 


% do again, but with overwrite:
% this should change the value of 'beta', was existed on both:
dloc_1.copy_from(dloc_2, true);
assert(dloc_1.load_var('beta') == "dloc2"); 


%% Changelog
import begonia.data_management.*
path_1 = tempname(); mkdir(path_1);
path_2 = tempname(); mkdir(path_2);

dloc_1 = DataLocation(path_1);  
dloc_2 = DataLocation(path_2);

assert(length(dloc_1.dl_changelog) == 0);
assert(length(dloc_2.dl_changelog) == 0);

% save two vars:
dloc_1.save_var('first_var', 'first');
assert(length(dloc_1.dl_changelog) == 1);
pause(1/100);
dloc_2.save_var('first_var', 'last');
dloc_2.save_var('second_var', 42);
assert(length(dloc_2.dl_changelog) == 1);

% validate times are correct (difficult, but dloc_2 is newer:
assert(dloc_2.dl_changelog('first_var') > dloc_1.dl_changelog('first_var'));

% sync: dloc 2 should overwrite 1, but not the other way around:
assert(dloc_2.load_var('first_var') == "last");
dloc_2.dl_update_from(dloc_1);
assert(dloc_2.load_var('first_var') == "last");

assert(dloc_1.load_var('first_var') == "first");
assert(~dloc_1.has_var('second_var'));

% this should overwrite 'first_var', and include 'second var'
dloc_1.dl_update_from(dloc_2);
assert(dloc_1.load_var('first_var') == "last");
assert(dloc_1.load_var('second_var') == 42);


