%{
The dependency manager is used to avoid old versions of data being used in
calculation by mistake. Take this example:

Scripts A generates a table. This tables is used by script B which
generates another table. Script C then generates a table C, so that:
A.xlsx -> B.xlsx -> C.xlsx.

If a bug is discovered in A, and A regenerated, but the user forgets to
run the second script, the changes from A are not in the output.

This example was rather trivial as it is fully linear, but it is easy to
forget to update all your data when your scripts come to a certain level of
complexity.

The behaviour we want in the scenario above is that C warns or stops if
table B is not update, or alternativly automaticlaly updates table B when 
C is requested. 

Automatic updates would be very complex. To keep concernt separated, the
Dependency Manager only keeps track of issue, but does not fx problems.

Basic methcniams:
- A "product" is the name of a script og function's outpout
- Scripts call update(prod) to register an update output
- Scripts check if all dependent products are up to date using check_valid(prods) 

Underlying mechanism:

depend_on() registers a dependency

%}

% initilialize new script

begonia.util.logging.set_global_level(2);

% load a depfile - will be created if it does not exist:
tdir = tempname(tempdir);
mkdir(tdir);
depfile = fullfile(tdir, 'unittest.mat');
depman = begonia.data_management.DependencyManager(depfile);

% simulation 1 : first script creates A:
%depman.update("A");

% simulation 2 : second script creates B using A:
depman.connect("B needs A");
depman.update("B");

% siluation 3 : third script does the same as seconed:
PROD = "C";  % product
depman.connect(PROD + " needs A");
depman.update(PROD);


























