function h5_transform(tseries,root)

if root(end) == filesep
    root(end) = [];
end

alignment_directoy = tseries.path;
alignment_directoy = strrep(alignment_directoy,root,[root,'_h5']);

if alignment_directoy(end-3:end) == '.tif'
    alignment_directoy(end-3:end) = [];
end

%%

begonia.util.logging.vlog(1, sprintf('Starting transfer of %s',tseries.path));

tic

% Delete previous folders. 
if exist(alignment_directoy,'dir')
    rmdir(alignment_directoy,'s');
end

metadata = begonia.scantype.h5.create_metadata(tseries);

% Align
for cycle = 1:tseries.cycles
    alignment_channel = 1;

    begonia.util.logging.vlog(1, 'Loading matrix');
    mat = tseries.get_mat(cycle,alignment_channel);

    % These stacks tend to be lazy and not actually matlab matricies.
    mat = mat(:,:,:);
    mat = uint16(mat);

    % Create the output folder.
    begonia.path.make_dirs(alignment_directoy);

    % Save as hdf5.
    begonia.util.logging.vlog(1, 'Saving matrix');
    path_h5 = fullfile(alignment_directoy,metadata.files{cycle,alignment_channel});
    mat_lazy = begonia.util.h5arr(path_h5,size(mat),'uint16');
    mat_lazy(:,:,:) = mat;

    % Shift and save the other channels.
    channels = 1:tseries.channels;
    % Ignore the channel that was just aligned. 
    channels(alignment_channel) = [];
    for channel = channels
        begonia.util.logging.vlog(1, 'Loading matrix');
        mat = tseries.get_mat(cycle,channel);
        mat = mat(:,:,:);
        mat = uint16(mat);

        % Save as hdf5.
        path_h5 = fullfile(alignment_directoy,metadata.files{cycle, channel});
        begonia.util.logging.vlog(1, 'Saving matrix');
        mat_lazy = begonia.util.h5arr(path_h5,size(mat),'uint16');
        mat_lazy(:,:,:) = mat;
    end
end

% Save metadata:
begonia.util.logging.vlog(2, 'Saving metadata');
save(fullfile(alignment_directoy,'metadata.mat'), 'metadata');

% Get the new stack and create other images.
begonia.util.logging.vlog(1, 'Creating reference images');
stack_out = begonia.scantype.h5.TSeriesH5(alignment_directoy);
for cy = 1:tseries.cycles
    for ch = 1:tseries.channels
        stack_out.get_avg_img(cy,ch);
        stack_out.get_std_img(cy,ch);
        stack_out.get_max_img(cy,ch);
    end
end

alignment_time = toc;
str = sprintf('Transfer completed in %d seconds.',alignment_time);
begonia.util.logging.vlog(1,str);

end

