classdef BackupTool < handle
    %BACKUPTOOL Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        editor
        config
        dlocs
    end
    
    methods
        
        function obj = BackupTool(sheet_path)
            if nargin < 1
                [sheet, path] = uigetfile({'*.xlsx';'*.xls';'*.csv'}, 'Setup sheet');
                if sheet == 0mo
                    error('Must provide a setup sheet');
                end
                sheet_path = fullfile(path, sheet);
            end

            % load config:
            obj.config = xylobium.backuptool.BackupToolConfig(sheet_path);
            obj.loadDlocs();

            % setup editor:
            actions = obj.setupActions();
            modifiers = xylobium.backuptool.NameExtractModifier('name');
            
            obj.editor = xylobium.backuptool.BackupToolEditor(obj.dlocs ...
                , actions ...
                , {'name', 'path', 'compressed', 'transmitted', 'backup_done', '!note'} ...
                , modifiers);
            obj.editor.config = obj.config;
            obj.editor.datagrid.table.ColumnWidth = {200, 100,70,70};
        end
        
        
        % loads the datalocations from the setup sheet
        function loadDlocs(obj)
            obj.dlocs = begonia.data_management.DataLocation.empty;
            for path = obj.config.backup_paths'
                dloc = begonia.data_management.DataLocation(char(path));
                obj.dlocs = [obj.dlocs dloc];
            end
        end
        
        % set up default actions
        function actions = setupActions(obj)
            ac1 = xylobium.dledit.Action('1) Compress' ...
                , @(d, m, e) xylobium.backuptool.actions.compressZip(d, m, e), true, false);
            
            ac2 = xylobium.dledit.Action('2) Transmit' ...
                , @(d, m, e) xylobium.backuptool.actions.transmit(d, m, e), true, false);
            
            ac3 = xylobium.dledit.Action('3) Remove compressed' ...
                , @(d, m, e) xylobium.backuptool.actions.removeCompressed(d, m, e), true, false);
            
            actions = [ac1 ac2 ac3];
            
        end
        
    end
end

