classdef DualSlider < handle
    
    properties (SetObservable,AbortSet)
        parent
        
        value
        limits
        string
    end
    
    properties (Access = private)
        main_graphics_object
        
        text
        text_left
        text_right
        jslider
    end
    
    methods
        
        function set.parent(self,parent)
            if ~isempty(self.main_graphics_object)
                delete(self.main_graphics_object)
            end
            
            if isempty(parent)
                self.main_graphics_object = [];
                self.parent = [];
            else
                %%
                vbox = uix.VBox('Parent',parent);
                
                self.parent = parent;
                self.main_graphics_object = vbox;
                %%
                text = uix.Text('Parent',vbox);
                text.FontSize = 12;
                self.text = text;
                %%
                sub_box = uix.HBox('Parent',vbox);
                text_left = uix.Text('Parent',sub_box);
                text_right = uix.Text('Parent',sub_box);
                text_left.HorizontalAlignment = 'left';
                text_right.HorizontalAlignment = 'right';
                self.text_left = text_left;
                self.text_right = text_right;
                %%
                jslider = com.jidesoft.swing.RangeSlider();
                [jslider,jslider_container] = javacomponent(jslider);
                jslider_container.Parent = vbox;
                jslider.Minimum = 0;
                jslider.Maximum = 1000;
                jslider.Low = 0;
                jslider.High = 1000;
                jslider_callback_handle = handle(jslider);
                jslider_callback_handle.MouseDraggedCallback = @(~,~) self.callback_jslider();
                self.jslider = jslider;
                %%
                vbox.Heights = [20,20,20];
            end
            self.reset();
        end
        
        function reset(self)
            self.limits = [];
            self.value = [];
            self.string = 'DualSlider';
        end
        
        function set.string(self,value)
            self.string = value;
            
            if isempty(self.parent)
                return;
            end
            
            if ~isempty(self.text)
                self.text.String = value;
            end
        end
        
        function set.value(self,value)
            len = length(value);
            assert(len == 0 || len == 2);
            self.value = value;
            
            self.update_slider_positions();
        end
        
        function set.limits(self,limits)
            len = length(limits);
            assert(len == 0 || len == 2);
            self.limits = limits;
            
            if isempty(self.parent)
                return;
            end
            
            if isempty(limits)
                self.text_left.String = 'N/A';
                self.text_right.String = 'N/A';
            else
                self.text_left.String = num2str(limits(1));
                self.text_right.String = num2str(limits(2));
                
                self.update_slider_positions();
            end
        end
    end
    
    methods (Access = private)
        function update_slider_positions(self)
            
            if isempty(self.parent)
                return;
            end
            
            if isempty(self.value) || isempty(self.limits)
                self.jslider.Low = 0;
                self.jslider.High = 1000;
                return;
            end
            
            ratio = (self.value - self.limits(1))/(self.limits(2)-self.limits(1));
            slider_val = round(ratio*1000);
            if any(slider_val < 0) || any(slider_val > 1000)
                warning('Slider value outside range.')
            end
            self.jslider.Low = slider_val(1);
            self.jslider.High = slider_val(2);
        end
        
        function callback_jslider(self)
            limits = self.limits;
            if isempty(limits)
                warning('Slider moved but no min/max limits are not set.')
                return;
            end
            slider_value = [self.jslider.Low,self.jslider.High];
            slider_value = round(slider_value);
            ratio = slider_value/1000;
            value = ratio*(limits(2)-limits(1)) + limits(1);
            self.value = value;
        end
    end
end

