classdef MovingRois < handle
    
    properties (SetObservable,AbortSet)
        parent
        
        roi_array_model
        axes
        frame
    end
    
    properties (Access = private)
        container
        listbox
        
        roi_plot_handles
        
        btn_save_rois
        btn_clear_offset
        
        listener_roi_array_model
        
        mouse_down = false;
        mouse_down_pos
    end
    
    methods
        function set.parent(self,parent)
            delete(self.container)
            
            self.parent = parent;
            
            if isempty(parent)
                return;
            end
            %% Container
            vbox = uix.VBox('Parent',parent);
            self.container = vbox;
            %%
            hbox = uix.HBox('Parent',vbox);
            save_rois = uicontrol('Parent',hbox,'Style','pushbutton');
            save_rois.String = 'Save ROIs';
            self.btn_save_rois = save_rois;
            clear_offset = uicontrol('Parent',hbox,'Style','pushbutton');
            clear_offset.String = 'Clear Offset';
            self.btn_clear_offset = clear_offset;
            %% text
            text = uix.Text('Parent',vbox);
            text.VerticalAlignment = 'middle';
            text.String = 'Offset Frames';
            text.FontSize = 14;
            %% listbox
            listbox = uitable(vbox);
            listbox.ColumnName = {'Frame','x','y'};
            listbox.CellSelectionCallback = @self.callback_uitable;
            self.listbox = listbox;
            %% other
            vbox.Heights = [20,20,-1];
            %%
            self.update_roi_plot_handles();
        end
        
        function set.roi_array_model(self,roi_array_model)
            delete(self.listener_roi_array_model);
            
            self.roi_array_model = roi_array_model;
            
            if isempty(roi_array_model)
                self.create_roi_plot_handles();
                self.btn_save_rois.Callback = [];
                self.btn_clear_offset.Callback = [];
            else
                self.listener_roi_array_model = roi_array_model.addlistener('roi_array','PostSet',@(~,~) self.create_roi_plot_handles());
                self.listener_roi_array_model(2) = roi_array_model.addlistener('selected_rois','PostSet',@(~,~) self.update_roi_plot_handles());
                
                self.btn_save_rois.Callback = @(~,~) roi_array_model.save_rois();
                self.btn_clear_offset.Callback = @self.btn_clear_offset_callback;
                
                self.create_roi_plot_handles();
            end
        end
        
        function set.axes(self,axes)
            % Reset previous figure mouse callbacks
            f = ancestor(self.axes,'figure','toplevel');
            f.WindowButtonMotionFcn = [];
            f.WindowButtonUpFcn = [];
            f.WindowButtonDownFcn = [];
            
            self.axes = axes;
            
            % Hijack figure mouse callbacks
            f = ancestor(self.axes,'figure','toplevel');
            f.WindowButtonMotionFcn = @(~,~) self.callback_mouse_move();
            f.WindowButtonUpFcn = @(~,~) self.callback_mouse_up();
            f.WindowButtonDownFcn = @(~,~) self.callback_mouse_down();
            
            self.create_roi_plot_handles();
        end
        
        function set.frame(self,frame)
            self.frame = frame;
            self.update_roi_plot_handles();
        end
        
    end
    
    methods (Access = private)
        
        
        function btn_clear_offset_callback(self,s,e)
            if isempty(self.roi_array_model)
                return;
            end
            self.roi_array_model.clear_offsets(self.roi_array_model.selected_rois);
            self.update_roi_plot_handles();
        end
        
        function callback_uitable(self,s,e)
            if ~isempty(self.listbox.Data) && ~isempty(e.Indices)
                frame = self.listbox.Data(e.Indices(1),1);
                self.frame = frame;
            end
        end
        
        function point = get_mouse_coordinates_in_axes(self)
            % Get mousepoint coordinates in ax and figure units
            ax = self.axes;
            point = ax.CurrentPoint(1,1:2);
            point = round(point);
            
            % Return if outside axis.
            if point(1) < ax.XLim(1) || ...
                    point(1) > ax.XLim(2) || ...
                    point(2) < ax.YLim(1) || ...
                    point(2) > ax.YLim(2)
                point = [];
                return
            end
        end
        
        function callback_mouse_down(self)
            self.mouse_down = true;
            
            point = self.get_mouse_coordinates_in_axes();
            frame = self.frame;
            
            self.mouse_down_pos = point;
            
            if ~isempty(point)
                self.roi_array_model.selected_rois = self.roi_array_model.get_rois_at_point(point,frame);
            end
        end
        
        function callback_mouse_up(self)
            self.mouse_down = false;
            
            point = self.get_mouse_coordinates_in_axes();
            
            if isempty(point) || isempty(self.mouse_down_pos)
                return;
            end
            
            r = self.roi_array_model;
            
            selected_rois = r.selected_rois;
            
            if ~isempty(selected_rois) && ~isempty(self.frame)
                offset = point - self.mouse_down_pos;

                rois = r.roi_array(selected_rois);
                rois.insert_xy_offset(self.frame,offset(1),offset(2));
            end
            
            self.update_roi_plot_handles();
        end
        
        function callback_mouse_move(self)
            point = self.get_mouse_coordinates_in_axes();
            if isempty(point) 
                self.callback_mouse_up();
                return;
            end
            
            selected_rois = self.roi_array_model.selected_rois;
            
            if self.mouse_down && ~isempty(selected_rois)
                self.update_roi_plot_handles();
            end
            
        end
        
        function create_roi_plot_handles(self)
            %% Delete the old handles
            for i = 1:length(self.roi_plot_handles)
                if isvalid(self.roi_plot_handles(i))
                    delete(self.roi_plot_handles(i));
                end
            end
            
            self.roi_plot_handles = gobjects(0);
            
            if isempty(self.roi_array_model) || isempty(self.axes)
                return;
            end
            %% Create new plot (handles)
            
            r = self.roi_array_model;
            
            ax = self.axes;
            hold(ax,'on');
            for i = 1:length(r.roi_array)
                self.roi_plot_handles(i) = plot( ...
                    r.roi_array(i).plot_boundary(:,1), ...
                    r.roi_array(i).plot_boundary(:,2), ...
                    'Parent',ax);
                self.roi_plot_handles(i).LineWidth = 2;
            end
            %%
            self.update_roi_plot_handles();
        end
        
        function update_roi_plot_handles(self)
            %% Roi positions
            
            r = self.roi_array_model;

            if ~isempty(r) && ~isempty(self.axes)
                %% Color
                for i = 1:length(r.roi_array)
                    roi = r.roi_array(i);
                    if ismember(i,r.selected_rois)
                        if ~isempty(roi.xy_offsets) && ismember(self.frame,roi.xy_offsets(:,1))
                            color = 'red';
                        else
                            color = 'white';
                        end
                    else
                        color_idx = strcmp(roi.group,r.ROI_GROUPS);
                        color = r.ROI_COLORS{color_idx};
                    end
                    self.roi_plot_handles(i).Color = color;
                end

                %% Position
                for i = 1:length(r.roi_array)
                    xdata = r.roi_array(i).plot_boundary(:,1);
                    ydata = r.roi_array(i).plot_boundary(:,2);

                    if ~isempty(r.roi_array(i).xy_offset_vector) && ~isempty(self.frame)
                        xdata = xdata + r.roi_array(i).xy_offset_vector(self.frame,1);
                        ydata = ydata + r.roi_array(i).xy_offset_vector(self.frame,2);
                    end

                    if self.mouse_down && ismember(i,r.selected_rois)
                        point = self.get_mouse_coordinates_in_axes();
                        xdata = xdata + point(1) - self.mouse_down_pos(1);
                        ydata = ydata + point(2) - self.mouse_down_pos(2);
                    end

                    self.roi_plot_handles(i).XData = xdata;
                    self.roi_plot_handles(i).YData = ydata;
                end
            end
            
            %% UI
            if ~isempty(self.parent)
                if isempty(r) || isempty(r.selected_rois)
                    self.listbox.Data = [];
                else
                    i = r.selected_rois(1);
                    data = r.roi_array(i).xy_offsets;
                    if ~isequal(self.listbox.Data,data)
                        self.listbox.Data = data;
                    end
                end
            end
        end
    end
end

