classdef RoaDisplay < handle
    
    properties (SetObservable)
        roa_mask
        frame
        video_axes
        alpha = 0.6
    end
    
    properties (SetAccess = private)
        ready
    end
    
    properties (Access = private)
        img
        image_handle
    end
    
    methods
        function val = get.ready(self)
            val = false;
            if isempty(self.roa_mask) || isempty(self.frame) || isempty(self.video_axes) || isempty(self.alpha)
                return;
            end
            
            if self.frame < 1 || self.frame > size(self.roa_mask,3)
                return;
            end
            val = true;
        end
        
        function set.roa_mask(self,roa_mask)
            self.roa_mask = roa_mask;
            color = [1,0,0];
            self.img = ones(size(roa_mask,1),size(roa_mask,2),3);
            self.img(:,:,1) = color(1);
            self.img(:,:,2) = color(2);
            self.img(:,:,3) = color(3);
            self.update();
        end
        
        function set.frame(self,frame)
            self.frame = round(frame);
            self.update();
        end
        
        function set.video_axes(self,video_axes)
            self.video_axes = video_axes;
            self.update();
        end
        
        function set.alpha(self,alpha)
            self.alpha = alpha;
            self.update();
        end
    end
    
    methods (Access = private)
        function update(self)
            if self.ready
                if isempty(self.image_handle)
                    hold(self.video_axes,'on');
                    self.image_handle = imshow(self.img,'Parent',self.video_axes);
                    uistack(self.image_handle,'top');
                end
                self.image_handle.Parent = self.video_axes;
                if ~isequal(size(self.img),size(self.image_handle.CData))
                    self.image_handle.CData = self.img;
                end
                self.image_handle.AlphaData = self.roa_mask(:,:,self.frame)*self.alpha;
            else
                if ~isempty(self.image_handle)
                    delete(self.image_handle);
                    self.image_handle = [];
                end
            end
        end
    end
end

