classdef RoaPlotPanel < handle
    
    properties (SetObservable,AbortSet)
        parent
        tseries_video
    end
    
    properties (Access = private)
        container
        frame_field_1
        frame_field_2
    end
    
    methods
        function set.parent(self,parent)
            delete(self.container)
            
            self.parent = parent;
            
            self.frame_field_1 = [];
            self.frame_field_2 = [];
            
            if isempty(parent)
                return;
            end
            %% vbox
            vbox = uix.VBox('Parent',parent);
            self.container = vbox;
            %% text
            text = uix.Text('Parent',vbox);
            text.VerticalAlignment = 'middle';
            text.String = 'ROA 3D';
            text.FontSize = 14;
            %%
            hbox = uix.HBox('Parent',vbox);
            
            text = uix.Text('Parent',hbox);
            text.VerticalAlignment = 'middle';
            text.String = 'Start Frame';
            text.FontSize = 14;
            
            text = uix.Text('Parent',hbox);
            text.VerticalAlignment = 'middle';
            text.String = 'End Frame';
            text.FontSize = 14;
            %%
            hbox = uix.HBox('Parent',vbox);
            frame_1 = uicontrol('Parent',hbox,'Style','edit');
            frame_2 = uicontrol('Parent',hbox,'Style','edit');
            self.frame_field_1 = frame_1;
            self.frame_field_2 = frame_2;
            %%
            roa_btn = uicontrol('Parent',vbox,'Style','pushbutton');
            roa_btn.Callback = @self.create_3d_plot;
            roa_btn.String = 'Create 3D Plot';
            %%
            roa_btn = uicontrol('Parent',vbox,'Style','pushbutton');
            roa_btn.Callback = @self.create_outline_movie;
            roa_btn.String = 'Create outline movie';
            %%
            roa_btn = uicontrol('Parent',vbox,'Style','pushbutton');
            roa_btn.Callback = @self.create_movie;
            roa_btn.String = 'Create movie';
            %% other
            vbox.Heights = [20,20,20,20,20,20];
            
        end
        
        function create_movie(self,~,~)
            if isempty(self.tseries_video) || ~self.tseries_video.ready
                return;
            end
            
            if isempty(self.frame_field_1) || isempty(self.frame_field_2)
                return;
            end
            
            frame_1 = str2double(self.frame_field_1.String);
            frame_2 = str2double(self.frame_field_2.String);
            
            if isnan(frame_1) || isnan(frame_2) || frame_2 < frame_1
                return;
            end
            %%
            a = self.tseries_video.video_axes;
            
            x_lim = round(a.XLim);
            y_lim = round(a.YLim);
            
            ts = self.tseries_video.tseries_model.get_tseries();
            
            dt = ts.dt;
            fs = 1/dt;
            fps = 30;
            playback_speed = 4;
            
            mat = ts.get_mat(1,1);
            
            x_lim(1) = max(x_lim(1),1);
            y_lim(1) = max(y_lim(1),1);
            x_lim(2) = min(x_lim(2),size(mat,2));
            y_lim(2) = min(y_lim(2),size(mat,1));
            
            mat = mat(y_lim(1):y_lim(2),x_lim(1):x_lim(2),frame_1:frame_2);
            mat = single(mat);
            
            %%
            
            % Filter
            sigma_dx = 0.5;
            sigma_dt = ceil(0.25/dt);
            filter = ones(1+2*sigma_dx,1+2*sigma_dx,1+2*sigma_dt,'single');
            filter = filter./sum(filter(:));

            begonia.util.logging.vlog(1,'Filtering video');
            mat = convn(mat,filter,'same');

            % Convert to rgb. 
            cm = colormap(a);
            mat_rgb = begonia.mat_functions.mat2rgb(mat,cm,a.CLim);
            
%             begonia.util.logging.vlog(1,'Saving RGB matrix')
%             save('~/Desktop/movie_rgb.mat','mat_rgb');
            
            % Resample
            I = begonia.util.calculate_movie_indices(size(mat_rgb,4),fs,fps,playback_speed);
            mat_rgb = mat_rgb(:,:,:,I);

            % Save movie
            path = '~/Desktop/movie.mp4';
            
            warning off
            begonia.util.logging.vlog(1,'Creating video : %s',path);
            v = VideoWriter(path,'MPEG-4');
            v.FrameRate = fps;
            v.Quality = 100;
            v.open();
            v.writeVideo(mat_rgb);
            v.close();
            warning on
            begonia.util.logging.vlog(1,'Creating video finished');
        end
        
        function create_outline_movie(self,~,~)
            if isempty(self.tseries_video) || ~self.tseries_video.ready
                return;
            end
            
            if isempty(self.frame_field_1) || isempty(self.frame_field_2)
                return;
            end
            
            frame_1 = str2double(self.frame_field_1.String);
            frame_2 = str2double(self.frame_field_2.String);
            
            if isnan(frame_1) || isnan(frame_2) || frame_2 < frame_1
                return;
            end
            
            ts = self.tseries_video.tseries_model.get_tseries();
            
            begonia.util.logging.vlog(1,'Loading ROA mask');
            roa_mask = ts.load_var('highpass_thresh_roa_mask');
            begonia.util.logging.vlog(1,'Loading ROA mask finished');

            a = self.tseries_video.video_axes;
            
            x_lim = round(a.XLim);
            y_lim = round(a.YLim);
            
            x_lim(1) = max(x_lim(1),1);
            y_lim(1) = max(y_lim(1),1);
            x_lim(2) = min(x_lim(2),size(roa_mask,2));
            y_lim(2) = min(y_lim(2),size(roa_mask,1));
            
            roa_mask = roa_mask(y_lim(1):y_lim(2),x_lim(1):x_lim(2),frame_1:frame_2);
            
            %%
            
            dt = ts.dt;
            fs = 1/dt;
            fps = 30;
            playback_speed = 4;
            
            mat = ts.get_mat(1,1);
            mat = mat(y_lim(1):y_lim(2),x_lim(1):x_lim(2),frame_1:frame_2);
            mat = single(mat);
            
            % Filter
            sigma_dx = 0.5;
            sigma_dt = ceil(0.5/dt);
            filter = ones(1+2*sigma_dx,1+2*sigma_dx,1+2*sigma_dt,'single');
            filter = filter./sum(filter(:));

            begonia.util.logging.vlog(1,'Filtering video');
            mat = convn(mat,filter,'same');

%             % Resample
%             I = begonia.util.calculate_movie_indices(size(mat,3),fs,fps,playback_speed);
%             mat = mat(:,:,I);
%             roa_mask = roa_mask(:,:,I);

            % Convert to rgb. 
            cm = colormap(a);
            roa_outline_rgb = begonia.mat_functions.mat2rgb(mat,cm,a.CLim);

            % Convert the roa mask to outlines
            begonia.util.logging.vlog(1,'Creating ROA outlines');
            se = strel('sphere', 2);
            for frame = 1:size(roa_mask,3)
                roa_mask(:,:,frame) = imdilate(roa_mask(:,:,frame), se) - roa_mask(:,:,frame);
            end

            % Apply the roa outlines to the movie. 
            begonia.util.logging.vlog(1,'Applying roa outlines to mat');
            roa_outline_rgb = begonia.binary_mat_functions.impose_mask_on_rgb_mat(roa_outline_rgb,roa_mask,[1,0,0]);
            
%             begonia.util.logging.vlog(1,'Saving RGB matrix')
%             save('~/Desktop/roa_outline_rgb.mat','roa_outline_rgb');
            
            % Resample
            I = begonia.util.calculate_movie_indices(size(roa_outline_rgb,4),fs,fps,playback_speed);
            roa_outline_rgb = roa_outline_rgb(:,:,:,I);

            % Save movie
            path = '~/Desktop/roa_outline.mp4';
            
            warning off
            begonia.util.logging.vlog(1,'Creating video : %s',path);
            v = VideoWriter(path,'MPEG-4');
            v.FrameRate = fps;
            v.Quality = 100;
            v.open();
            v.writeVideo(roa_outline_rgb);
            v.close();
            warning on
            begonia.util.logging.vlog(1,'Creating video finished');
        end
        
        function create_3d_plot(self,~,~)
            if isempty(self.tseries_video) || ~self.tseries_video.ready
                return;
            end
            
            if isempty(self.frame_field_1) || isempty(self.frame_field_2)
                return;
            end
            
            frame_1 = str2double(self.frame_field_1.String);
            frame_2 = str2double(self.frame_field_2.String);
            
            if isnan(frame_1) || isnan(frame_2) || frame_2 < frame_1
                return;
            end
            
            ts = self.tseries_video.tseries_model.get_tseries();
            
            begonia.util.logging.vlog(1,'Loading ROA mask');
            roa_mask = ts.load_var('highpass_thresh_roa_mask');
            begonia.util.logging.vlog(1,'Loading ROA mask finished');

            a = self.tseries_video.video_axes;
            
            x_lim = round(a.XLim);
            y_lim = round(a.YLim);
            
            x_lim(1) = max(x_lim(1),1);
            y_lim(1) = max(y_lim(1),1);
            x_lim(2) = min(x_lim(2),size(roa_mask,2));
            y_lim(2) = min(y_lim(2),size(roa_mask,1));
            
            roa_mask = roa_mask(y_lim(1):y_lim(2),x_lim(1):x_lim(2),frame_1:frame_2);
            roa_mask = single(roa_mask);
%             roa_mask = flip(roa_mask,3); % First frame on top

            % Limit the plotting to a reduced number of frames to reduce
            % computation time. 
            I = round(linspace(1,size(roa_mask,3),1000));
            roa_mask = roa_mask(:,:,I);
            %%
            [X,Y,Z] = meshgrid(1:size(roa_mask,2), 1:size(roa_mask,1), 1:size(roa_mask,3));
            
            begonia.util.logging.vlog(1,'Plotting : %s','~/Desktop/roa_splatter.png');
            begonia.util.logging.vlog(1,'Sit tight, this might take forever.');
            
            f = figure;

            try 
                p = patch(isosurface(X,Y,Z,roa_mask,0.5));
                p.FaceColor = 'r';
                p.EdgeColor = 'none';
                p.FaceAlpha = 1;

                daspect([1.5 1.5 6])
                view(3); 
                axis tight
                camlight
                lighting('gouraud')

                xlim([0,size(roa_mask,1)])
                ylim([0,size(roa_mask,2)])
                zlim([0,size(roa_mask,3)])

                ax = gca;
                ax.Box = 'on';
                ax.BoxStyle = 'full';
                ax.XTickLabel = [];
                ax.YTickLabel = [];
                ax.ZTickLabel = [];
                ax.XTick = [];
                ax.YTick = [];
                ax.ZTick = [];
                
                
                savefig(f,'~/Desktop/roa_splatter.fig');
                export_fig(f,'~/Desktop/roa_splatter.png');

%                 file_name = sprintf('%s_%s_%d_%d',id_genotype,id_trial,t_start,t_width);
%                 file_name = fullfile(output_folder,file_name);
% 
%                 export_fig(f,[file_name,'.fig']);
%                 export_fig(f,[file_name,'.png'],'-native','-transparent');
% 
%                 view(0,0);
%                 export_fig(f,[file_name,'_side.png'],'-native','-transparent');
% 
%                 view(0,90);
%                 export_fig(f,[file_name,'_top.png'],'-native','-transparent');
            catch e
                e
            end
        end
        
        
    end
    
    methods (Access = private)
        
        
    end
end

