classdef ScrollSlider < handle
    
    properties (SetObservable,AbortSet)
        parent
        
        scroll_speed = 1
        value
        limits
    end
    
    properties (Access = private)
        slider
        text
        container
    end
    
    methods
        
        function set.parent(self,parent)
            delete(self.container)
            
            if isempty(parent)
                self.parent = [];
                self.container = [];
                self.text = [];
                self.slider = [];
            else
                self.parent = parent;
                %% Container
                container = uix.VBox('Parent',parent);
                self.container = container;
                %% Text 
                text = uix.Text('Parent',container);
                text.VerticalAlignment = 'middle';
                self.text = text;
                %% Slider
                slider = uicontrol('Parent',container,'Style', 'slider');

                slider.Callback = @(~,~) self.callback_slider();

                self.slider = slider;
                %% Hijack scroll callback for the figure
                f = ancestor(text,'figure','toplevel');
                f.WindowScrollWheelFcn = @(~,e) self.callback_scroll_wheel(e.VerticalScrollCount);
            end
            %% Other
            self.reset();
        end
        
        function set.value(self,value)
            self.slider.Value = value;
            
            if isempty(value)
                self.text.String = 'N/A';
            else
                self.text.String = num2str(value);
            end
            self.value = value;
        end
        
        function set.limits(self,limits)
            if ~isempty(limits)
                self.slider.Min = limits(1);
                self.slider.Max = limits(2);
            end
            self.limits = limits;
        end
        
        function reset(self)
            self.limits = [-1,1];
            self.value = 0;
        end
    end
    
    methods (Access = private)
        
        function callback_slider(self)
            self.value = round(self.slider.Value);
        end
        
        function callback_scroll_wheel(self,scroll_count)
            value = self.value + scroll_count * self.scroll_speed;
            if value < self.limits(1)
                value = self.limits(1);
            end
            if value > self.limits(2)
                value = self.limits(2);
            end
            self.value = round(value);
        end
    end
end

