classdef TSeriesModel < handle
    
    properties (SetObservable,AbortSet)
        tseries_index
        tseries
    end
    
    properties (SetObservable,AbortSet,SetAccess = private)
        cycles
        channels
        tseries_mats
        clim_minmax
        frames
        video_dims
        ready
    end
    
    methods
        function val = get.ready(self)
            val = false;
            if isempty(self.tseries_index) || isempty(self.tseries)
                return;
            end
            if self.tseries_index < 1 || self.tseries_index > length(self.tseries)
                return;
            end
            val = true;
        end
        
        function set.tseries_index(self,tseries_index)
            self.tseries_index = tseries_index;
            self.load();
        end
        
        function set.tseries(self,tseries)
            self.tseries = tseries;
            self.load();
        end
        
        function ts = get_tseries(self)
            if self.ready
                ts = self.tseries(self.tseries_index);
            else
                ts = [];
            end
        end
        
        function img = get_img(self,frame,cycle,channel)
            img = [];
            
            if ~self.ready
                return;
            end
            if frame < 1 || frame > self.frames(cycle,channel)
                return;
            end
            
            mat = self.tseries_mats{cycle,channel};

            L = 15;
            if frame <= L
                win = -frame+1:L;
            elseif frame > size(mat,3) - L
                win = -L:size(mat,3) - frame;
            else
                win = -L:L;
            end
            I = win + frame;
            img = mat(:,:,I);
            img = mean(img,3);
        end
    end
    
    methods (Access = private)
        
        function load(self)
            if ~self.ready
                self.cycles = [];
                self.channels = [];
                self.tseries_mats = [];
                self.video_dims = [];
                self.frames = [];
                self.clim_minmax = [];
                return;
            end
            
            tseries = self.tseries;
            tseries_index = self.tseries_index;
            
            tseries_mats = {};
            video_dims = {};
            frames = [];
            for i = 1:tseries(tseries_index).cycles
                for j = 1:tseries(tseries_index).channels
                    tseries_mats{i,j} = tseries(tseries_index).get_mat(i,j);
                    video_dims{i,j} = size(tseries_mats{i,j});
                    frames(i,j) = video_dims{i,j}(3);
                end
            end
            
            switch class(tseries_mats{1,1})
                case 'uint16'
                    clim_minmax = [0,4095];
                case 'double'
                    clim_minmax = [0,1];
                otherwise
                    error('oh no');
            end
            
            self.cycles = tseries(tseries_index).cycles;
            self.channels = tseries(tseries_index).channels;
            self.tseries_mats = tseries_mats;
            self.video_dims = video_dims;
            self.frames = frames;
            self.clim_minmax = clim_minmax;
        end
    end
end

