classdef TSeriesPicker < handle
    properties (SetObservable,AbortSet)
        parent
        
        tseries_model
    end
    
    properties (SetAccess = private)
        previous_path = ''
    end
    
    properties (Access = private)
        listener_tseries_model
        popupmenu
    end
    
    methods
        function set.tseries_model(self,tseries_model)
            if ~isempty(self.listener_tseries_model)
                delete(self.listener_tseries_model);
            end
            self.tseries_model = tseries_model;
            self.listener_tseries_model = tseries_model.addlistener('tseries','PostSet',@(~,~) self.redraw());
            self.listener_tseries_model(2) = tseries_model.addlistener('tseries_index','PostSet',@(~,~) self.redraw());
            self.redraw();
        end
        
        function set.parent(self,parent)
            if ~isempty(self.popupmenu)
                delete(self.popupmenu);
            end
            
            if isempty(parent)
                self.parent = [];
                self.popupmenu = [];
            else
                popupmenu = uicontrol('Parent',parent,'Style','popupmenu');
                popupmenu.Callback = @(~,~) self.callback_popupmenu();
                
                self.parent = parent;
                self.popupmenu = popupmenu;
            end
            
            self.redraw();
        end
    end
    
    methods (Access = private)
        function redraw(self)
            if isempty(self.popupmenu)
                return;
            end
            
            t = self.tseries_model;
            if isempty(t)
                self.popupmenu.String = {'Search Directory'};
                self.popupmenu.Value = 1;
                return;
            end
            
            elem = {};
            elem{1} = 'Search Directory';
            for i = 1:length(t.tseries)
                elem{i+1} = t.tseries(i).name;
%                 if isprop(t.tseries(i), 'associated_trials')
%                     elem{i+1} = t.tseries(i).associated_trials(1).load_var('trial');
%                 else
%                     elem{i+1} = t.tseries(i).name;
%                 end
            end
            self.popupmenu.String = elem;
            
            i = t.tseries_index + 1;
            if isempty(i)
                i = 1;
            end
            self.popupmenu.Value = i;
        end
        
        function callback_popupmenu(self)
            
            if isempty(self.tseries_model)
                self.popupmenu.Value = 1;
                return;
            end
            
            if self.popupmenu.Value == 1
                path = uigetdir(self.previous_path,'Find TSeries');
                if path == 0
                    return;
                end

                self.previous_path = fileparts(path);

                tseries = begonia.scantype.find_tseries(path);
                
                if ~isempty(tseries)
                    self.tseries_model.tseries = tseries;
                    self.tseries_model.tseries_index = 1;
                end
            else
                self.tseries_model.tseries_index = self.popupmenu.Value - 1;
            end
            self.redraw();
        end
    end
end

