%{ 
Actions panel takes a list of EditorActions and creates buttons for them.
When an action is chosen, the on_action_chosen event is fired carryin with
it and ActionButtonEvent with a .action propery that can be used to
retrieve the action.
%}

classdef ActionsPanel < handle
    
    properties
        model
        actions
        figure
        panel
        
        button_height = 35;
        button_margin = 5;
        
        buttons = [];
    end
    
    properties(Dependent)
        position
        enabled
    end
    
    events
       on_action    
       on_queue
    end

    methods
        
        function obj = ActionsPanel(model, actions, fig, pos)
            if nargin < 3
                fig = figure('Position', [100,100, 250, 500]);
                %fig.CloseRequestFcn = @obj.handleClose;
                
                fig.ToolBar = 'none';
                fig.MenuBar = 'none';
                fig.Name = 'Actions';
                fig.NumberTitle = 'off';
                pos = [0, 0, 250, 500];
            end
            
            obj.model = model;
            obj.actions = actions;
            obj.figure = fig;
            
            obj.setupGui(pos);
        end
        
        
        function delete(obj)
            delete(obj.figure);
        end
        
        
        % Craetes buttons from the actions provided in the constructor
        function setupGui(obj, pos)
            obj.panel = uipanel(obj.figure ...
                , 'Units', 'pixels' ...
                , 'Position', pos ...
                , 'BorderType', 'none' ...
                );
            
            % position for buttons inside panel, starting from top:
            x = 0 + obj.button_margin;
            y = pos(4);
            w = pos(3) - obj.button_margin * 2;
            h = obj.button_height;
            
            for action = obj.actions
                
                % callback button:
                y = y - (obj.button_height + obj.button_margin);
                pos = [x, y, w - 25, h];
                action.button = uicontrol(obj.panel ...
                    , 'Style', 'pushbutton' ...
                    , 'String',action.title ...
                    , 'Position', pos ...
                    , 'HorizontalAlignment', 'Left' ...
                    , 'Units', 'normalized');
                action.button.Callback = ...
                    @(src, ev) obj.handleButtonClick(action);
                obj.buttons = [obj.buttons action.button];
                
                % queue button if applicable
                qbutton = uicontrol(obj.panel ...
                    , 'Style', 'pushbutton' ...
                    , 'String', ' ' ...
                    , 'Position', pos ...
                    , 'HorizontalAlignment', 'Left' ...
                    , 'Position', [x + (w - 20), y, 20, h] ...
                    , 'Enable', 'off' ...
                    , 'Units', 'normalized');
                qbutton.Callback = ...
                    @(src, ev) obj.handleQueueButtonClick(action);

                if action.can_queue
                   qbutton.Enable = 'on' ;
                   qbutton.String = 'Q' ;
                   obj.buttons = [obj.buttons qbutton];
                end
            end
        end
        
        
        
        function set.position(obj, pos)
            obj.panel.Position = pos;
        end
        
        function pos = get.position(obj)
            pos = obj.panel.Position;
        end
        
        function set.enabled(obj, state)
            set(obj.buttons, 'Enable', state);
        end
        
        % handles shortcut codes sent from the editor:
        function handleShortcutCode(obj, shortcut)
            shortcut_no_alt = replace(shortcut, 'alt-', '');
            for action = obj.actions
                if string(action.shortcut) == string(shortcut_no_alt)
                    
                   
                    % if keycode contains alt, we queue:
                    if contains(shortcut, 'alt') && action.can_queue
                        obj.handleQueueButtonClick(action);
                        return;
                    else
                        obj.handleButtonClick(action);
                    end
                end
            end
        end
        
        function handleButtonClick(obj, action)
            ev = xylobium.dledit.comps.ActionButtonEvent(action);
            notify(obj, 'on_action', ev);
        end
        
        function handleQueueButtonClick(obj, action)
            ev = xylobium.dledit.comps.ActionButtonEvent(action);
            notify(obj, 'on_queue', ev);
        end
        
        
        % Cleans up events on exit
        function handleClose(obj, ~, ~)
            delete(obj.figure);
        end
        
    end
end





