classdef Editor < handle

    properties
        model
        actions
        figure
        
        datagrid
        def_actions_panel
        cust_actions_panel

        jobview
        queueview
        
        varpicker
        
        prev_title
        
        clipboard_value = [];
    end
    
    properties(Dependent)
        enabled
    end

    methods 
        
        % Constructor
        function obj = Editor(dlocs, actions, initial_vars, modifiers, show_queue)
            
            % i hate you matlab. So much.
            if nargin < 5
                show_queue = true;
            end
            if nargin < 4
                modifiers = xylobium.dledit.model.Modifier.empty;
            end
            if nargin < 3
                initial_vars = {'name','path'};
            end
            if nargin < 2
                actions = [];
            end
             if nargin < 1
                dlocs = [];
             end
            
            left_extra = 0;
            if isempty(actions) 
                left_extra = 150;
            end
             
            obj.model = xylobium.dledit.model.EditorModel(dlocs);
            obj.model.modifiers = modifiers;
            obj.model.varlist = initial_vars;
           
            % user interface:
            obj.actions = actions;
            obj.figure = figure('Position', [200,200, 800,500]);
            obj.figure.ToolBar = 'none';
            obj.figure.MenuBar = 'none';
            obj.figure.NumberTitle = 'off';
            obj.figure.Name = 'DataLocation Editor - Letten Center 2018';
            
            obj.datagrid = xylobium.dledit.comps.DataGrid(...
                obj.model, obj.figure , [0,0, 500  + left_extra, 502]);
            
            def_actions = xylobium.dledit.getDefaultActions();
            obj.def_actions_panel = xylobium.dledit.comps.ActionsPanel(...
                obj.model...
                , def_actions...
                , obj.figure...
                , [501 + left_extra, 0, 150, 502]);
            
            if ~isempty(actions) 
                obj.cust_actions_panel = xylobium.dledit.comps.ActionsPanel(...
                    obj.model...
                    , actions...
                    , obj.figure...
                    , [651, 0, 150, 502]);
                disp('No actions')
            end
            
            if show_queue
                obj.showQueue();
            end
            
            % resize handling:
            obj.figure.ResizeFcn = @obj.handleResize;
            
            % event handling:
            obj.figure.KeyPressFcn = @obj.handleKeyPress;
            obj.datagrid.table.KeyPressFcn = @obj.handleKeyPress;
            
            addlistener(obj.model, 'on_selection_changed'... 
                , @(~,~) obj.handleSelectionChange());
            
            addlistener(obj.model, 'on_queue_execution_start'... 
                , @(~,~) obj.handleQueueExecutionStart());
            
            addlistener(obj.model, 'on_queue_execution_end'... 
                , @(~,~) obj.handleQueueExecutionEnd());
            
            addlistener(obj.def_actions_panel, 'on_action'... 
                , @(~,ev) obj.handleActionRequested(ev.action));
            
            if ~isempty(obj.cust_actions_panel)
                addlistener(obj.cust_actions_panel, 'on_action'... 
                    , @(~,ev) obj.handleActionRequested(ev.action));
            
                addlistener(obj.def_actions_panel, 'on_queue'... 
                    , @(~,ev) obj.handleQueueRequested(ev.action));

                addlistener(obj.cust_actions_panel, 'on_queue'... 
                    , @(~,ev) obj.handleQueueRequested(ev.action));
            end

            obj.figure.CloseRequestFcn = @(src, cbd) obj.handleCloseRequest(src, cbd);
            
            % notify user of keyboard shortcuts:
            obj.dump_shortcuts();
        end

        
        function handleCloseRequest(obj, src,callbackdata)
             delete(obj.figure)
             try
                delete(obj.queueview.figure);
             catch
                % naughty... 
             end
        end
        
        % called when the selection of dataloctions in datagrid has changed -
        % needs to update the title primarely
        function handleSelectionChange(obj)
            selected = obj.model.selected;
            count = length(selected);
            obj.figure.Name = [num2str(count) 'x ' class(selected)  ' selected' ];
        end
        
        
        % called when a user presses a button in the figure. 
        function handleKeyPress(obj, sender, ev)
            code = xylobium.shared.keyboard_event_to_str(ev);
            
            % pass this message to the actions panels:
            if ~isempty(code)
                obj.def_actions_panel.handleShortcutCode(code);
                obj.cust_actions_panel.handleShortcutCode(code);
            end
        end
        
        
        % checks if an action supports single  or multiple datalocations,
        % then calls the execution command on them
        function handleActionRequested(obj, action)
            
            if isempty(obj.model.selected)
                if action.can_execute_without_dloc
                    obj.executeActionCallback(action, []);
                else 
                    msgbox('Need to select something first');
                end
                return;
            end
            
            if action.accept_multiple_dlocs
                % execute on all selected dloc:
                obj.executeActionCallback(action, obj.model.selected)
            else
                % execute once pr. selected dloc
                for dloc = obj.model.selected
                    obj.executeActionCallback(action, dloc)
                end
            end
        end
        
        
        % executes a callback from an action on one or mroe datalocations
        function executeActionCallback(obj, action, dloc)
            % ensure we catch errors, but do not permanently disable the
            % user interface.  We'll capture errors and rethrow after
            % re-enabling user interface:
            obj.enabled = 'off';
            err = [];

            try
                action.click_callback(dloc, obj.model, obj);
            catch err
                disp('Error!');
            end

            obj.enabled = 'on';
            
            % rethrow if needed:
            if ~isempty(err)
               rethrow(err);
            end
        end
        
        % called when an action is requested to be queued from one of the
        % action panels
        function handleQueueRequested(obj, action)
            
            for dloc = obj.model.selected
                name = dloc.path;
                if isprop( dloc, 'name')
                    name = ['"' dloc.name '"'];
                end
                
                obj.model.enqueueAction(...
                    [action.title ' ' name] ...
                    , action.click_callback...
                    , dloc ...
                    , obj);
            end
            
        end
        
        % Changes the state of the user interface to enable or disable
        % buttons. Used to lock the interface during processing
        function set.enabled(obj, state)
            if strcmp(state, 'off')
                obj.prev_title = obj.figure.Name;
                obj.figure.Name = '*** WORKING *** WORKING *** WORKING *** WORKING ***';
            else
                obj.figure.Name = obj.prev_title;
            end
            
            obj.def_actions_panel.enabled = state;
            obj.cust_actions_panel.enabled = state;
            obj.datagrid.enabled = state;
            drawnow();
        end
        
        
        function dump_shortcuts(obj)
            if ~isempty(obj.cust_actions_panel)
                actions = [obj.def_actions_panel.actions ...
                    obj.cust_actions_panel.actions];
            

                disp('---')
                disp('TIP: keyboard shortcuts:')
                disp(' ')
                for action = actions
                    if ~isempty(action.shortcut)
                        s = sprintf(['\t' action.title ':   \t\t' action.shortcut]);
                        disp(s);
                    end
                end
                disp(' ')
                disp('For queueable actions, hold "alt" in to queue')
                disp('---')
            end
            
        end
        
        % creates a queue view figure
        function showQueue(obj)
            if ~isempty(obj.queueview)
                delete(obj.queueview)
            end
            obj.queueview = xylobium.dledit.comps.QueueView(obj.model);
        end
        
        
        function handleQueueExecutionStart(obj)
            obj.enabled = 'off';
        end
        
        function handleQueueExecutionEnd(obj)
            obj.enabled = 'on';
        end
        
        
        % called when the window change size, andresized the individual 
        % GUI elements to fit the new position
        function handleResize(obj, ~, ~)
            left_extra = 0;
            if isempty(obj.actions) 
                left_extra = 150;
            end
            
            win = obj.figure.Position;
            grid_pos = ceil(ceil([0,0, win(3)-(300 - left_extra), win(4)]));
            defac_pos = ceil([win(3)-299 + left_extra, 0, 150, win(4)]);
            cusac_pos = ceil([win(3)-149,0, 150, win(4)]);
            
            obj.datagrid.position = grid_pos;
            obj.def_actions_panel.position = defac_pos;
            obj.cust_actions_panel.position = cusac_pos;
        end
        
        
        
    end
end














