classdef TSeriesModel < handle
    
    properties
        frame
        clim
    end
    
    properties (SetAccess = private)
        cycle
        channel
        tseries
        tseries_mats
        tseries_list
        
        clim_maxmin
        
        frames
        video_dim
        
        current_img
    end
    
    properties (Access = private)
    end
    
    events
        TSeriesChanged
        FrameChanged
        CLimChanged
    end
    
    methods
        
        function load_tseries(self,tseries,i)
            if nargin < 3
                i = 1;
            end
            
            if length(tseries) == 1 && ismember(tseries,self.tseries_list)
                self.tseries = tseries;
            else
                self.tseries_list = tseries;
                tseries = tseries(i);
                self.tseries = tseries;
            end
            
            self.tseries_mats = {};
            for i = 1:tseries.cycles
                for j = 1:tseries.channels
                    self.tseries_mats{i,j} = tseries.get_mat(i,j);
                end
            end
            
            switch class(self.tseries_mats{1,1})
                case 'uint16'
                    clim = [0,4095];
                case 'double'
                    clim = [0,1];
                otherwise
                    error('oh no');
            end
            self.clim_maxmin = clim;
            self.clim = clim;
            
            cycle = 1;
            channel = 1;
            
            self.video_dim = size(self.tseries_mats{cycle,channel});
            self.frames = self.video_dim(3);
            
            self.notify('TSeriesChanged');
            
            self.cycle = cycle;
            self.channel = channel;
            self.frame = 1;
        end
        
        
        function set.frame(self,val)
            if isempty(self.tseries)
                self.frame = [];
            else
                val = floor(val);
                if (val < 1) || (val > self.frames)
                    return;
                end
                self.frame = val;
                
                self.notify('FrameChanged');
            end
        end
        
%         function set.cycle(self,val)
%             self.cycle = val;
%         end
%         
%         function set.channel(self,val)
%             self.channel = val;
%         end

        function set.clim(self,val)
            if isempty(self.tseries)
                return;
            end
            
            if length(val) ~= 2
                return;
            end
            
            if val(1) >= val(2)
                return;
            end
            
            self.clim = val;
            self.notify('CLimChanged');
        end
        
        function val = get.current_img(self)
            if isempty(self.tseries)
                val = [];
            else
                mat = self.tseries_mats{self.cycle,self.channel};
                % Its here
                L = 15;
                if self.frame <= L
                    win = -self.frame+1:L;
                elseif self.frame > size(mat,3) - L
                    win = -L:size(mat,3) - self.frame;
                else
                    win = -L:L;
                end
                I = win + self.frame;
                val = mat(:,:,I);
                val = mean(val,3);
%                 val = self.tseries_mats{self.cycle,self.channel}(:,:,self.frame);
            end
        end
    end
    
    methods (Access = private)
        
    end
end

