classdef TSeriesSlider < handle
    
    properties
        scroll_speed = 10
    end
    
    properties (SetAccess = private)
        tseries_model
        
        slider
        text
        container
    end
    
    methods
        function self = TSeriesSlider(parent,tseries_model)
            self.tseries_model = tseries_model;
            %% Container
            container = uix.VBox('Parent',parent);
            self.container = container;
            %% Text 
            text = uix.Text('Parent',container);
            text.VerticalAlignment = 'middle';
            self.text = text;
            %% Slider
            slider = uicontrol('Parent',container,'Style', 'slider');
            
            slider.Callback = @(~,~) self.callback_slider();
            
            tseries_model.addlistener('TSeriesChanged',@(~,~) self.on_video_change());
            tseries_model.addlistener('FrameChanged',@(~,~) self.on_frame_change());
            
            self.slider = slider;
            %% Hijack scroll callback for the figure
            f = ancestor(text,'figure','toplevel');
            f.WindowScrollWheelFcn = @(~,e) self.callback_scroll_wheel(e.VerticalScrollCount);
            %% Other
%             container.Padding = 10;
            self.on_reset();
        end
        
    end
    
    methods (Access = private)
        
        function callback_slider(self)
            self.tseries_model.frame = self.slider.Value;
        end
        
        function callback_scroll_wheel(self,scroll_count)
            self.tseries_model.frame = self.tseries_model.frame + scroll_count * self.scroll_speed;
        end
        
        function on_frame_change(self)
            self.slider.Value = self.tseries_model.frame;
            self.text.String = num2str(self.tseries_model.frame);
        end
        
        function on_video_change(self)
            self.slider.Min = 1;
            self.slider.Max = self.tseries_model.frames;
        end
        
        function on_reset(self)
            self.text.String = '0';
            
            self.slider.Min = 0;
            self.slider.Max = 1;
            self.slider.Value = 0.5;
        end
    end
end

