classdef PhaseCorrConfig < handle
    %PHASE_FIX Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        Figure
        FullAxes = [];
        ZoomedAxes = [];
        
        StackList
        CurrentStack
        
        FullImage
        CorrectedImage
        ZoomedImage
        ZoomAreaSelector
        OffsetSlider
        OffsetText
        
        Button_ProccessAll
        Button_ProccessThis
        Button_FixAll
        Button_FixThis
        
        Offset = 0
    end
    
    methods
        
        function obj = PhaseCorrConfig(stacklist)
            if ~isa(stacklist, 'begonia.scantype.TSeries')
                error('Can only work with begonia.stack.StackPrarie');
            end
            
            obj.StackList = stacklist;
            obj.CurrentStack = obj.StackList(1);
            obj.setup();
        end
        
        function setup(obj)
            obj.Figure = figure('Visible','on','Position',[100,100,1000,500]);
            
            obj.FullAxes = axes(obj.Figure);
            
            setpixelposition(obj.FullAxes, [200 50 400 450]);
            obj.ZoomedAxes = axes(obj.Figure);
            setpixelposition(obj.ZoomedAxes, [600 50 400 450]);
            
            lb = uicontrol(obj.Figure, 'Style','listbox');
            setpixelposition(lb, [10 50 200 450]);
            if ~isempty(obj.StackList)
                lb.String = {obj.StackList.name};
            end
            lb.Callback = @obj.handle_list_selection;
            
            % alignment slider
            obj.OffsetSlider = uicontrol(obj.Figure, 'Style','slider', ...
                'Min',-20,'Max',20,'Value',0);
            setpixelposition(obj.OffsetSlider, [10 10 400 30]);
            addlistener(obj.OffsetSlider, 'Value', 'PostSet',@obj.handle_new_offset);
            
            % value view
            obj.OffsetText = uicontrol(obj.Figure, 'Style','text')
            setpixelposition(obj.OffsetText , [440 10 50 20]);
            
            % align all button:
%             obj.Button_ProccessAll = uicontrol(obj.Figure, 'Style','pushbutton', 'String','Fix/align all');
%             setpixelposition(obj.Button_ProccessAll , [500 10 90 30]);
%             obj.Button_ProccessAll.Callback = @obj.handle_process_all;
%             
%             obj.Button_ProccessThis = uicontrol(obj.Figure, 'Style','pushbutton', 'String','Fix /align current');
%             setpixelposition(obj.Button_ProccessThis , [600 10 90 30]);
%             obj.Button_ProccessThis.Callback = @obj.handle_process_current;
%             
%             obj.Button_FixAll = uicontrol(obj.Figure, 'Style','pushbutton', 'String','Fix all');
%             setpixelposition(obj.Button_FixAll , [700 10 90 30]);
%             obj.Button_FixAll.Callback = @obj.handle_fix_all;
%             
%             obj.Button_FixThis = uicontrol(obj.Figure, 'Style','pushbutton', 'String','Fix current');
%             setpixelposition(obj.Button_FixThis , [800 10 90 30]);
%             obj.Button_FixThis.Callback = @obj.handle_fix_current;
            
            obj.load_image();
            obj.update_view();
            obj.update_zoom();
            
            if obj.CurrentStack.has_var('phase_offset_px')
                offset = obj.CurrentStack.load_var('phase_offset_px');
                obj.set_offset(offset);
            else
                obj.set_offset(0)
            end
        end
        
        function load_image(obj)

            stack = obj.CurrentStack.get_mat(1,1);
            h = waitbar(0.1, 'Making reference image');
            obj.FullImage = mean(stack(:,:,1:30), 3);
            close(h);

        end
        
        
        function set_offset(obj, offset)
            obj.Offset = offset;
            obj.OffsetText.String = num2str(obj.Offset);
            obj.CurrentStack.save_var('phase_offset_px', obj.Offset);
            obj.update_view();
            obj.update_zoom();
        end
        
        
        function update_view(obj, ev, sender)
            img = obj.FullImage;
            rect = [100 100 100 100];
            
            % correct phase:
            for i = 1:2:size(img, 2)
                row = img(i,:);
                row_shifted = circshift(row, obj.Offset);
                img(i,:) = row_shifted;
            end

            % set as new main image:
            obj.CorrectedImage = img;
            if ~isempty(obj.ZoomAreaSelector)
                rect = obj.ZoomAreaSelector.getPosition();
                delete(obj.ZoomAreaSelector);
            end
            
            imagesc(img, 'Parent', obj.FullAxes);
            obj.ZoomAreaSelector = imrect(obj.FullAxes, rect);
            obj.ZoomAreaSelector.addNewPositionCallback(@obj.update_zoom);
            
            obj.FullAxes.XTickLabel = [];
        end
        
        
        function update_zoom(obj, ev, sender)
            % display images:
            rect = obj.ZoomAreaSelector.getPosition();
            img_zoom = imcrop(obj.CorrectedImage , rect);
            imagesc(img_zoom, 'Parent', obj.ZoomedAxes);     
            obj.ZoomedAxes.XTickLabel = [];
        end
        

        function handle_new_offset(obj, ev, sender)
            obj.set_offset(floor(obj.OffsetSlider.Value));  
        end
        
        function handle_list_selection(obj, ev, sender)
            obj.CurrentStack = obj.StackList(ev.Value);

            obj.load_image();
                        
            if obj.CurrentStack.has_var('phase_offset_px')
                offset = obj.CurrentStack.load_var('phase_offset_px');
                obj.set_offset(offset);
            else
                obj.set_offset(0)
            end
            
            %obj.update_view();
            %obj.update_zoom();
        end
        
        function handle_process_all(obj, ev, sender)
            ok = [];
            failed = [];
            for i = 1:length(obj.StackList)
                tseries = obj.StackList(i);
                try
                    xylobium.phasecorr.fix_phase_offset(tseries.path, 1);
                    ok = [ok tseries];
                catch err
                    failed = [failed err];
                end
            end
            disp(['DONE - OK: ' num2str(length(ok)) '/' num2str(length(tseries))]);
            assignin('base','failed_tseries',failed);
        end
        
        function handle_process_current(obj, ev, sender)
            xylobium.phasecorr.fix_phase_offset(obj.CurrentStack.path, 1);
        end
        
        function handle_fix_all(obj, ev, sender)
            ok = [];
            failed = [];
            for i = 1:length(obj.StackList)
                tseries = obj.StackList(i);
                try
                    xylobium.phasecorr.fix_phase_offset(tseries.path, 0);
                    ok = [ok tseries];
                catch err
                    failed = [failed err];
                end
            end
            disp(['DONE - OK: ' num2str(length(ok)) '/' num2str(length(tseries))]);
            assignin('base','failed_tseries',failed);
        end
        
        function handle_fix_current(obj, ev, sender)
            xylobium.phasecorr.fix_phase_offset(obj.CurrentStack.path, 0);
        end
        
        
    end
    
end

