classdef PickerRow < handle
    
    properties (SetObservable)
        parameter_name
        parameter_options
        selected
    end
    
    events
        ClearCacheClicked
    end
    
    properties (Access = private)
        hbox_main
        scrolling_panel_buttons
        hbox_buttons
        
        button_clear_cache
        buttons_parameter
        text_parameter
    end
    
    methods
        function self = PickerRow(parent,parameter_name,parameter_options)
            if nargin < 2
                parameter_name = '';
            end
            if nargin < 3
                parameter_options = {};
            end
            
            self.parameter_name = '';
            self.parameter_options = {};
            
            self.buttons_parameter = {};
            
            %
            hbox_main = uix.HBox('Parent',parent);
            self.hbox_main = hbox_main;
            
            button_clear_cache = uicontrol('Parent',hbox_main,'Style', 'pushbutton');
            button_clear_cache.String = 'Refresh';
            self.button_clear_cache = button_clear_cache;
            
            text_parameter = uix.Text('Parent',hbox_main);
            text_parameter.HorizontalAlignment = 'left';
            text_parameter.VerticalAlignment = 'middle';
            self.text_parameter = text_parameter;
            
            self.scrolling_panel_buttons = uix.ScrollingPanel('Parent',hbox_main);
            self.hbox_buttons = uix.HBox('Parent',self.scrolling_panel_buttons);
            
            self.button_clear_cache.Callback = @(~,~) self.notify('ClearCacheClicked');
            
            self.parameter_name = parameter_name;
            self.parameter_options = parameter_options;
        end
        
        function delete(self)
            delete(self.hbox_main);
        end
        
        function set.selected(self,val)
            assert(islogical(val) && length(val) == length(self.parameter_options))
            if ~isequal(self.selected,val)
                for i = 1:length(val)
                    self.buttons_parameter{i}.Value = val(i);
                end
            end
        end
        
        function val = get.selected(self)
            val = false(1,length(self.parameter_options));
            for i = 1:length(self.buttons_parameter)
                val(i) = self.buttons_parameter{i}.Value;
            end
        end
        
        function set.parameter_name(self,val)
            self.parameter_name = val;
            if isempty(val)
                self.text_parameter.String = 'N/A';
            else
                self.text_parameter.String = val;
            end
        end
        
        function set.parameter_options(self,val)
            self.parameter_options = val;
            self.update_buttons();
        end
        
    end
    
    methods (Access = private)
        function update_buttons(self)
            for i = 1:length(self.buttons_parameter)
                delete(self.buttons_parameter{i});
            end
            
            for i = 1:length(self.parameter_options)
                btn = xylobium.components.ColoredPushbutton(self.hbox_buttons);
                btn.String = self.parameter_options{i};
                btn.addlistener('Value','PostSet',@(~,~) self.dummy_set_selected());
                self.buttons_parameter{i} = btn;
            end
            self.hbox_main.Widths = [100,150,-1];
            self.scrolling_panel_buttons.Widths = 100*length(self.parameter_options);
        end
        
        function dummy_set_selected(self)
            self.selected = self.selected;
        end
    end
end

