classdef Picker < handle
    
    properties
        function_handles
        function_names
        map
    end
    
    methods
        function self = Picker()
            self.reset();
        end
        
        function reset(self)
            self.function_handles = {};
            self.function_names = {};
            self.map = containers.Map();
        end

        function clear_cache(self)
            for i = 1:length(self.function_handles)
                self.function_handles{i}.clearCache();
            end
        end
        
        function val = get_column_functions(self,data)
            val = {};
            cnt = 1;
            for i = 1:length(self.function_names)
                tbl = self.function_handles{i}(data);
                for j = 1:width(tbl)
                    val{cnt} = self.function_handles{i};
                    cnt = cnt + 1;
                end
            end
        end
        
        function val = get_column_names(self,data)
            val = cell(1,length(self.function_names));
            for i = 1:length(self.function_names)
                ret = self.function_handles{i}(data);
                column_names = ret.Properties.VariableNames;
                
                val{i} = column_names;
            end
            val = cat(2,val{:});
        end
        
        function val = get_column_categories(self,data)
            val = cell(1,length(self.function_names));
            for i = 1:length(self.function_names)
                tbl = self.function_handles{i}(data);
                % Convert to categorical.
                for j = 1:width(tbl)
                    tbl.(tbl.Properties.VariableNames{j}) = cellstr(categorical(tbl.(tbl.Properties.VariableNames{j})));
                end
                val{i} = cell(1,width(tbl));
                for j = 1:width(tbl)
                    val{i}{j} = sort(unique(tbl{:,j}))';
                end
            end
            val = cat(2,val{:});
        end
        
        function summary(self)
            fprintf('Functions \n');
            for i = 1:length(self.function_names)
                fprintf('\t%s\n',self.function_names{i});
            end
            fprintf('Filters \n');
            key_set = self.map.keys;
            for i = 1:length(key_set)
                filter_vals = self.map(key_set{i});
                if ischar(filter_vals)
                    filter_vals = {filter_vals};
                end
                fprintf('\t%s : \n',key_set{i});
                for j = 1:length(filter_vals)
                    fprintf('\t\t{''%s''}\n',filter_vals{j});
                end
            end
        end
        
        function I = run_gui(self,data)
            if isempty(data)
                warning('No data to display.')
                I = [];
                return;
            end
            xylobium.picker.PickerGUI(self,data);
            uiwait;
            I = self.run(data);
            self.summary();
        end
        
        function I = run(self,data)
            if isempty(data)
                I = [];
                return;
            end
            
            if isempty(self.function_handles)
                warning('No filter functions added.');
                I = [];
                return;
            end
            
            tbls = cell(1,length(self.function_handles));
            
            for i = 1:length(self.function_handles)
                tbls{i} = self.function_handles{i}(data);
            end
            
            tbl = cat(2,tbls{:});
            
            % Convert to categorical.
            % Then convert to cell to match undefined.
            for i = 1:width(tbl)
                tbl.(tbl.Properties.VariableNames{i}) = cellstr(categorical(tbl.(tbl.Properties.VariableNames{i})));
            end
            
            keys = self.map.keys();
            I = true(height(tbl),1);
            for i = 1:length(keys)
                val = self.map(keys{i});
                try
                    I = I & ismember(tbl.(keys{i}),val);
                catch e
                    switch e.identifier
                        case 'MATLAB:table:UnrecognizedVarName'
                            error('No columned named ''%s'' in the Picker functions.',keys{i});
                        otherwise
                            rethrow(e);
                    end
                end
            end
        end
        
        function set_column_filter(self,column_name,filter_value)
            assert(isempty(filter_value) || ischar(filter_value) ...
                || iscellstr(filter_value), ...
                'filter_value must be empty, char or a cellstring.');
            if isempty(filter_value)
                if self.map.isKey(column_name)
                    self.map.remove(column_name);
                end
            else
                self.map(column_name) = filter_value;
            end
        end
        
        function param = get_column_filter(self,column_name)
            if self.map.isKey(column_name)
                param = self.map(column_name);
                if ischar(param)
                    param = {param};
                end
            else
                param = {};
            end
        end
        
        function add_function(self,func,name)
            func_cached = memoize(func);
            if nargin < 3
                func_stat = functions(func);
                name = func_stat.function;
            end
            
            self.function_handles{end+1} = func_cached;
            self.function_names{end+1} = name;
        end

        function remove_function(self,func_name)
            I = ismember(self.function_names,func_name);
            if any(I)
                self.function_handles(I) = [];
                self.function_names(I) = [];
            end
        end
    end
end