classdef PickerGUI < handle
    
    properties %(Access = private)
        data
        picker
        picker_rows
        text_header
        scrolling_panel
        vbox
    end
    
    methods
        function self = PickerGUI(picker,data)
            self.picker = picker;
            self.data = data;
            
            f = figure;
            f.MenuBar = 'none';
            f.ToolBar = 'none';
            f.Position(3:4) = [650,400];
            
            bigboi_vbox = uix.VBox();
            
            text_header = uix.Text('Parent',bigboi_vbox);
            text_header.VerticalAlignment = 'middle';
            text_header.HorizontalAlignment = 'center';
            text_header.FontSize = 15;
            self.text_header = text_header;
            
            scrolling_panel = uix.ScrollingPanel('Parent',bigboi_vbox);
            self.scrolling_panel = scrolling_panel;
            
            vbox = uix.VBox('Parent',scrolling_panel);
            self.vbox = vbox;
            
            bigboi_vbox.Heights = [40,-1];
            
            self.recreate_picker_rows();
            self.update_header();
            
            bigboi_vbox.Parent = f;
        end
        
        function clear_cache(self,i)
            parameter_functions = self.picker.get_column_functions(self.data);
            parameter_functions{i}.clearCache();
            
            self.recreate_picker_rows();
            self.update_header();
        end
        
    end
    
    methods (Access = private)
        
        function recreate_picker_rows(self)
            for i = 1:length(self.picker_rows)
                delete(self.picker_rows{i});
            end
            self.picker_rows = {};
            
            parameter_names = self.picker.get_column_names(self.data);
            parameter_options = self.picker.get_column_categories(self.data);
            
            picker_rows = cell(1,length(parameter_names));
            for i = 1:length(parameter_names)
                picker_rows{i} = xylobium.picker.components.PickerRow(self.vbox);
                picker_rows{i}.parameter_name = parameter_names{i};
                picker_rows{i}.parameter_options = parameter_options{i};
                picker_rows{i}.selected = ismember(parameter_options{i},self.picker.get_column_filter(parameter_names{i}));
                picker_rows{i}.addlistener('selected','PostSet',@(~,~) self.update_header());
                picker_rows{i}.addlistener('ClearCacheClicked',@(~,~) self.clear_cache(i));
            end
            
            self.picker_rows = picker_rows;
            self.scrolling_panel.Heights = 40*(length(self.vbox.Children));
        end
        
        function update_header(self)
            for i = 1:length(self.picker_rows)
                I = self.picker_rows{i}.selected;
                self.picker.set_column_filter(self.picker_rows{i}.parameter_name,self.picker_rows{i}.parameter_options(I));
            end
            I = self.picker.run(self.data);
            self.text_header.String = sprintf('Selected elements %d/%d',sum(I),length(I));
        end
    end
end

