classdef ResampleTSeries < handle
    properties (SetObservable,AbortSet)
        dir_in
        dir_out
    end
    
    properties (Access = private)
        tseries
        tseries_I
        
        text_dir_in
        text_dir_out
        edit_merged_frames
        uit
    end
    
    methods
        function self = ResampleTSeries(dir_in)
            if nargin < 1
                dir_in = '';
            end
            %%
            f = figure;
            f.Position(3:4) = [700,500];
            vbox = uix.VBox('Parent',f);
            %% vbox
            hbox_1 = uix.HBox('Parent',vbox);
            hbox_2 = uix.HBox('Parent',vbox);
            hbox_3 = uix.HBox('Parent',vbox);
            
            vbox.Heights = [20,20,-1];
            vbox.Padding = 20;
            vbox.Spacing = 10;
            
            %% hbox_1
            text_dir_in = uix.Text('Parent',hbox_1);
            text_dir_in.HorizontalAlignment = 'left';
            text_dir_in.String = 'N/A';
            self.text_dir_in = text_dir_in;
            
            btn_dir_in = uicontrol('Parent',hbox_1,'Style','pushbutton');
            btn_dir_in.String = '...';
            btn_dir_in.Callback = @self.callback_btn_dir_in;
            
            hbox_1.Widths = [-1,20];
            %% hbox_2
            text_dir_out = uix.Text('Parent',hbox_2);
            text_dir_out.HorizontalAlignment = 'left';
            text_dir_out.String = 'N/A';
            self.text_dir_out = text_dir_out;
            
            btn_dir_out = uicontrol('Parent',hbox_2,'Style','pushbutton');
            btn_dir_out.String = '...';
            btn_dir_out.Enable = 'off';
            
            hbox_2.Widths = [-1,20];
            %% hbox_3
            uit = uitable(hbox_3);
            uit.CellSelectionCallback = @self.callback_uit;
            self.uit = uit;
            
            vbox_3_1 = uix.VBox('Parent',hbox_3);
            
            hbox_3.Widths = [-1,100];
            hbox_3.Spacing = 10;
            
            %% vbox_3_1
            text_merged_frames = uix.Text('Parent',vbox_3_1);
            text_merged_frames.String = 'Merged Frames';
            
            edit_merged_frames = uicontrol('Parent',vbox_3_1,'Style','edit');
            edit_merged_frames.String = '1';
            self.edit_merged_frames = edit_merged_frames;
            
            btn_resample = uicontrol('Parent',vbox_3_1,'Style','pushbutton');
            btn_resample.String = 'Resample';
            btn_resample.Callback = @(~,~)self.resample_selected_tseries();
            
            vbox_3_1.Heights = [20,20,20];
            vbox_3_1.Spacing = 10;
            %%
            self.dir_in = dir_in;
            self.update_uitable();
        end
        
        function set.dir_in(self,val)
            if isnumeric(val)
                val = '';
            end
            
            if isempty(val)
                self.dir_in = '';
                self.text_dir_in.String = 'N/A';
                
                self.tseries = [];
                
                self.dir_out = '';
            else
                self.dir_in = val;
                self.text_dir_in.String = val;
                
                self.tseries = begonia.scantype.find_tseries(val);
                
                self.dir_out = [val,'_resampled'];
            end
        end
        
        function set.dir_out(self,val)
            if isempty(self.dir_in)
                self.dir_out = '';
                self.text_dir_out.String = 'N/A';
            else
                self.dir_out = val;
                self.text_dir_out.String = val;
            end
        end
        
        function set.tseries(self,val)
            self.tseries = val;
            self.update_uitable();
        end
        
        function resample_selected_tseries(self)
            ts = self.tseries(self.tseries_I);
            if isempty(ts)
                return;
            end
            
            merge_frames = str2double(self.edit_merged_frames.String);
            merge_frames = round(merge_frames);
            
            if isnan(merge_frames) || merge_frames < 1
                begonia.util.logging.vlog(1,'Invalid number of frames to merge.');
                return;
            end
            
            for i = 1:length(ts)
                dir_out = strrep(ts(i).path,self.dir_in,self.dir_out);
                
                begonia.util.logging.vlog(1,sprintf('Resampling %s',ts.name));
                begonia.tseries_operations.resample(ts(i),dir_out,merge_frames);
            end
            begonia.util.logging.vlog(1,'Finished');
        end
        
    end
    
    methods (Access = private)
        
        function update_uitable(self)
            self.uit.ColumnName = {'Path','Finished'};
            self.uit.ColumnWidth = {400,'auto'};
            
            if isempty(self.tseries)
                self.uit.Data = {};
            else
                paths = {self.tseries.path}';
                paths = strrep(paths,self.dir_in,'...');

                finished = false(length(paths),1);
                finished = num2cell(finished);

                self.uit.Data = [paths,finished];
            end
        end
        
        function callback_btn_dir_in(self,s,e)
            self.dir_in = uigetdir();
        end
        
        function callback_uit(self,s,e)
            self.tseries_I = unique(e.Indices(:,1));
        end
        
    end
end

