classdef Roa3DTool < handle
    
    
    properties
    end
    
    properties %(Access = private)
        tseries_model
        tseries_video
        tseries_picker
        frame_slider
        contrast_slider
    end
    
    methods
        function self = Roa3DTool(tseries)
            assert(~isempty(tseries));
            %% Instantiate components
            tseries_model     	= xylobium.components.TSeriesModel();
            tseries_video    	= xylobium.components.TSeriesVideo();
            tseries_picker      = xylobium.components.TSeriesPicker();
            frame_slider    	= xylobium.components.ScrollSlider();
            contrast_slider 	= xylobium.components.DualSlider();
            colormap_picker     = xylobium.components.ColormapPicker();
            roa_plot_panel      = xylobium.components.RoaPlotPanel();

            %% Assign
            self.tseries_model      = tseries_model;
            self.tseries_video      = tseries_video;
            self.frame_slider       = frame_slider;
            self.contrast_slider    = contrast_slider;
            self.tseries_picker     = tseries_picker;
            
            %% Load gui 
            f = figure;
            big_boi_box = uix.HBox('Parent',f);
            
            vbox_1 = uix.VBox('Parent',big_boi_box);
            contrast_slider.parent  = vbox_1;
            colormap_picker.parent  = vbox_1;
            roa_plot_panel.parent = vbox_1;
            
            vbox_2 = uix.VBox('Parent',big_boi_box);
            tseries_picker.parent   = vbox_2;
            tseries_video.parent    = vbox_2;
            frame_slider.parent     = vbox_2;
            
            %% Adjust gui sizes
            big_boi_box.Widths = [300,-1];
%             vbox_1.Heights = [60,30];
            vbox_2.Heights = [40,-1,40];
            vbox_2.Padding = 20;
            
            %% Listeners
            tseries_model.addlistener('clim_minmax','PostSet',@self.on_clim_minmax_change);
            
            tseries_video.addlistener('frame','PostSet',@self.on_frame_change);
            frame_slider.addlistener('value','PostSet',@self.on_frame_change);
            
            contrast_slider.addlistener('value','PostSet',@self.on_clim_change);
            tseries_video.addlistener('clim','PostSet',@self.on_clim_change);
            
            tseries_video.addlistener('frames','PostSet',@self.on_frames_change);
            %% Init
            
            tseries_video.tseries_model     = tseries_model;
            tseries_picker.tseries_model    = tseries_model;
            
            roa_plot_panel.tseries_video = tseries_video;
            
            tseries_model.tseries_index = 1;
            tseries_model.tseries = tseries;
            
            contrast_slider.string = 'Contrast';
            
            frame_slider.scroll_speed = 10;
            
            colormap_picker.axes = tseries_video.video_axes;
            
            f.Position(3:4) = [300+600,700];
        end
        
    end
    
    methods (Access = private)
        
        function on_frame_change(self,s,e)
            val = e.AffectedObject.(s.Name);
            self.tseries_video.frame = val;
            self.frame_slider.value = val;
        end
        
        function on_clim_change(self,s,e)
            val = e.AffectedObject.(s.Name);
            
            self.tseries_video.clim = val;
            self.contrast_slider.value = val;
        end
        
        function on_clim_minmax_change(self,s,e)
            val = e.AffectedObject.(s.Name);
            self.contrast_slider.limits = val;
        end
        
        function on_frames_change(self,s,e)
            val = e.AffectedObject.(s.Name);
            self.frame_slider.limits = [1,val];
        end
    end
end

