classdef RoaIgnoreGUI < handle
    
    properties
        tseries
        tseries_index
        
        var_name_roa_mask
        var_name_roa_ignore_mask
    end
    
    properties (Access = private)
        tseries_model
        tseries_video
        tseries_picker
        frame_slider
        contrast_slider
        roa_display
        mask_editor
        
        button_save_roa
        button_clear_roa
        button_load_roa
        slider_brush_radius
        text_roa_mode
        popupmenu_roa_mode
    end
    
    methods
        function self = RoaIgnoreGUI(tseries,var_name_roa_mask,var_name_roa_ignore_mask)
            if nargin < 1
                tseries = [];
            end
            if nargin < 2
                self.var_name_roa_mask = 'roa_mask';
            else
                self.var_name_roa_mask = var_name_roa_mask;
            end
            if nargin < 3
                self.var_name_roa_ignore_mask = 'roa_ignore_mask';
            else
                self.var_name_roa_ignore_mask = var_name_roa_ignore_mask;
            end
            %% Instantiate components
            tseries_model     	= xylobium.components.TSeriesModel();
            tseries_video    	= xylobium.components.TSeriesVideo();
            tseries_picker      = xylobium.components.TSeriesPicker();
            frame_slider    	= xylobium.components.ScrollSlider();
            contrast_slider 	= xylobium.components.DualSlider();
            roa_display         = xylobium.components.RoaDisplay();
            mask_editor         = xylobium.components.MaskEditor();

            %% Load gui 
            f = figure;
            big_boi_box = uix.HBox('Parent',f);
            
            vbox_1 = uix.VBox('Parent',big_boi_box);
            contrast_slider.parent = vbox_1;
            button_save_roa     = uicontrol('Style','pushbutton','Parent',vbox_1,'String','Clear ROA Mask');
            button_clear_roa    = uicontrol('Style','pushbutton','Parent',vbox_1,'String','Save ROA Mask');
            button_load_roa     = uicontrol('Style','togglebutton','Parent',vbox_1,'String','Load ROA');
            text_roa_mode = uix.Text('Parent',vbox_1,'String','ROA Mode','VerticalAlignment','middle');
            popupmenu_roa_mode = uicontrol('Style','popupmenu','Parent',vbox_1,'String',{'Add','Subtract'});
            text_brush_size = uix.Text('Parent',vbox_1,'String','Brush Size','VerticalAlignment','middle');
            slider_brush_radius = uicontrol('Style','slider','Parent',vbox_1,'String','Brush Size','Min',1,'Max',40,'Value',1);
            
            vbox_2 = uix.VBox('Parent',big_boi_box);
            tseries_picker.parent = vbox_2;
            tseries_video.parent = vbox_2;
            frame_slider.parent = vbox_2;
            
            %% Adjust gui sizes
            big_boi_box.Widths = [150,-1];
            vbox_1.Heights = [60,30,30,30,30,30,30,30];
            vbox_2.Heights = [40,-1,40];
            vbox_2.Padding = 20;
            
            %% Assign
            self.tseries_model      = tseries_model;
            self.tseries_video      = tseries_video;
            self.frame_slider       = frame_slider;
            self.contrast_slider    = contrast_slider;
            self.tseries_picker     = tseries_picker;
            self.roa_display        = roa_display;
            self.mask_editor        = mask_editor;
            self.button_clear_roa   = button_clear_roa;
            self.button_save_roa    = button_save_roa;
            self.button_load_roa    = button_load_roa;
            self.popupmenu_roa_mode = popupmenu_roa_mode;
            self.text_roa_mode      = text_roa_mode;
            self.slider_brush_radius = slider_brush_radius;
            
            %% Listeners
            tseries_model.addlistener('clim_minmax','PostSet',@self.handle_tseries_model_events);
            tseries_model.addlistener('tseries','PostSet',@self.handle_tseries_model_events);
            tseries_model.addlistener('tseries_index','PostSet',@self.handle_tseries_model_events);
            
            tseries_video.addlistener('frame','PostSet',@self.handle_tseries_video_events);
            tseries_video.addlistener('frames','PostSet',@self.handle_tseries_video_events);
            tseries_video.addlistener('clim','PostSet',@self.handle_tseries_video_events);
            
            frame_slider.addlistener('value','PostSet',@self.handle_frame_slider_events);
            
            contrast_slider.addlistener('value','PostSet',@self.handle_contrast_slider_events);
            
            mask_editor.addlistener('brush_radius','PostSet',@self.handle_mask_editor_events);
            
            button_save_roa.Callback = @self.handle_button_events;
            button_clear_roa.Callback = @self.handle_button_events;
            button_load_roa.Callback = @self.handle_button_events;
            slider_brush_radius.Callback = @self.handle_button_events;
            popupmenu_roa_mode.Callback = @self.handle_popupmenu_roa_mode_events;
            %% Init
            
            tseries_video.tseries_model = tseries_model;
            
            tseries_picker.tseries_model = tseries_model;
            
            tseries_model.tseries_index = 1;
            tseries_model.tseries = tseries;
            
            roa_display.video_axes = tseries_video.video_axes;
            
            mask_editor.video_axes = tseries_video.video_axes;
            mask_editor.brush_radius = 20;
            
            contrast_slider.string = 'Contrast';
            
            frame_slider.scroll_speed = 10;
            
        end
        
        function set.tseries(self,val)
            self.tseries_model.tseries = val;
        end
        
        function val = get.tseries(self)
            val = self.tseries_model.tseries;
        end
        
        function set.tseries_index(self,val)
            self.tseries_model.tseries_index = val;
        end
        
        function val = get.tseries_index(self)
            val = self.tseries_model.tseries_index;
        end
    end
    
    methods (Access = private)
        function handle_button_events(self,s,e)
            switch s.String
                case 'Clear ROA Mask'
                    self.mask_editor.mask(:) = false;
                case 'Save ROA Mask'
                    if self.tseries_model.ready && self.mask_editor.ready
                        ts = self.tseries_model.tseries(self.tseries_model.tseries_index);
                        begonia.util.logging.vlog(1,'Saving roa_ignore_mask');
                        roa_ignore_mask = self.mask_editor.mask;
                        if sum(roa_ignore_mask) == 0
                            ts.clear_var('roa_ignore_mask');
                        else
                            ts.save_var(roa_ignore_mask);
                        end
                    end
                case 'Load ROA'
                    % Load ROA
                    if self.tseries_model.ready && self.button_load_roa.Value
                        ts = self.tseries_model.tseries(self.tseries_model.tseries_index);
                        if ts.has_var(self.var_name_roa_mask)
                            begonia.util.logging.vlog(1,sprintf('Loading %s',self.var_name_roa_mask));
                        else
                            begonia.util.logging.vlog(1,'TSeries missing ROA mask');
                        end
                        self.roa_display.roa_mask = ts.load_var(self.var_name_roa_mask);
                    else
                        self.roa_display.roa_mask = [];
                    end
                case 'Brush Size'
                    self.mask_editor.brush_radius = self.slider_brush_radius.Value;
            end
        end
        
        function handle_popupmenu_roa_mode_events(self,s,e)
            switch s.String{s.Value}
                case 'Add'
                    self.mask_editor.mode = 'add';
                case 'Subtract'
                    self.mask_editor.mode = 'subtract';
            end
        end
        
        function handle_mask_editor_events(self,s,e)
            switch s.Name
                case 'brush_radius'
                    self.slider_brush_radius.Value = self.mask_editor.brush_radius;
            end
        end
        
        function handle_tseries_model_events(self,s,e)
            switch s.Name
                case 'tseries_index'
                    if self.tseries_model.ready
                        % Load tseries
                        ts = self.tseries_model.tseries(self.tseries_model.tseries_index);
                        
                        % Load ROA
                        if self.button_load_roa.Value
                            if ts.has_var(self.var_name_roa_mask)
                                begonia.util.logging.vlog(1,sprintf('Loading %s',self.var_name_roa_mask));
                            else
                                begonia.util.logging.vlog(1,'TSeries missing ROA mask');
                            end
                            self.roa_display.roa_mask = ts.load_var(self.var_name_roa_mask);
                        else
                            self.roa_display.roa_mask = [];
                        end
                        
                        % Load ROA ignore mask
                        default_mask = false(self.tseries_model.video_dims{1,1}(1:2));
                        self.mask_editor.mask = ts.load_var('roa_ignore_mask',default_mask);
                    end
                case 'tseries'
                    if self.tseries_model.ready
                        % Load tseries
                        ts = self.tseries_model.tseries(self.tseries_model.tseries_index);
                        
                        % Load ROA
                        if self.button_load_roa.Value
                            if ts.has_var(self.var_name_roa_mask)
                                begonia.util.logging.vlog(1,'Loading ROA mask');
                            else
                                begonia.util.logging.vlog(1,'TSeries missing ROA mask');
                            end
                            self.roa_display.roa_mask = ts.load_var(self.var_name_roa_mask);
                        else
                            self.roa_display.roa_mask = [];
                        end
                        
                        % Load ROA ignore mask
                        default_mask = false(self.tseries_model.video_dims{1,1}(1:2));
                        self.mask_editor.mask = ts.load_var('roa_ignore_mask',default_mask);
                    end
                case 'clim_minmax'
                    if isempty(self.contrast_slider.limits)
                        self.contrast_slider.limits = self.tseries_model.clim_minmax;
                    end
            end
        end
        
        function handle_tseries_video_events(self,s,e)
            switch s.Name
                case 'frame'
                    self.frame_slider.value = self.tseries_video.frame;
                    self.roa_display.frame = self.tseries_video.frame;
                case 'frames'
                    self.frame_slider.limits = [1,self.tseries_video.frames];
                case 'clim'
                    self.contrast_slider.value = self.tseries_video.clim;
            end
        end
        
        function handle_frame_slider_events(self,s,e)
            switch s.Name
                case 'value'
                    self.tseries_video.frame = self.frame_slider.value;
            end
        end
        
        function handle_contrast_slider_events(self,s,e)
            switch s.Name
                case 'value'
                    self.tseries_video.clim = self.contrast_slider.value;
            end
        end
    end
end

