function align(dloc, model, editor)

    target_dir = dloc.path;
    phase_corr_path = [];
    
    % are we asked to phase correct first?
    if dloc.has_var('phase_offset_px') && dloc.load_var('phase_offset_px') ~= 0
        disp('Phase correction factor found - correcting before alignment')
        
        offset = dloc.load_var('phase_offset_px');
        disp(['Offset is: ' num2str(offset)]);
        
        target_dir = [dloc.path '_phasecorr'];
        phase_corr_path = target_dir; % remember for later so we can delete
        
        yucca.phasecorr.fixPhaseOffset(dloc.path, target_dir, offset); 
    end
    
    % perform alignment
    if editor.config.alignment_dir == "_SAME_"
        output = [dloc.path '_aligned'];
    else
        output = fullfile(editor.config.alignment_dir, [dloc.name '_aligned']);
    end
    if dloc.has_var('ncconfig')
        disp('Loading custom aligment settings from "align_conf"');
        ncconfig = dloc.load_var('ncconfig');
    else
        disp('Using default rigid aligment on channel 1')
        ncconfig = begonia.stack_operations.AlignmentSettings();
    end
    disp(['Aligning: ' dloc.name ' > ' output])

    
    
    begonia.stack_operations.align(dloc.path, output, ncconfig, true);
    model.save(dloc, 'aligned', output);
    
    % remove the phase correction dir if it exists:
    if ~isempty(phase_corr_path)
        rmdir(phase_corr_path, 's');
    end
end

