function preview(data, model, editor)
    
    if data.has_var('aligned')
        aligned_path = data.load_var('aligned')
        video_path = fullfile(aligned_path, [data.name '-aligned.avi']);
        fluo = makePreview(aligned_path, video_path);
        model.save(data, 'vidprev', video_path);
        model.save(data, 'fulltrace', fluo);
    else
       error('No aligned stack found'); 
    end
end

function fluo = makePreview(src_path, video_path)
    % get stack:
    stack = begonia.scantype.find_scans(src_path);
    mat = stack.get_mat(1,1);
    all_frames = 1:size(mat, 3);
    
    % downsample by factor of 10:
    frames = downsample(all_frames,10);
    
    % grab frames and full field fluoresence from these frames:
    writer = VideoWriter(video_path);
    disp(['Generating video: ' video_path]);
    open(writer)
    
    fluo = [];
    
    % process frames:
    for framenr = frames
        
        % video frame:
        frame_mat = yucca.tseries.average_frame(mat, framenr, 30);
        frame_mat = yucca.util.as_double_frame(frame_mat);
        
        position =  [1 20; 120 20 ; 190 20];
        sec = seconds(framenr * stack.dt);
        sec_str = [datestr(sec, 'HH::MM:SS:FFF') ' sec'];
        frame_str = ['#' num2str(framenr)];
        
        value = {sec_str frame_str stack.name};
        RGB = insertText(frame_mat,position,value,'AnchorPoint','LeftBottom');
        writeVideo(writer, uint8(RGB));
        
        % full field fluoresence:
        fluo = [fluo ; mean2(frame_mat)];
    end
    close(writer);

end


















