function writeToIndex(tseries, model, editor)
    config = editor.config;
    index_file = config.index_file;
    
    % we'll open the index file, check the tseries is not already there,
    % then write some of our data to it:
    
    % create file if it does not exis:
    if ~exist(index_file, 'file')
        warning('Index not found - creating.  If not first run - has file been moved?');
        disp(['Creating: ' index_file]);
        tsindex = struct();
        tsindex.series = [];
        tsindex.created = datetime();
        tsindex.updated = datetime();
        tsindex.api = 'StackProcessor=1.0';
        
        save(index_file, 'tsindex');
    end
    
    % load index:
    data = load(index_file, 'tsindex');
    tsindex = data.tsindex;
    
    % create tseries struct:
    tseries.dl_ensure_has_uuid();
    
    entry = struct();
    entry.uuid = tseries.dl_unique_id;
    entry.name = tseries.name;
    entry.src_path = tseries.path;
    entry.aligned_path = tseries.load_var('aligned');
    
    % remove from index if it already exists:
    found = false;
    for i = 1:length(tsindex.series)
        old_entry = tsindex.series(i);
        if old_entry.uuid == entry.uuid
            tsindex.series(i) = entry;
            found = true;
        end
    end
    
    if ~found
        tsindex.series = [tsindex.series entry];
    end
    
    % re-save index:
    save(index_file, 'tsindex');
end


