classdef StackProcessorSheetConfig < xylobium.stackproc.IStackProcessorConfig
    %STACKPROCESSORCONFIG Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        setup_sheet
        table_tseries 
        table_config
        
        index_file
        summary_dir
        
        backup_cmd
        tseries_paths
        
        alignment_dir
        proxy_output_dir
    end
    
    methods
        function obj = StackProcessorSheetConfig(setup_sheet)
            obj.setup_sheet = setup_sheet;
            obj.table_tseries = readtable(setup_sheet, 'sheet', 'TSeries');
            obj.table_config = readtable(setup_sheet, 'sheet', 'Config');
            
            if isempty(obj.table_tseries) || isempty(obj.table_config)
                error('Cannot load setup sheet. Look at template for proper configuration');
            end
        end
        
        % index file for storing info on processed outputs
        function index = get.index_file(obj)
            t = obj.table_config;
            index = t(strcmp(t.Key, 'index.file'),:).Setting{:};
        end
        
        % command to backup data to remote storage
        function backup_cmd = get.backup_cmd(obj)
            t = obj.table_config;
            backup_cmd = t(strcmp(t.Key, 'backup.command'),:).Setting{:};
        end
        
        % command to backup data to remote storage
        function sumdir = get.summary_dir(obj)
            t = obj.table_config;
            sumdir = t(strcmp(t.Key, 'summary.dir'),:).Setting{:};
        end
        
        % directory to output proxy stacks to:
        function proxydir = get.proxy_output_dir(obj)
            t = obj.table_config;
            proxydir = t(strcmp(t.Key, 'proxy.output_dir'),:).Setting{:};
            if isempty(proxydir)
                error('proxy.output_dir not set in configuration sheet');
            end
        end
        
        % directory to output aligned stacks to:
        function aldir = get.alignment_dir(obj)
            t = obj.table_config;
            aldir = t(strcmp(t.Key, 'alignment.output_dir'),:).Setting{:};
            if isempty(aldir)
                error('alignment.output_dir not set in configuration sheet');
            end
        end
        
        % list of tseries to include
        function paths = get.tseries_paths(obj)
            paths = obj.table_tseries.Series;
        end
        

    end
end

