classdef ExperimentTimeLine < xylobium.timeline.TimeLineFcn & handle
    
    
    properties
        fig
        ax
        time_objects
        text_field
    end
    
    methods
        function obj = ExperimentTimeLine(varargin)
            if nargin < 1
                import begonia.scantype.prairie.*;
                fldr = uigetdir();
                objs = begonia.scantype.find_scans(fldr);
                objs2 = electrophysiologyviewer.LDToEphysAdapter(fldr);
                objs3 = electrophysiologyviewer.ICPToEphysAdapter(fldr);
                
                objs = [objs, objs2, objs3];
                 srcs = unique(arrayfun(@(x) {x.source}, objs));
                numlines = length(srcs);
                for i = 1:numlines
                     obj.time_objects{i} =  objs(contains(arrayfun(@(x) {x.source}, objs), srcs{i}));
                end
                

                obj.draw_fig;
                obj.draw_axis; 
            else
                classes = cellfun(@(x) {class(x)}, varargin);
                if any(contains(classes, 'matlab.graphics.axis.Axes'))
                    obj.ax = varargin{contains(classes, 'matlab.graphics.axis.Axes')};
                    obj.fig = obj.ax.Parent;
                    objs = varargin(~contains(classes, 'matlab.graphics.axis.Axes'));
                    srcs = unique(arrayfun(@(x) {x.source}, objs));
                    %srcs = unique({objs.source});
                    numlines = length(srcs);
                    for i = 1:numlines
                        obj.time_objects{i} = objs(contains(arrayfun(@(x) {x.source}, objs), srcs{i}));
                    end
                    
                    
%                 elseif any(contains(classes, 'yucca.datacat.Entry'))
%                     %dtactntry = varargin(contains(classes, 'yucca.datacat.Entry'));
%                     %other = varargin(~contains(classes, 'yucca.datacat.Entry'));
% %                     ix_prairie = contains(dtactntry.source, 'Prairie');
% %                     ix_ephys = contains(dtactntry.source, 'Ephys');
% %                     cnt = 1;
% %                     if any(ix_prairie)
% %                         obj.time_objects{cnt} = dtactntry{ix_prairie};
% %                         cnt = cnt+1;
% %                     end
% %                     if any(ix_ephys)
% %                         obj.time_objects{cnt} = dtactntry{ix_ephys};
% %                         cnt = cnt+1;
% %                     end
% %                     if any(~ix_ephys || ~ix_prairie)
% %                         obj.time_objects{cnt} = dtactntry{~ix_ephys & ~ix_prairie};
% %                         cnt = cnt+1;
% %                     end
%                     objs = [objs, objs2, objs3];
%                     srcs = unique({objs.source});
%                     numlines = length(srcs);
%                     for i = 1:numlines
%                         obj.time_objects{i} = objs(contains({objs.source}, srcs{i}));
%                     end
%                 
%                     obj.draw_fig;
%                      obj.draw_axis;
                 else
                    objs = varargin{~contains(classes, 'matlab.graphics.axis.Axes') & ~contains(classes, 'double')};
                    srcs = unique(arrayfun(@(x) {x.source}, objs));
                    numlines = length(srcs);
                    for i = 1:numlines
                        obj.time_objects{i} = objs(contains(arrayfun(@(x) {x.source}, objs), srcs{i}));
                    end
                    obj.draw_fig;
                    obj.draw_axis;
                    
                end
               
            end
   
            obj.add_rects;
            obj.draw_buttons;
            zoom('xon');
            zoom('off');
            pan('xon');
            pan('off');
            
            
            obj.fig.Units = 'normalized';
            set(findall(obj.fig, '-property', 'Units' ), 'Units', 'Normalized')
            obj.fig.Position(3) = 0.84;
            obj.fig.Position = [0.1, 0.3, 0.8, 0.4];
        
        end
        
        function draw_fig(obj)
            obj.fig = figure('Units', 'centimeters', 'Position', [20, 10, 50, 18], 'Tag', 'ExperimentTimeLine');
            obj.fig.Name = '';
            obj.fig.NumberTitle = 'off';
            obj.fig.KeyPressFcn = @obj.keypress;
           
            xylobium.shared.layout.remove_tools(obj.fig);
        end
        
        function draw_axis(obj)
            obj.ax = axes('Units', 'centimeters', 'Position', [0.5, 3, 49, 15]);
            obj.ax.XTick = [];
            obj.ax.YTick = [];
            obj.ax.YLim = [0,1];
            
        end
         
        function highlight_rect(obj, src, event)
            coords = get(obj.ax, 'CurrentPoint');
            if strcmp(src.Tag, 'Selected')
                src.EdgeColor = 'k';
                src.LineWidth = 0.5;
                src.Tag = '';
                delete(obj.text_field)
            else
                src.EdgeColor = 'g';
                src.LineWidth = 3;
                src.Tag = 'Selected';
                delete(obj.text_field)
                [~,txt] = fileparts(src.UserData.path);
                txt = src.UserData.name;
                if isempty(txt)
                    [~,txt] = fileparts(src.UserData.path);
                end
                obj.text_field = text((coords(1,1)+range(obj.ax.XLim)/100),(coords(1,2)-range(obj.ax.XLim)/100),txt);
                
            end
        end
        
        function add_rects(obj)
            num_lines = length(obj.time_objects);
            lims = obj.ax.YLim(1):range(obj.ax.YLim)/num_lines:obj.ax.YLim(2);
            
            for i = 1:length(obj.time_objects)
                objects = obj.time_objects{i};
                for j = 1:length(objects)
                    
                    if ~any(contains(superclasses(objects(j)), 'begonia.data_management.DataInfo'))
                        break;
                        
                    end
                   
                   
                    if isa(objects(j), 'begonia.data_management.DataLocation')
                        if objects(j).has_var('UseNot')
                            UseNot = objects(j).load_var('UseNot');
                        else
                            UseNot = 1;
                            objects(j).save_var('UseNot');
                        end
                    else
                        UseNot = 1;
                    end
                    
                    switch UseNot
                        case 1
                            hold on;
                            c(i) = rectangle('Position', [datenum(objects(j).start_time), lims(i),datenum(objects(j).duration),(range(obj.ax.YLim)/num_lines)]);
                            c(i).ButtonDownFcn = @obj.highlight_rect;
                            if isa(objects(j), 'yucca.trial.Trial')
                                c(i).FaceColor = [1, 0, 0, 0.3];
                            elseif isa(objects(j), 'begonia.scantype.prairie.LinescanPrairie')
                                c(i).FaceColor = [0, 0, 1, 0.3];
                                
                            elseif isa(objects(j), 'begonia.scantype.prairie.TSeriesPrairie')
                                c(i).FaceColor = [0, 1, 0, 0.3];
                                
                            elseif isa(objects(j), 'begonia.scantype.prairie.ZStackPrairie')
                                c(i).FaceColor = [0, 1, 1, 0.3];
                            else
                                c(i).FaceColor = [0.1, 0.5, 0.1, 0.3];
                            end
                            c(i).UserData = objects(j);
                            clear strt stp
                    end
                end
                min_time(i) = min(arrayfun(@(x)datenum(x.start_time), objects));
                max_time(i) = max(arrayfun(@(x)datenum(x.end_time), objects));
            end
            hold on;
            obj.ax.XLim = [min(min_time), max(max_time)];
            
            datetick(obj.ax);

        end
   
    end
    
end

