classdef ExperimentTimeLine2 < xylobium.timeline.TimeLineFcn & handle
    
    
    properties
        fig
        ax
        time_objects
        text_field
    end
    
    methods
        function obj = ExperimentTimeLine2(varargin)
            if nargin < 1
                import begonia.scantype.prairie.*;
                fldr = uigetdir();
                objs = begonia.scantype.find_scans(fldr);
                objs2 = electrophysiologyviewer.LDToEphysAdapter(fldr);
                objs3 = electrophysiologyviewer.ICPToEphysAdapter(fldr);
                
                objs = [objs, objs2, objs3];
                 srcs = unique(arrayfun(@(x) {x.source}, objs));
                numlines = length(srcs);
                for i = 1:numlines
                     obj.time_objects{i} =  objs(contains(arrayfun(@(x) {x.source}, objs), srcs{i}));
                end
                

                obj.draw_fig;
                obj.draw_axis; 
            else
                classes = cellfun(@(x) {class(x)}, varargin);
                if any(contains(classes, 'matlab.graphics.axis.Axes'))
                    obj.ax = varargin{contains(classes, 'matlab.graphics.axis.Axes')};
                    obj.fig = obj.ax.Parent;
                    objs = varargin(~contains(classes, 'matlab.graphics.axis.Axes'));
                    srcs = unique(arrayfun(@(x) {x.source}, objs));
                    %srcs = unique({objs.source});
                    numlines = length(srcs);
                    for i = 1:numlines
                        obj.time_objects{i} = objs(contains(arrayfun(@(x) {x.source}, objs), srcs{i}));
                    end
                    
                    
%                 elseif any(contains(classes, 'yucca.datacat.Entry'))
%                     %dtactntry = varargin(contains(classes, 'yucca.datacat.Entry'));
%                     %other = varargin(~contains(classes, 'yucca.datacat.Entry'));
% %                     ix_prairie = contains(dtactntry.source, 'Prairie');
% %                     ix_ephys = contains(dtactntry.source, 'Ephys');
% %                     cnt = 1;
% %                     if any(ix_prairie)
% %                         obj.time_objects{cnt} = dtactntry{ix_prairie};
% %                         cnt = cnt+1;
% %                     end
% %                     if any(ix_ephys)
% %                         obj.time_objects{cnt} = dtactntry{ix_ephys};
% %                         cnt = cnt+1;
% %                     end
% %                     if any(~ix_ephys || ~ix_prairie)
% %                         obj.time_objects{cnt} = dtactntry{~ix_ephys & ~ix_prairie};
% %                         cnt = cnt+1;
% %                     end
%                     objs = [objs, objs2, objs3];
%                     srcs = unique({objs.source});
%                     numlines = length(srcs);
%                     for i = 1:numlines
%                         obj.time_objects{i} = objs(contains({objs.source}, srcs{i}));
%                     end
%                 
%                     obj.draw_fig;
%                      obj.draw_axis;
                 else
                    objs = varargin{~contains(classes, 'matlab.graphics.axis.Axes') & ~contains(classes, 'double')};
                    srcs = unique(arrayfun(@(x) {x.source}, objs));
                    numlines = length(srcs);
                    for i = 1:numlines
                        obj.time_objects{i} = objs(contains(arrayfun(@(x) {x.source}, objs), srcs{i}));
                    end
                    obj.draw_fig;
                    obj.draw_axis;
                    
                end
               
            end
   
            obj.add_rects;
            obj.draw_buttons;
            zoom('xon');
            zoom('off');
            pan('xon');
            pan('off');
            
            set(findall(obj.fig, '-property', 'Units' ), 'Units', 'Normalized')
            obj.fig.Position = [0.1, 0.3, 0.8, 0.4];
        
        end
        
        function draw_fig(obj)
            obj.fig = figure('Units', 'centimeters', 'Position', [20, 10, 50, 18], 'Tag', 'ExperimentTimeLine');
            obj.fig.Name = '';
            obj.fig.NumberTitle = 'off';
            obj.fig.KeyPressFcn = @obj.keypress;
            obj.remove_tools;
        end
        
        function draw_axis(obj)
            obj.ax = axes('Units', 'centimeters', 'Position', [0.5, 3, 49, 15]);
            obj.ax.XTick = [];
            obj.ax.YTick = [];
            obj.ax.YLim = [0,1];
            obj.fig.Units = 'normalized';
            obj.fig.Position(3) = 0.84;
        end
         
        function highlight_rect(obj, src, event)
            coords = get(obj.ax, 'CurrentPoint');
            if strcmp(src.Tag, 'Selected')
                src.EdgeColor = 'k';
                src.LineWidth = 0.5;
                src.Tag = '';
                delete(obj.text_field)
            else
                src.EdgeColor = 'g';
                src.LineWidth = 3;
                src.Tag = 'Selected';
                delete(obj.text_field)
                [~,txt] = fileparts(src.UserData.path);
                txt = src.UserData.name;
                if isempty(txt)
                    [~,txt] = fileparts(src.UserData.path);
                end
                obj.text_field = text((coords(1,1)+range(obj.ax.XLim)/100),(coords(1,2)-range(obj.ax.XLim)/100),txt);
                
            end
        end
        
        function add_rects(obj)
            num_lines = length(obj.time_objects);
            lims = obj.ax.YLim(1):range(obj.ax.YLim)/num_lines:obj.ax.YLim(2);
            
            for i = 1:length(obj.time_objects)
                objects = obj.time_objects{i};
                for j = 1:length(objects)
                    
                    if ~any(contains(superclasses(objects(j)), 'begonia.data_management.DataInfo'))
                        break;
                        
                    end
                   
                   
                    if isa(objects(j), 'begonia.data_management.DataLocation')
                        if objects(j).has_var('UseNot')
                            UseNot = objects(j).load_var('UseNot');
                        else
                            UseNot = 1;
                            objects(j).save_var('UseNot');
                        end
                    end
                    
                    switch UseNot
                        case 1
                            hold on;
                            c(i) = rectangle('Position', [datenum(objects(j).start_time), lims(i),datenum(objects(j).duration),(range(obj.ax.YLim)/num_lines)]);
                            c(i).ButtonDownFcn = @obj.highlight_rect;
                            if isa(objects(j), 'yucca.trial.Trial')
                                c(i).FaceColor = [1, 0, 0, 0.3];
                            elseif isa(objects(j), 'begonia.scantype.prairie.LinescanPrairie')
                                c(i).FaceColor = [0, 0, 1, 0.3];
                                
                            elseif isa(objects(j), 'begonia.scantype.prairie.TSeriesPrairie')
                                c(i).FaceColor = [0, 1, 0, 0.3];
                                
                            elseif isa(objects(j), 'begonia.scantype.prairie.ZStackPrairie')
                                c(i).FaceColor = [0, 1, 1, 0.3];
                            else
                                c(i).FaceColor = [0.1, 0.5, 0.1, 0.3];
                            end
                            c(i).UserData = objects(j);
                            clear strt stp
                    end
                end
                min_time(i) = min(arrayfun(@(x)datenum(x.start_time), objects));
                max_time(i) = max(arrayfun(@(x)datenum(x.end_time), objects));
            end
            hold on;
            obj.ax.XLim = [min(min_time), max(max_time)];
            
            datetick(obj.ax);
            obj.add_plots;

        end
        
        function add_plots(obj)
             num_lines = length(obj.time_objects);
            lims = obj.ax.YLim(1):range(obj.ax.YLim)/num_lines:obj.ax.YLim(2);
            
            for i = 1:length(obj.time_objects)
                objects = obj.time_objects{i};
                for j = 1:length(objects)
                    
                    if ~any(contains(superclasses(objects(j)), 'begonia.data_management.DataInfo'))
                        break;
                        
                    end
                   
                   
                    if isa(objects(j), 'begonia.data_management.DataLocation')
                        if objects(j).has_var('UseNot')
                            UseNot = objects(j).load_var('UseNot');
                        else
                            UseNot = 1;
                            objects(j).save_var('UseNot');
                        end
                    end
                      
                     if objects(j).has_var('PlotData')
                        PlotData = objects(j).load_var('PlotData');
                    else
                        PlotData = 1;
                        objects(j).save_var('PlotData');
                     end
                    data = [];
                    switch PlotData
                        case 1
                            if isa(objects(j), 'yucca.trial.Trial')
                                test = 9;
                                if has_var(objects(j), 'CameraTrace')
                                    
                                elseif has_var(objects(j), 'Wheel')
                                    %tmp = objects(j).load_var('wheel');
                                    %data = tmp(2).Distance.Data;
                                    %time = datenum(seconds(tmp.time)+objects(j).start_time_abs);
                                    
                                end
                                
                            elseif isa(objects(j), 'electrophysiologyviewer.Ephys')
                            
                                if ~isempty(objects(j).data)
                                     tmp = seconds(objects(j).time) + objects(j).start_time_abs;
                                    data = objects(j).data;
                                elseif has_var(objects(j), 'data')

                                    tmp = seconds(objects(j).load_var('time')) + objects(j).start_time_abs;
                                    time = datenum(tmp);
                                    data = objects(j).load_var('data');
                                end
                            end
                           
                            if ~isempty(data)
                                data = (range(obj.ax.YLim)/num_lines)*((data-(prctile(data, 1)-0.05))/((prctile(data, 99.99)+0.05)-prctile(data,0.2)))+lims(i);
                                axes(obj.ax)
                                hold on;
                                plot(time, data)
                            end
              
                       
                  
                    end
                end

            end
            
            
        end
        
     
        
        function remove_tools(obj)
            dck = findall(obj.fig, 'Tooltipstring', 'Show Plot Tools and Dock Figure');
            dck.Visible = 'off';
            hdpl = findall(obj.fig, 'Tooltipstring', 'Hide Plot Tools');
            hdpl.Visible = 'off';
            insl = findall(obj.fig, 'Tooltipstring', 'Insert Legend');
            insl.Visible = 'off';
            insc = findall(obj.fig, 'Tooltipstring', 'Insert Colorbar');
            insc.Visible = 'off';
            lnkpl = findall(obj.fig, 'Tooltipstring', 'Link Plot');
            lnkpl.Visible = 'off';
            brsh = findall(obj.fig, 'Tooltipstring', 'Brush/Select Data');
            brsh.Visible = 'off';
            dta = findall(obj.fig, 'Tooltipstring', 'Data Cursor');
            dta.Visible = 'off';
            rt = findall(obj.fig, 'Tooltipstring', 'Rotate 3D');
            rt.Visible = 'off';
            ed = findall(obj.fig, 'Tooltipstring', 'Edit Plot');
            ed.Visible = 'off';
            op = findall(obj.fig, 'Tooltipstring', 'Open File');
            op.Visible = 'off';
            nf = findall(obj.fig, 'Tooltipstring', 'New Figure');
            nf.Visible = 'off';
            pf = findall(obj.fig, 'Tooltipstring', 'Print Figure');
            pf.Visible = 'off';
            sf = findall(obj.fig, 'Tooltipstring', 'Save Figure');
            sf.Visible = 'off';
            hFileMenu = findall(obj.fig, 'tag', 'figMenuFile');
            hFileMenu.Visible = 'off';
            hFileMenu = findall(obj.fig, 'tag', 'figMenuEdit');
            hFileMenu.Visible = 'off';
            hFileMenu = findall(obj.fig, 'tag', 'figMenuView');
            hFileMenu.Visible = 'off';
            hFileMenu = findall(obj.fig, 'tag', 'figMenuInsert');
            hFileMenu.Visible = 'off';
            hFileMenu = findall(obj.fig, 'tag', 'figMenuTools');
            hFileMenu.Visible = 'off';
            hFileMenu = findall(obj.fig, 'tag', 'figMenuDesktop');
            hFileMenu.Visible = 'off';
            hFileMenu = findall(obj.fig, 'tag', 'figMenuWindow');
            hFileMenu.Visible = 'off';
            hFileMenu = findall(obj.fig, 'tag', 'figMenuHelp');
            hFileMenu.Visible = 'off';
        end
        
        
    end
    
end

