classdef TimeLineFcn < handle
    % Buttons and functions for experiment-timeline. Now optimized for
    % Luca.
    
    methods
        
        function correct_time(obj, src, event)
            try
                classes = arrayfun(@(x) {class(x{:})}, obj.time_objects);
                stack_ix = contains(classes, {'begonia.data_management.DataLocation', 'begonia.scantype.prairie.PrairieOutput'});
                trial_ix = contains(classes, 'yucca.trial.Trial');
                [obj.time_objects{stack_ix}, obj.time_objects{trial_ix}] = yucca.trial.correlate_stacks_alt(obj.time_objects{stack_ix}, obj.time_objects{trial_ix}, 'do_correct', true);
                xlimz = obj.ax.XLim;
                delete(findobj(obj.ax, 'Type', 'rectangle'));
                obj.add_rects;
                obj.ax.XLim = xlimz;
                
            catch
                error("Only works for trials and stacks")
            end
        end
        
        function keypress(obj, src, event)
            switch event.Key
                case 'backspace'
                    obj.delete_objects;
            end
        end
        
        
        function delete_objects(obj, src, event)
            active = findobj(obj.ax, 'Tag', 'Selected');
            if isempty(active)
                msgbox('No trials or stacks selected')
                return;
            end
            for i = 1:length(active)
                active(i).UserData.save_var('UseNot', 0)
                delete(active(i))
                delete(obj.text_field)
            end
        end
        
        function draw_buttons(obj, pos)
            if nargin < 2
                pos = [0.01, 0.01,0.99,0.1];
            end
            buttonpanel = uipanel('Title','','FontSize',12,...
                'Position',pos);
            
            xpos = linspace(0.02, 1, 14);
            xwidth = mean(diff(xpos))*0.9;
            e = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(1), 0.0, xwidth, 0.9], 'String', 'Open Linescan', 'Callback', @obj.timeline_open);
            e2 = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(2), 0.0, xwidth, 0.9], 'String', 'Mark Video', 'Callback', @obj.timeline_open);
            e3 = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(3), 0.0, xwidth, 0.9], 'String', 'Open Linescan Results', 'Callback', @obj.timeline_open);
            e4 = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(4), 0.0, xwidth, 0.9], 'String', 'Omit element', 'Callback', @obj.timeline_open);
            e5 = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(5), 0.0, xwidth, 0.9], 'String', 'Open TSeries', 'Callback', @obj.timeline_open);
            e6 = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(6), 0.0, xwidth, 0.9], 'String', 'Correct time', 'Callback', @obj.timeline_open);
            e7 = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(11), 0.0, xwidth, 0.9], 'String', 'Add note', 'Callback', @obj.timeline_open);
            e8 = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(8), 0.0, xwidth, 0.9], 'String', 'Roimanager (old)', 'Callback', @obj.timeline_open);
            e9 = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(9), 0.0, xwidth, 0.9], 'String', 'To workspace', 'Callback', @obj.timeline_open);
            e10 = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(10), 0.0, xwidth, 0.9], 'String', 'Open Ephys', 'Callback', @obj.timeline_open);
            e11 = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(7), 0.0, xwidth, 0.9], 'String',  'Correct time single', 'Callback', @obj.timeline_open);
            e12 = uicontrol(buttonpanel, 'Style', 'Pushbutton','Units', 'Normalized', 'Position', [xpos(12), 0.0, xwidth, 0.9], 'String',  'Open trial', 'Callback', @obj.timeline_open);
            e13 = uicontrol(buttonpanel, 'Style', 'Togglebutton','Units', 'Normalized', 'Position', [xpos(13), 0.0, xwidth, 0.9], 'String',  'Water intox time', 'Callback', @obj.timeline_open);
            
            
            
        end
        
        
        
        function timeline_open(obj, src, event)
            switch src.String
                case 'Water intox time'
                    all = findobj(obj.ax, 'Type', 'rectangle');
                    if src.Value == 1
                        delete(findobj(obj.ax, 'Tag', 'waterintox'));
                        a = electrophysiologyviewer.MoveLine(gcf, gca, (obj.ax.XLim(1)+range(obj.ax.XLim)/2), 'waterintox', false);
                    else
                        a = findobj(obj.ax, 'Tag', 'waterintox');
                        water_intox_time = datetime(a.XData(1),'ConvertFrom', 'datenum');
                        for i = 1:length(all)
                            all(i).UserData.save_var('water_intox_time');
                        end
                        
                    end
                    return;
            end
            active = findobj(obj.ax, 'Tag', 'Selected');
            if isempty(active)
                msgbox('No trials or stacks selected')
                return;
            end
            switch src.String
                case 'Add note'
                    if length(active) == 1
                        str = inputdlg('Add note to object', 'Notes', 1);
                        active.UserData.save_var('notes', char(str));
                    else
                        msgbox('You need to select only one object')
                    end
                    
                    
                case 'Open Linescan'
                    for i = 1:length(active)
                        if strcmp(class(active(i).UserData), 'begonia.scantype.prairie.LinescanPrairie')
                            linescananalysis.LineScanController(active(i).UserData)
                            %else
                            %   msgbox('No linescans selected')
                        end
                    end
                case 'Mark Video'
                    for i = 1:length(active)
                        if strcmp(class(active(i).UserData), 'yucca.trial.Trial')
                            yucca.mod.camera_regions.configure(active(i).UserData)
                            
                        end
                    end
                case 'Open Linescan Results'
                    for i = 1:length(active)
                        if strcmp(class(active(i).UserData), 'yucca.trial.Trial')
                            if any(contains(active(i).UserData.associated_stacks.saved_vars, 'Results'))
                                obj.open_res(active(i).UserData)
                            end
                        elseif strcmp(class(active(i).UserData), 'begonia.scantype.prairie.LinescanPrairie')
                            if any(contains(active(i).UserData.saved_vars, 'Results'))
                                obj.open_res(active(i).UserData)
                            end
                        end
                        
                    end
                case 'Omit element'
                    obj.delete_objects;
                case 'To workspace'
                    for i = 1:length(active)
                        try
                            assignin('base', matlab.lang.makeValidName(active(i).UserData.name), active(i).UserData);
                        catch
                            [~, nm] = fileparts(active(i).UserData.path);
                            nms = evalin('base','who');
                            assignin('base', matlab.lang.makeUniqueStrings(nm, nms), active(i).UserData);
                            
                        end
                    end
                    temp = who;
                case 'Open TSeries'
                    for i = 1:length(active)
                        if strcmp(class(active(i).UserData), 'begonia.scantype.prairie.TSeriesPrairie') || ...
                                strcmp(class(active(i).UserData), 'begonia.scantype.prairie.ZStackPrairie') || ...
                                strcmp(class(active(i).UserData), 'begonia.scantype.prairie.SingleImagePrairie');
                            g = roimanagerbeta.AnalyzeCSD(active(i).UserData);
                        end
                    end
                    
                case 'Roimanager (old)'
                    for i = 1:length(active)
                        if strcmp(class(active(i).UserData), 'begonia.scantype.prairie.TSeriesPrairie') || ...
                                strcmp(class(active(i).UserData), 'begonia.scantype.prairie.ZStackPrairie') || ...
                                strcmp(class(active(i).UserData), 'begonia.scantype.prairie.SingleImagePrairie');
                            g = roimanager_lite(active(i).UserData);
                            
                        end
                    end
                    
                case 'Correct time'
                    obj.correct_time();
                case 'Open trial'
                    trialobj = [];
                    
                    for i = 1:length(active)
                        trialobj = [trialobj, active(i).UserData];
                    end
                    lucaproject.TrialAnalyzer(trialobj)
                case 'Open Ephys'
                    for i = 1:length(active)
                        if strcmp(class(active(i).UserData), 'electrophysiologyviewer.Ephys')
                            
                            electrophysiologyviewer.EphysViewer(active(i).UserData);
                        end
                    end
                    
                case 'Correct time single'
                    for i = 1:length(active)
                        prompt = {'Years:','Months:', 'Days:','Hours:', 'Minutes:', 'Seconds:'};
                        title = 'Input';
                        dims = [1 30];
                        definput = {'0','0', '0','0','0', '0'};
                        answer = inputdlg(prompt,title,dims,definput);
                        time_correction = calendarDuration(str2num(answer{1}), str2num(answer{2}),str2num(answer{3}), str2num(answer{4}),str2num(answer{5}), str2num(answer{6}));
                        active(i).UserData.time_correction = time_correction;
                        active(i).UserData.save_var('time_correction');
                        
                    end
                    
                    %xlimz = obj.ax.XLim;
                    delete(findobj(obj.ax, 'Type', 'rectangle'));
                    obj.add_rects;
                    %obj.ax.XLim = xlimz;
                    
            end
        end
        
        function open_res(obj, dataobj)
            if isa(dataobj, 'yucca.trial.Trial') && any(contains(dataobj.saved_vars, 'Results'))
                Res = dataobj.load_var('Results');
                try
                    videodata = dataobj.load_var('camera_regions');
                    Res.add_video(videodata);
                catch
                    obj.write_video_vars(dataobj.associated_trials)
                    if contains(dataobj.saved_vars, 'camera_regions')
                        videodata = dataobj.load_var('camera_regions');
                        Res.add_video(videodata);
                    end
                end
                try
                    wheeldata = dataobj.load_var('wheel');
                    Res.add_wheel(wheeldata);
                end
                try
                    Res.add_descriptive(dataobj.associated_stacks);
                end
            elseif isa(dataobj, 'begonia.scantype.prairie.LinescanPrairie') && any(contains(dataobj.saved_vars, 'Results'))
                Res = dataobj.load_var('Results');
                if isprop(dataobj, 'associated_trials')
                    try
                        videodata = dataobj.associated_trials.load_var('camera_regions');
                        Res.add_video(videodata);
                    catch
                        obj.write_video_vars(dataobj.associated_trials)
                        if contains(dataobj.associated_trials.saved_vars, 'camera_regions')
                            videodata = dataobj.associated_trials.load_var('camera_regions');
                            Res.add_video(videodata);
                        end
                    end
                    try
                        wheeldata = dataobj.associated_trials.load_var('wheel');
                        Res.add_wheel(wheeldata);
                    end
                end
                try
                    Res.add_descriptive(dataobj);
                end
                
                
            else
                msgbox('No linescan results saved...')
                return;
            end
            
            Res.show_results_new(dataobj);
        end
        
        function write_video_vars(obj, trials)
            yucca.mod.camera_regions.write_vars(trials);
            
        end
    end
end

