function [fig] = plot_vdist(tseries, use_z_score)

    subp_count = 4;

    % option to show z-score:
    if nargin < 2
        use_z_score = true;
        
    end

    
    if ~tseries.has_var('vestool_results')
        error('Not results parameter given, and tseries is not abalysed by vessel tool (lacks "vestool_results" var)');
    end
    results = tseries.load_var('vestool_results');

    % figure:
    fig = figure();
    fig.Position(4) = fig.Position(4) * 2;
    fig.Name = ['Vessel dynamics ' tseries.name];
    
    dms = results.diam_min_mean;
    dms_lrd = results.diam_left_right_dynamic_mean;
    dms_l = results.diam_locked_mean;
    dms_hp = results.diam_half_peak;
    
    vframes = results.frame_analysed; % valid frames
    t = results.time_s; % time in seconds of emasurement
    
    %% MINIMUM DYNAMIC MEANS diameter plots:
%     subplot(subp_count,1,1);
%     plot_vals(dms, t, vframes, results.maker_names, tseries.dx, use_z_score);
%     title('Vessel diameter (minimum dynamic min + (max-min)/2)');
    
    %% LEFT-RIGTH-DYNAMIC  mean diameter plots:
    subplot(subp_count,1,1);
    leg = plot_vals(dms_lrd, t, vframes, results.maker_names, tseries.dx, use_z_score);
    title('Vessel diameter (bilateral dynamic min + (max-min)/2))');
    
    %% locked mean diameter plots:
    subplot(subp_count,1,2);
    plot_vals(dms_l, t, vframes, results.maker_names, tseries.dx, use_z_score);
    title('Vessel diameter (locked f_0 means)');
    
    %% half peak intercepts:
%     subplot(subp_count,1,4);
%     leg = plot_vals(dms_hp, t, vframes, results.maker_names, tseries.dx, use_z_score);
%     title('Vessel diameter (half peak)');
    
    %% plot area under mean treshold:
    subplot(subp_count,1,3);
    for i = 1:size(dms,2)
        vals = results.area_under_min_treshold(:,i);
        vals_t = t(vframes);
        vals = vals(vframes);
        vals_z = zscore(vals);
        
        plot(vals_t, smooth(vals_z,10) * tseries.dx, 'LineWidth',1.25);
        hold on;
    end
    title('Area under min dynamic mean');
    ylabel('Pixels (z-score)');
    xlabel('Time (s)');
    legend(leg);
    grid on;
    
    
    %% plot mean channel fluorescense
    subplot(subp_count,1,4);
    plot(t(vframes), smooth(results.channel_mean_fluo(vframes)))
    title('Mean channel fluoresence');
    ylabel('Fluorescence (A.U.)');
    xlabel('Time (s)');
    legend([tseries.name '(analysis channel)']);
    grid on;
    
end

function leg = plot_vals(vals_all, t, valid, maker_names, dx ,use_z_score)
    leg = {};
    for i = 1:size(vals_all,2)
        vals = vals_all(:,i);
        
        vals_t = t(valid);
        vals = vals(valid);
        
        if use_z_score
            vals_z = zscore(vals);
        else
            vals_z = vals;
        end
        
        plot(vals_t, smooth(vals_z,10) * dx, 'LineWidth',1.25);
        hold on;
        leg(i) = { char(maker_names(i)) };
    end
    
        
    if use_z_score
        ylabel('Diameter (z-score)');
    else
        ylabel('Diameter (um)');
    end
    
    xlabel('Time (s)');
    legend(leg);
    grid on;
end


