function video_path = record_analysis(stack, frames, outdir)
    import xylobium.vesseltool.vdist.*;
    recsize = 150;
    
    config = stack.load_var('vestool_config');
    sp_count = length(config.markers);
    
    % set up video writer:
    video_path = fullfile(outdir, [stack.name '.avi']);
    vwriter = VideoWriter(video_path);
    open(vwriter);
    
    fig_hidden = figure('Visible', 'off');
    plot(1);
    ax_hidden = fig_hidden.CurrentAxes;
    
    fig = figure();
    fig.Position(3) = round((recsize + 40) * sp_count);
    fig.Position(4) = round(recsize * 6);
    subplot(1, sp_count, 1);
    plot(1);
    
    ax = fig.CurrentAxes;
    
    mat = stack.get_mat(config.cycle, config.channel);
    diams = nan(length(frames), length(config.markers));
    diams_l = nan(length(frames), length(config.markers));
    auts = nan(length(frames), length(config.markers));
    
    % get the linescan plots max and min in first frame, the lock at that:
    lscan_locked_ylim = [];
    
    % perform analysis pr. frame:
    i = 0;
    for fnr = frames
        i = i + 1;
        avg_grab = round(config.average/2);
        fs = fnr-avg_grab:fnr+avg_grab; % frames we'll average:

        % trim frames to ensure we dont exceed matrix bounds, then
        % retrieve and average:
        fs = fs(fs > 0 & fs < stack.frames_in_cycle);
        mat_fs = mat(:,:,fs);
        frame = mean(mat_fs,3);
        
        % for a marker on each row:
        for m_idx = 1:length(config.markers)
            m = config.markers(m_idx);
            
            % plot frame imagesc
            subplot(6, sp_count, m_idx);
            imagesc(frame);
            line(round([m.start_point(1) m.end_point(1)])...
                , round([m.start_point(2) m.end_point(2)]) ...
                , 'Color', 'w' ...
                , 'LineWidth', 2);
            title([num2str(round(fnr * stack.dt)) 's #' num2str(fnr)]);
            
            % first row: rotation plot:
            subplot(6, sp_count, m_idx + sp_count);
            try
            m = config.markers(m_idx);
                [diam, diam_lrd, diam_l, diam_hp, aut, chmean, lscan, means, mat_rot, mat_aut, peaks, peak_interceps] = analyse_frame( ...
                    frame ...
                    ,  [m.start_point(1) m.end_point(1)] ...
                    , [m.start_point(2) m.end_point(2)] ...
                    , ax_hidden ...
                    , 3 ...
                    , m.locked_means);
            catch error
                % occationally we crash, and need to move on:
                title('Error in frame capture');
                continue;
            end
                
            % remember limits of the lscan if this is the first:
            if isempty(lscan_locked_ylim)
                lscan_locked_ylim = [min(lscan)-250, max(lscan)+250];
            end
                
            imagesc(mat_rot);
            title(m.name);
            
            %% plot lscan and mean lines:
            subplot(6, sp_count, m_idx + (sp_count * 2));
            plot(lscan);
            ylim(lscan_locked_ylim);
            hold on;
            xlim([0 length(lscan)]);
            xlims = xlim();
            
            line([0, xlims(2)], [mean(m.locked_means) mean(m.locked_means)], 'Color', 'g');
%             line([0, xlims(2)], [min(means) min(means)], 'Color', [0.8,0.8,0.8], 'LineStyle','--');
            
            % left mean:
            line([0, round(xlims(2)/2)], [means(1) means(1)], 'Color', 'b');

            % right mean:
            line([round(xlims(2)/2), xlims(2)], [means(2) means(2)], 'Color', 'b');

%             if ~isnan(peaks(1))
%                 plot(peak_interceps,[lscan(peak_interceps(1)) lscan(peak_interceps(2))],'r*');
%             end
            
            title('g = locked, b = dynamic');
            hold off
            
            %% keep the dist measurements, and plot the trace over itme:
            subplot(6, sp_count, m_idx + (sp_count  * 3));
            diams_l(i, m_idx) = diam_l * stack.dx;
            diams_lrd(i, m_idx) = diam_lrd * stack.dx;
            
            auts(i,m_idx) = aut * stack.dx;
            
            plot(diams_l(:, m_idx), 'Color', [0.8,0.8,0.8]);
            hold on;
            plot(diams_lrd(:, m_idx), 'Color',[0.8,0.8,0.8]);
            plot(smooth(diams_l(1:i, m_idx),10), 'LineWidth', 1, 'Color', 'g');
            plot(smooth(diams_lrd(1:i, m_idx),10), 'LineWidth', 1, 'Color', 'b');
            
            try
            title(['ULM: ' num2str(round(diam_lrd  * stack.dx)) ' um LM:' ...
                num2str(round(diam_l  * stack.dx)) ' um']);
            catch err
                title('ULM: ? LM: ?')
            end
            hold off;
            
            %% plot AUT mask:
            subplot(6, sp_count, m_idx + (sp_count  * 4));
            imagesc(mat_aut);
            title('AUT mask');
            
            %% plot AUT:
            subplot(6, sp_count, m_idx + (sp_count  * 5));
            plot(auts(:, m_idx), 'Color',[0.8,0.8,0.8]);
            hold on;
            plot(smooth(auts(1:i, m_idx),10), 'LineWidth', 1, 'Color', 'b');
            hold off;
        end
        
        vwriter.writeVideo(getframe(fig));
    end
    
    % clean up:
    close(vwriter);
    delete(fig_hidden);
    delete(fig)
end







