classdef TrialManager < handle
    
    properties
        picker
        trials
    end
    
    methods
        function self = TrialManager()
            self.picker = xylobium.picker.Picker();
        end
        
        function open_gui(self)
            self.picker.run_gui(self.trials);
        end
        
        function trials = get_trials(self,show_gui)
            if nargin < 2
                show_gui = false;
            end

            if show_gui
                I = self.picker.run_gui(self.trials);
            else
                I = self.picker.run(self.trials);
            end

            if isempty(self.picker.function_handles)
                I = 1:length(self.trials);
            end
            
            trials = self.trials(I);
        end
        
        function add_filter_function(self,func_or_varname)
            if ischar(func_or_varname)
                var_name = func_or_varname;
                func = @(trials) xylobium.picker.default_filter(trials,var_name);
                self.picker.add_function(func,var_name);
            else
                func = func_or_varname;
                self.picker.add_function(func);
            end
        end
        
        function print_filters(self)
            self.picker.summary();
        end
        
        function reset_filters(self)
            self.picker.reset();
        end
        
        function set_filter(self,key,value)
            self.picker.set_column_filter(key,value);
        end
        
        function clear_cache(self)
            self.picker.clear_cache();
        end
    end
end

