classdef DisplayAction < calendula.actions.Action
    %DISPLAYACTION Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        mode
        mouse_clicked
        mouse_coordinates
        image_marked
    end
    
    properties (Access = private)
        prev_action
    end
    
    methods
        function self = DisplayAction(editor)
            self@calendula.actions.Action(editor);
            
            self.mode = editor.mode;
            self.image_marked = editor.image_marked;
            
            self.prev_action = editor.get_current_action();
        end
        
        function do(self)
            self.editor.mode = self.mode;
            self.editor.image_marked = self.image_marked;
            switch self.mode
                case 'mark'
                    if ~isempty(self.mouse_coordinates)
                        self.editor.show_brush(self.mouse_coordinates);
                    end
                    
                    if self.mouse_clicked
                        self.editor.click_at_coord(self.mouse_coordinates);
                    end
                    self.editor.update_marked_plot();
                case 'select'
                    self.editor.clear_marked();
            end
            
            if isempty(self.prev_action)
                return
            end
            
            if ~strcmp(self.prev_action.mode,self.mode)
                self.editor.notify('OnModeChanged');
            end
        end
        
        function undo(self)
            self.prev_action.do();
        end
        
        
    end
    
end

