classdef EditorDisplay < calendula.editors.Memento
    %EDITORGUI Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        axes
        
        image_handle
        
        brush_radius = 10;
        
        image_marked
        image_marked_alpha = 0.3
        image_marked_color = uint8([255,105,180])
        mode = 'none'
    end
    
    properties (Access = private)
        image_marked_handle
        brush_handle
    end
    
    events
        OnModeChanged
    end
    
    methods
        function self = EditorDisplay(axes,image_handle)
            self.axes = axes;
            self.image_handle = image_handle;
            self.image_marked = zeros(size(image_handle.CData),'logical');
        end
        
        function clear_marked(self)
            if ~isempty(self.image_marked_handle)
                delete(self.image_marked_handle)
                self.image_marked_handle = [];
                self.image_marked(:) = false;
            end
        end
        
        function show_brush(self,xy)
            th = 0:pi/50:2*pi;
            xdata = self.brush_radius * cos(th) + xy(1);
            ydata = self.brush_radius * sin(th) + xy(2);
            
            if isempty(self.brush_handle)
                hold(self.axes,'on')
                self.brush_handle = plot(self.axes,xdata,ydata,'yellow');
                hold(self.axes,'off');
                self.brush_handle.HitTest = 'off';
                self.brush_handle.PickableParts = 'none';
            else
                self.brush_handle.XData = xdata;
                self.brush_handle.YData = ydata;
            end
        end
        
        function click_at_coord(self,xy)
            xy = round(xy);
            r = round(self.brush_radius);
            side = round(r);
            [i,j] = meshgrid(-side:side,-side:side);
            mat = i.*i + j.*j <= r*r;
            x1 = xy(1) - r;
            x2 = xy(1) + r;
            y1 = xy(2) - r;
            y2 = xy(2) + r;
            self.image_marked(y1:y2,x1:x2) = mat | self.image_marked(y1:y2,x1:x2);
        end
        
        function update_marked_plot(self)
            if isempty(self.image_marked_handle)
                solid_color_image = ones([size(self.image_marked),3],'uint8');
                solid_color_image(:,:,1) = solid_color_image(:,:,1)*self.image_marked_color(1);
                solid_color_image(:,:,2) = solid_color_image(:,:,2)*self.image_marked_color(2);
                solid_color_image(:,:,3) = solid_color_image(:,:,3)*self.image_marked_color(3);
                
                axes(self.axes);
                hold on
                self.image_marked_handle = imshow(solid_color_image);
                self.image_marked_handle.HitTest = 'off';
                self.image_marked_handle.PickableParts = 'none';
                hold off
            end
            self.image_marked_handle.AlphaData = self.image_marked*self.image_marked_alpha;
        end
    end
    
end

