classdef signal_inspect < handle
    %SIGNAL_INSPECT Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        fig
        stack
        axes_traces
        gui_roi_list
        signal
        signal_variable_name
    end
    
    methods
        function self = signal_inspect(varargin)
            p = inputParser;
            p.addOptional('stack',[], ...
                @(x) validateattributes(x,{'begonia.stack.Stack'},{}));
            p.addOptional('signal_variable_name','ca_signal', ...
                @(x) validateattributes(x,{'char'},{}));
            p.parse(varargin{:});
            stack = p.Results.stack;
            signal_variable_name = p.Results.signal_variable_name;
            %% UI setup
            
            fig = figure;
            fig.Units = 'centimeter';
            fig.Position(1:2) = [5,5];
            fig.Name = 'Signal Inspector';
            fig.NumberTitle = 'off';
            self.fig = fig;
            
            gui_roi_list = calendula.gui.roi_list(fig);
            self.gui_roi_list = gui_roi_list;
            
            panel = gui_roi_list.panel;
            
            axes_traces = axes;
            axes_traces.Parent = fig;
            axes_traces.Units = 'centimeter';
            axes_traces.Position(1) = panel.Position(1) + panel.Position(3) + 1;
            axes_traces.Position(2) = panel.Position(1);
            axes_traces.Position(3) = 20;
            axes_traces.Position(4) = panel.Position(4);
            self.axes_traces = axes_traces;
            
            begonia.gui.autosize_container(fig,0.5);
            set(findall(fig, '-property', 'Units' ), 'Units', 'Normalized');
            
            %% Callbacks
            gui_roi_list.listbox_rois.addlistener('Value','PostSet',@(s,e) self.plot_traces());
            gui_roi_list.addlistener('RoiArrayChanged',@(s,e) self.onRoiArrayChanged(e));
            gui_roi_list.addlistener('RoiArraySaved',@(s,e) self.onRoiArraySaved());
            
            %% Load data
            if ~isempty(stack)
                self.load_stack(stack,signal_variable_name);
            end
        end
        
        function load_stack(self,stack,signal_variable_name)
            signal = stack.load_var(signal_variable_name);
            self.fig.Name = signal.Name;
            self.signal = signal;
            self.signal_variable_name = signal_variable_name;
            self.stack = stack;
            self.gui_roi_list.load(stack);
        end
        
    end
    
    
    methods (Access = private)
        
        function onRoiArraySaved(self)
            self.stack.save_var(self.signal_variable_name,self.signal);
            fprintf('Signal saved.\n');
        end
        
        function onRoiArrayChanged(self,event)
            data = self.signal.Data;
            data(:,event.removed_roi_idx) = [];
            self.signal.Data = data;
            self.plot_traces();
        end
        
        
        function plot_traces(self)
            listbox = self.gui_roi_list.listbox_rois;
            
            t = seconds(self.signal.Time);
            
            axes(self.axes_traces);
            cla(self.axes_traces);
            if isempty(listbox.Value)
                x = zeros(1,length(t));
                plot(t,x);
                return
            end
            hold on
            for i = self.gui_roi_list.listbox_rois.Value
                x = self.signal.Data(:,i);
                offset = i;
                plot(t,x + offset);
                text('Parent',self.axes_traces, ...
                    'String',self.gui_roi_list.roi_tags{i},...
                    'Position',[0,offset - 0.3]);
            end
            hold off
        end
    end
    
end

