classdef roi_list < handle
    %ROI_LIST Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        panel
        listbox_rois
        pushbutton_remove_rois
        pushbutton_save_rois
        stack
        roi_array
        roi_tags
        
        button_panel
    end
    
    events
        % Event spesific data:
        %   none
        RoiArrayChanged 
        % Event spesific data:
        %   removed_roi_idx     - Index of the removed roi.
        RoiArraySaved
    end
    
    methods
        function self = roi_list(varargin)
            p = inputParser;
            p.addOptional('fig',[], ...
                @(x) validateattributes(x,{'matlab.ui.Figure'},{}));
            p.parse(varargin{:});
            fig = p.Results.fig;
            %% figure setup
            if isempty(fig)
                create_figure = true;
            else 
                create_figure = false;
            end
            
            if create_figure
                fig = figure;
                fig.Name = 'ROI List';
                fig.NumberTitle = 'off';
                fig.MenuBar = 'none';
                fig.Units = 'centimeter';
            end
            
            %% panel
            panel = uipanel;
            panel.Parent = fig;
            panel.Units = 'centimeter';
            panel.Position(1:2) = [5,1];
            panel.Position(3:4) = [4,15];
            self.panel = panel;
            
            button_panel = begonia.gui_tools.ButtonPanel(panel);
            self.button_panel = button_panel;
            
            if create_figure
                fig.Position = [5,5,panel.Position(3)+2,panel.Position(4)+2];
            end
            
            %% gui components
            listbox_rois = uicontrol('Style','listbox');
            listbox_rois.Parent = fig;
            listbox_rois.Units = 'centimeters';
            listbox_rois.Position(1:2) = [1,1];
            listbox_rois.Position(3:4) = [3,15];
            listbox_rois.Max = 999999;
            listbox_rois.Min = 0;
            self.listbox_rois = listbox_rois;
            
            pushbutton_remove_rois = uicontrol('Style','pushbutton');
            pushbutton_remove_rois.String = 'Remove ROI';
            pushbutton_remove_rois.Callback = @(s,e) self.remove_roi_by_idx(self.listbox_rois.Value);
            button_panel.add_graphics_object(pushbutton_remove_rois);
            self.pushbutton_remove_rois = pushbutton_remove_rois;
            
            pushbutton_save_rois = uicontrol('Style','pushbutton');
            pushbutton_save_rois.String = 'Save ROIs';
            pushbutton_save_rois.Callback = @(s,e) self.save_rois();
            button_panel.add_graphics_object(pushbutton_save_rois);
            self.pushbutton_save_rois = pushbutton_save_rois;
            
            %% Size adjustment
            if create_figure
                begonia.gui_tools.autosize_container(fig,0.5);
                set(findall(fig, '-property', 'Units' ), 'Units', 'Normalized')
            end
            
            
        end
        
        
        function load(self,stack)
            self.stack = stack;
            
            roi_array = stack.load_var('roi_array',calendula.RoI.empty);
            roi_array = calendula.RoI(roi_array);
            self.roi_array = roi_array;
            
            self.populate_listbox();
        end
        
        
        function save_rois(self)
            if isempty(self.stack)
                return
            end
            
            roi_array = self.roi_array.to_struct();
            self.stack.save_var(roi_array);
            fprintf('ROIs saved.\n');
            self.notify('RoiArraySaved');
        end
        
        
        function remove_roi_by_idx(self,idx)
            if isempty(self.roi_array)
                return;
            end
            self.roi_array(idx) = [];
            
            event = begonia.gui_tools.UniversalEvent;
            event.addprop('removed_roi_idx');
            event.removed_roi_idx = idx;
            % The onRoiArrayChanged was initially added as a listener but
            % this function should be run before other events as the gui
            % elements should get updated before other listeneres will
            % handle the event.
            self.onRoiArrayChanged(event);
            self.notify('RoiArrayChanged',event);
        end
        
    end
    
    methods (Access = private)
        
        function onRoiArraySaved(self)
        end
        
        function populate_listbox(self)
            if isempty(self.roi_array)
                self.roi_tags = {};
                self.listbox_rois.String = {};
                return
            end
            % Create incrementing numbers for each group.
            groups = {self.roi_array.group};
            nums = zeros(1,length(groups));
            [C,IA,IC] = unique(groups);
            for i = 1:length(C)
                ind = IC == i;
                nums(ind) = 1:sum(ind);
            end
            
            % Create tags based on group and number.
            roi_tags = {};
            for i = 1:length(self.roi_array)
                tag = sprintf([self.roi_array(i).group,'%02d'],nums(i));
                roi_tags{i} = tag;
            end
            
            % Add the tag of a connected roi
            for i = 1:length(self.roi_array)
                roi = self.roi_array(i);
                if isempty(roi.id_connected_roi); continue; end;
                % Find the connected roi.
                id = roi.id_connected_roi;
                ids = {self.roi_array.id};
                match = strcmp(id,ids);
                if sum(match) ~= 1; continue; end;
                connected_tag = roi_tags{match};
                roi_tags{i} = [roi_tags{i},'_',connected_tag];
            end
            
            self.roi_tags = roi_tags;
            self.listbox_rois.String = roi_tags;
        end
        
        function onRoiArrayChanged(self,event)
            self.listbox_rois.Value = [];
            if isempty(self.roi_array)
                self.roi_tags = {};
                self.listbox_rois.String = {};
            else
                self.roi_tags(event.removed_roi_idx) = [];
                self.listbox_rois.String(event.removed_roi_idx) = [];
            end
            
        end
    end
    
end

