classdef stack_display < handle
    %STACK_DISPLAY Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        % GUI handles
        panel
        axes_display
        image_display
        slider_frames
        
    end
    
    properties (Access = private)
        roi_array
        roi_array_plot_handles
    end
    
    methods
        function self = stack_display(varargin)
            p = inputParser;
            p.addOptional('fig',[], ...
                @(x) validateattributes(x,{'matlab.ui.Figure'},{}));
            p.parse(varargin{:});
            fig = p.Results.fig;
            %% Figure setup
            if isempty(fig)
                create_figure = true;
            else 
                create_figure = false;
            end
            
            if create_figure
                fig = figure;
                fig.Name = 'Stack Display';
                fig.NumberTitle = 'off';
                fig.Units = 'centimeter';
            end
            
            %% Panel
            panel = uipanel;
            panel.Parent = fig;
            panel.Units = 'centimeters';
            self.panel = panel;
            
            %% UI components inside the panel
            axes_display = axes();
            axes_display.Parent = panel;
            axes_display.Units = 'centimeters';
            axes_display.Position(1:2) = [1,1];
            axes_display.Position(3:4) = [13,13];
            colormap(axes_display,begonia.colormaps.viridis);
            self.axes_display = axes_display;
            
            
            image_display = imshow([], [0, 1]);
            image_display.Parent = axes_display;
            self.image_display = image_display;
            
            
            slider_frames = uicontrol(panel,'Style','slider');
            slider_frames.Units = 'centimeters';
            slider_frames.Position(1:2) = axes_display.Position(1:2) - [0,1];
            slider_frames.Position(3) = axes_display.Position(3);
            slider_frames.Position(4) = 0.5;
            slider_frames.Callback = @(s,e) self.callback_slider_frames();
            slider_frames.Min = 1;
            slider_frames.Max = 10;
            slider_frames.Value = 1;
            fig.WindowScrollWheelFcn = @(s,e) self.callback_scroll(e);
            self.slider_frames = slider_frames;
            
            
            
            %% Panel and figure size adjustment.
            begonia.gui_tools.autosize_container(panel,0.5);
            if create_figure
                begonia.gui_tools.autosize_container(fig,0.5);
                fig.Position(1:2) = [5,5];
                % Enables rescaling.
                set(findall(fig, '-property', 'Units' ), 'Units', 'Normalized')
            end
        end
        
        function plot_rois(self,rois)
            if isempty(rois)
                return
            end
            
            if isstruct(rois)
                rois = calendula.RoI(rois);
            end
            
            if isempty(self.image_display.CData)
                self.image_display.CData = zeros(size(rois(1).mask));
            end
            
            %% Remove plots from old rois. 
            h = self.roi_array_plot_handles;
            del = false(size(h));
            for i = 1:length(h)
                if any(strcmp(h(i).Tag,{rois.id}))
                    del(i) = true;
                end
            end
            delete(h(del));
            h(del) = [];
            self.roi_array_plot_handles = h;
            
            %% Draw rois
            h = [];
            hold on
            for i = 1:length(rois)
                roi = rois(i);
                % Add each boundary to the plot
                for j = 1:length(roi.boundary)
                    boundary = roi.boundary{j};
                    if j == 1
                        h(i) = plot(boundary(:,2), boundary(:,1));
%                         h(i).Color = roiColor;
                    else
                        h(i).XData = horzcat(h(i).XData, nan, boundary(:,2).');
                        h(i).YData = horzcat(h(i).YData, nan, boundary(:,1).');
                    end
                end
            end
            hold off
            
            self.roi_array = union(self.roi_array,rois);
        end
        
    end
    
    methods (Access = private)
        function callback_slider_frames(self)
        end
        
        
        function callback_scroll(self,event)
            val = round(self.slider_frames.Value);
            val_max = round(self.slider_frames.Max);
            val_min = round(self.slider_frames.Min);
            val_add = max(1,round(event.VerticalScrollCount));
            val_add = sign(event.VerticalScrollCount)*val_add;
            val = val + val_add;
            val = min(val,val_max);
            val = max(val,val_min);
            self.slider_frames.Value = val;
        end
    end
    
end

