function ca_signal = read(stack)

global BEGONIA_VERBOSE;


roi_array = stack.load_var('roi_array');
roi_array = calendula.RoI(roi_array);

% Assume all cycles have equally many frames.
frames = stack.frames_in_cycle(1);
signal = NaN(frames, length(roi_array));

if BEGONIA_VERBOSE >= 1
    fprintf('Extracting roi signals: %s\n',stack.path);
end

for cycle = 1:stack.cycles
    for channel = 1:stack.channels
        % Get the rois of the correct cycle and channel.
        idx_cycle = [roi_array.cycle] == cycle;
        idx_channel = [roi_array.channel] == channel;
        roi_indices = find(idx_cycle & idx_channel);

        % Get the matrix.
        mat = stack.get_mat(cycle,channel);
        mat = mat(:,:,:);

        cnt = 1;
        for roi_idx = roi_indices
            if BEGONIA_VERBOSE >= 2
                fprintf('RoI #%d\n',cnt);
                cnt = cnt + 1;
            end
            vec = roi_array(roi_idx).extract_signal(mat);
            signal(:,roi_idx) = vec;
        end
    end
end
signal_t = 0:(frames-1);
signal_t = signal_t * stack.dt;
signal_t = signal_t';
ca_signal = timeseries(signal,signal_t,'Name','ca_signal');

assert(~isnan(ca_signal.TimeInfo.Increment),'why:is:this:nan','What the hell');

end

