function trace = read(stack)
global BEGONIA_VERBOSE;

assert(stack.channels == 2,'calendula:missing_neuron_channel','Stack does not have exatcly 2 channels.');

mat = stack.get_stack(1,2);

roi_array = stack.load_var('roi_array');
roi_array = calendula.RoI(roi_array);

% Pick out the neuron rois.
neurons = [roi_array.channel] == 2;
neurons = roi_array(neurons);

if isempty(neurons)
    trace = [];
    return
end
% assert(~isempty(neurons),'calendula:missing_neuron_rois','No neuron type rois found.')

% Get the mask of all the rois and create a mask.
super_roi = neurons.merge();
% Invert the mask to "choose" the area outside the soma. 
mask = ~super_roi.mask;
% Make the mask flat.
mask = reshape(mask,1,[]);

function trace_part = op_func(mat_part)
    mat_part = single(mat_part);
    
    axis = 3;
    dim = size(mat_part);
    
    % Shift the matrix such that that time is the first axis (along the colums).
    mat_part = shiftdim(mat_part,axis-1);
    % Reshape all the other dimensions to a 2d matrix, with time along the
    % columns as before. 
    mat_part = reshape(mat_part,dim(axis),[]);
    % Multiply the 2d matrix with the mask, ignoring the columns (pixels)
    % that are in the mask.
    mat_part = mat_part.*mask;
    
    trace_part = sum(mat_part,2)/sum(mask);
end

function trace = collection_func(traces)
    trace = [];
    for i = 1:length(traces)
        trace = cat(1,trace,traces{i});
    end
end

c = begonia.util.Chunker(mat,'chunk_size_mb',1000);
tic
trace = c.do_operation(@op_func,@collection_func,true);
if BEGONIA_VERBOSE >= 2
    fprintf('Trial: %s\n',stack.path);
    toc
end

signal_t = 0:(length(trace)-1);
signal_t = signal_t * stack.dt;
signal_t = signal_t';

trace = timeseries(trace,signal_t,'Name','Neuron no soma trace');

assert(~isnan(trace.TimeInfo.Increment),'why:is:this:nan','What the hell');
end